        .LI     OFF
        .LF     pm6668.lst
;---------------------------------------------------------------------------
;
;  PM6668.ASM
;
;  Original program for the PM6668 high resolution frequency
;  counter. The program runs on a 8048 micro-processor and is
;  restricted to the 1k byte of available ROM.
;
;  Disassembled by San Bergmans Oisterwijk, May 1994
;
;---------------------------------------------------------------------------

        .CR     8048
        .OR     $000
        .TF     pm6668.int,INT,16

;---------------------------------------------------------------------------
;  Constants
;---------------------------------------------------------------------------

WAIT1ST .EQ     -3846   x 90s  Waiting time for 1st transition
FAST    .EQ     -1958   x102s  Waiting time for fast rate
SLOW    .EQ     -9176   x108s  Waiting time for slow rate
WAITLST .EQ     -15872  x 90s  Waiting time for last transition

;---------------------------------------------------------------------------
;  Memory declaration
;---------------------------------------------------------------------------

R2RB1   .EQ     $1A             Register 2 of RB1 (flagbyte)
;                                b0=0: display change allowed
;                                b0=1: don't change display
;                                b4=0: max fast display is 2199999
;                                b4=1: max fast display is 1999999
EVENT   .EQ     $20,$25         Event counter evaluation     (BCD)
REFCNTR .EQ     $26,$2B         Reference counter evaluation (BCD)
DISPBUF .EQ     $2C,$33         Display buffer
;                                ann,digit7,....,digit1 (1=left)
;                                ann.3 = LOW BAT
;                                ann.2 = M
;                                ann.1 = Hz
;                                ann.0 = kHz
RATE    .EQ     $34             0 if fast mode, 1 if slow mode

;---------------------------------------------------------------------------
;  ROUTINE: RESET         ($000)
;  Get the thing started
;---------------------------------------------------------------------------

RESET   INS     A,BUS           Ensure BUS is used as inputs
        CLR     A
        ANL     P2,#%1111.0011  Disable both decade counter outputs
        JMP     INIT             a.s.a.p. (they're both active
        NOP                                 during reset!)

;---------------------------------------------------------------------------
;  ROUTINE: TIMER         ($007)
;  Timer interrupt request
;  Extends the range of the internal hardware counter.
;  Used, in conjunction with the external counter IC106, to count
;  the reference pulses comming from the 10MHz clock.
;  This softwarepart consists of 4 BCD digits.
;---------------------------------------------------------------------------

TIMER   SEL     RB1             Use registerbank RB1 during
        MOV     R1,A             interrupt, and save Accu
        MOV     A,#1            Increment 4 MSD of reference counter
        MOV     R0,#REFCNTR      in software.
        ADD     A,@R0
        DA      A               (BCD counter!)
        MOV     @R0,A
        CLR     A
        INC     R0
        ADDC    A,@R0
        DA      A
        MOV     @R0,A
        MOV     A,R1            Restore Accu
        SEL     RB0             Not needed, would have been restored
        RETR                     by RETR

;---------------------------------------------------------------------------
;  ROUTINE: ROMTEST       ($018)
;  Calculate CRC of ROM page 0
;---------------------------------------------------------------------------

ROMTEST CLR     A               Clear
        MOV     R0,A             pointer in ROM page
        MOV     R1,A             and both CRC bytes
        MOV     R2,A
.LOOP   MOV     A,R0            Get ROM byte
        MOVP    A,@A
        CALL    CRC             Calculate CRC from it
        DJNZ    R0,.LOOP        Do all 256 bytes!
        JMP     ROMTST1         Continue for ROM page 1

;---------------------------------------------------------------------------
;  ROUTINE: RETRY.Z       ($024)
;  Rate changed, clear hysteresis flag and try again
;---------------------------------------------------------------------------

RETRY.Z SEL     RB1             Clear both display lock and
        MOV     A,#%1110.1110    hysteresis flag
        ANL     A,R2            (could have been done with
        MOV     R2,A             MOV R2,#%1110.1110 too)
        SEL     RB0

;---------------------------------------------------------------------------
;  ROUTINE: RETRY         ($02A)
;  Last measurement failed or was aborted, retry new measurement
;---------------------------------------------------------------------------

RETRY   ORL     P2,#%0010.0000  Stop measurement no matter what
        MOV     R0,#DISPBUF+1   Clear the display buffer
        MOV     R4,#8            (one byte to many done!)
        CALL    CLEAR
        MOV     R1,#DISPBUF     Clear the annunciators
        MOV     @R1,#%1111.1111
        ANL     P1,#%1111.1011   (Why EN1 is made low is a mystery)
        CALL    SHOWDSP         Show empty display

;---------------------------------------------------------------------------
;  ROUTINE: AGAIN         ($03A)
;  Last measurement is on the display now, start new one
;---------------------------------------------------------------------------

AGAIN   INS     A,BUS           Enshure BUS is inputs
        NOP
        MOV     R3,#0           In case LF input is used
        ANL     P1,#%1111.0000  Disable display controllers
        ANL     P2,#%1110.1111  Reset Flip-flop of IC110
        ORL     P1,#%0100.0000  Is HF input used?
        IN      A,P1
        JB6     .LF             No! Use LF input then
        MOV     R3,#8           Select HF input (2^8=256 presc)
.LF     ORL     P2,#%1000.0000  Activate Reset
        ANL     P2,#%1011.1111   and -Reset
        MOV     R0,#EVENT       Clear entire buffer area
        MOV     R4,#6+6+8        from event counter, to reference
        CALL    CLEAR             counter, to display buffer
        ORL     P1,#%1000.0000  Select LF input
        MOV     A,#8            Must LF input be selected?
        XRL     A,R3
        JB3     .SELLF          Yes!
        ANL     P1,#%0111.1111  Deselect LF input
.SELLF  CLR     A               Clear internal counter
        MOV     T,A              and start it running and
        STRT    CNT              allow interrupts at every
        EN      TCNTI            $FF to $00 transition
        ANL     P2,#%0111.1111  Release Reset
        ORL     P2,#%0101.0000  Release -Reset and IC110
        ANL     P2,#%1101.1111  Start measurement
        MOV     R4,/WAIT1ST     Setup software measurement time
        MOV     R5,#WAIT1ST      (350ms)
.WAIT   CALL    CHCK.HF         Abort if HF status changed
        JNI     .NOPULS         No transition on input sensed yet!
        JMP     MEASURE         Transition sensed! We're started
.NOPULS CALL    DELAY           Wait a while for 1st transition
        JNZ     .WAIT           Time not up yet!
        JMP     RETRY           Nothing happened! Show 0 & try again

;---------------------------------------------------------------------------
;  ROUTINE: MEASURE       ($076)
;  1st transition sensed! Start measurement
;---------------------------------------------------------------------------

MEASURE CLR     A               Setup fast mode flag
        MOV     R0,#RATE
        MOV     @R0,A
        MOV     R6,A            Setup flags for fast mode
        MOV     R4,/FAST         (R6=0 means fast)
        MOV     R5,#FAST         (200ms + interrupt time)
        JNT0    .LOOP           It realy is fast mode!
        MOV     R4,/SLOW        Setup flags for slow mode
        MOV     R5,#SLOW         (991ms + interrupt time)
        INC     R6               (R6=1 means slow)
        INC     @R0
.LOOP   CLR     A               Is rate switch changed?
        JNT0    .FAST           It's in FAST position now
        MOV     A,#1
.FAST   XRL     A,R6            Compare with last state
        JB0     RETRY.Z         State changed! Restart counting

        CALL    CHCK.HF         Abort if HF status changed
        CALL    DELAY           Decrement delay counter
        JNZ     .LOOP           Measurement time not up yet!
        ORL     P2,#%0010.0000  Signal end of measurement
        MOV     R4,/WAITLST     Wait a while for last transition
        MOV     R5,#WAITLST      (ca. 1.5s)
.WAIT   CALL    CHCK.HF         Abort if HF status changed
        JNI     CALCRES         Signalled last transition!
        CALL    DELAY           Decrement delay counter
        JNZ     .WAIT           Time not up yet!
        JMP     RETRY           No life on input! Show 0 & restart

;---------------------------------------------------------------------------
;  ROUTINE: CALCRES       ($0A5)
;  Calculate end result from counted values
;
;  The reference counter is formed from 3 different sources.
;  -The least significant 8 bits come from IC106 in binary format
;  -Then come the 8 bits from the Timer/counter in binary format
;  -Then come the 4 BCD digits that were counted in software by
;   the interrupt routine.
;  To get a total BCD counter, the software counted BCD part must
;  be multiplied by 65526 (2^16). The 8 bits from the
;  timer/counter must be converted to BCD and then multiplied by
;  256. The 8 bits from IC106 must be converted to BCD. Then
;  these three results can be added together to form the final
;  result.
;
;---------------------------------------------------------------------------

CALCRES CALL    CHCK.HF         Abort if HF status changed
        STOP    TCNT            Stop counting
        DIS     TCNTI

        MOV     R5,#16          Double the Rc 16 times (x65536)
.DOUBLE CALL    AD.RCEC         Make copy of Rc to Ec
        CALL    ADDCLR          Rc = Rc + Ec, Ec = 0
        DJNZ    R5,.DOUBLE      Not done yet!

        MOV     A,T             Convert binary timer value to BCD
        MOV     R0,#EVENT        (together with contents of IC106
        CALL    BIN2BCD           this is a 16 bit value)

        INS     A,BUS           Get contents of IC106
        MOV     R7,A            MSB is missing! It is connected to
        ANL     A,#%0111.1111    input T1 to inc. internal timer
        JNT1    .ZERO           MSB is low!
        ORL     A,#%1000.0000   Make MSB high
.ZERO   CALL    BIN2BCD         Convert these 8 bits to BCD too
        CALL    ADDCLR          Rc = Rc + Ec, Ec = 0 (Rc is done)

        ORL     P1,#%0000.1111  Enable inputs on P1
        MOV     R0,#DECADE1     Get event counter digits
        MOV     R1,#EVENT
        MOV     R4,#4           Get 4 pairs of decimal digits
.GETEVC CALL    GET2DEC         Get 2 adjacent decades
        DJNZ    R4,.GETEVC      Do all digits of event counter
        ANL     P2,#%1111.0000  Disable display control lines
        CALL    EXCH.RC         Save Rc, and clear Rc
        CALL    EX.ECRC         Exchange event with reference cntr
        CALL    AD.RCEC         Make copy of event counter
        CALL    AD.RCEC          and add copy to event counter (x2)
        MOV     A,R7            Was 2nd flip-flop set? (IC110)
        JB7     .ONE            Yes!
        JMP     RDCONT          Skip page without problems
.ONE    MOV     R0,#EVENT       Increment event counter. Will always
        INC     @R0              remain BCD.
        JMP     RDCONT          Skip page without problems

;---------------------------------------------------------------------------
;  ROUTINE: BIN2BCD       ($0E2)
;  Convert binary code byte to 6 BCD digits  (max is 65535)
;---------------------------------------------------------------------------

BIN2BCD MOV     R2,#8           Convert a total of 8 bits
        CPL     A               Complement due to lack of JNB0 inst.
        MOV     R4,A            Save byte to be converted to BCD
.LOOP   MOV     A,@R0           Multiply temp result by 2
        ADD     A,@R0           Remember that the temp result is
        DA      A                in BCD!
        MOV     @R0,A
        MOV     A,R4            Get new bit from input byte
        RL      A
        MOV     R4,A
        JB0     .ZERO           Bit was 0! (complemented remember!)
        INC     @R0             Add new bit (Always remains BCD!)
.ZERO   INC     R0              Point to 2nd BCD pair
        MOV     A,@R0            to be multiplied by 2 in BCD format
        ADDC    A,@R0
        DA      A
        MOV     @R0,A
        INC     R0              And finally double the 3d pair
        MOV     A,@R0            as well
        ADDC    A,@R0
        MOV     @R0,A
        DEC     R0              Point to first BCD pair again
        DEC     R0
        DJNZ    R2,.LOOP        Do all 8 new bits!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: ADDCLR        ($0FE)
;  Add event counter to reference counter and clear event counter
;---------------------------------------------------------------------------

ADDCLR  CALL    AD.RCEC         Add Rc to Ec
        CALL    EX.ECRC         Exchange Ec with Rc
        CALL    CLREVEN         Clear Ec
        RET

        NOP

;---------------------------------------------------------------------------
;  ROUTINE: RDCONT        ($106)
;  Read results continued from page 0
;
;  We read the reference counter, but we still must read the
;  least significant bit from Flip-Flop IC105.
;  After getting the least significant bit we have to deal with
;  the difference between the HF and LF inputs.
;  The LF input was prescaled by 10, therefore we set the
;    exponent in R7 to be 8 (10MHz x10 => 10^7 x10 = 10^8).
;  The HF input was prescaled by 256. This is not easyly compen-
;    sated by an exponent. We have to multiply the result by 256.
;    This is done by doubling the result 8 times (R3 contains 8
;    if HF input was selected). After that the exponent in R7 is
;    set to 7 (10MHz => 10^7).
;  After setting the exponent correctly, the reference counter
;   the event counter are normalised. This means that the numbers
;   are shifted left (multiplied by 10) until the MSD is at the
;   2nd BCD position of the BCD buffer. Everytime the Event
;   counter is multiplied by 10, the exponent is decremented.
;   Everytime the reference counter is multiplied by 10, the
;   exponent is incremented.
;  Then the event counter is divided by the reference counter.
;  When the result is known the measurement rate is tested. No
;   action is taken if the rate was slow. But if the rate was
;   fast we have to check the range to see if it does exceed
;   2199999. If it does, the number is shifted one digit to the
;   right to display only 6 relevant digits. The counter is not
;   accurate enough to display all 7 digits in the fast mode.
;   To avoid rapid shifting of the display when the counter
;   crosses this boundary time after time, an hysteresis is
;   introduced (flag byte bit 4). If the counter desides to
;  display only 6 digits, the hysteresis flag is set, and the
;   counter will only switch back to 7 digit display if the
;   range becomes less than 2000000.
;  After this we only have to place the decimal point in the
;   propper location and display the propper annunciators.
;   This is done in software in a difficult way. It would have
;   have been easyer to use a table with 10 entries.
;   Exponent -1        x.xxxxxx Hz   (slow)
;                       x.xxxxx Hz   (fast above 2199999)
;   Exponent 0         xx.xxxxx Hz   (slow)
;                       xx.xxxx Hz   (fast above 2199999)
;   Exponent 1         xxx.xxxx Hz   (slow)
;                       xxx.xxx Hz   (fast above 2199999)
;   Exponent 2         xxxx.xxx Hz   (slow)
;                       x.xxxxx kHz  (fast above 2199999)
;   Exponent 3         xx.xxxxx kHz  (slow)
;                       xx.xxxx kHz  (fast above 2199999)
;   Exponent 4         xxx.xxxx kHz  (slow)
;                       xxx.xxx kHz  (fast above 2199999)
;   Exponent 5         xxxx.xxx kHz  (slow)
;                       x.xxxxx MHz  (fast above 2199999)
;   Exponent 6         xx.xxxxx MHz  (slow)
;                       xx.xxxx MHz  (fast above 2199999)
;   Exponent 7         xxx.xxxx MHz  (slow)
;                       xxx.xxx MHz  (fast above 2199999)
;   Exponent 8         xxxx.xxx MHz  (slow)
;                       xxxx.xx MHz  (fast above 2199999)
;---------------------------------------------------------------------------

RDCONT  CALL    EX.ECRC         Again a very complex way to add a
        CALL    CLREVEN          number to it's self (x2).
        CALL    AD.RCEC          Rc = Ec, Ec = 0, Ec = 0 + Rc,
        CALL    AD.RCEC          Ec = Ec + Rc (what fool did this?)
        ORL     P1,#%0001.0000  Allow input of the very least
        IN      A,P1             significant bit of the event cntr
        JB4     .ONE            The LSB was 1!
        JMP     .ZERO           Don't do anything if it's 0!
.ONE    MOV     R0,#EVENT       Increment event counter. Will alays
        INC     @R0              remain BCD
.ZERO   CALL    EX.ECRC         Put result in Rc buffer
        CALL    CLREVEN         Clear event counter buffer

        MOV     R7,#8           LF exponent (10MHz reference x10)
        MOV     A,R3            Was it the HF input?
        JZ      .LF             No! Don't multiply with 256 (presc)
.DOUBLE CALL    AD.RCEC         Copy result to Ec (silly fool)
        CALL    ADDCLR          Rc = Rc + Ec, Ec = 0 (x2)
        DJNZ    R3,.DOUBLE      Do this 8 times! (is x256)
        MOV     R7,#7           HF exponent (10MHz reference)
.LF     CALL    EX.ECRC         Put result in event counter
        CALL    EXCH.RC         Restore reference counter

        CALL    NORMAL          Normalise the event counter
        CALL    EX.ECRC         Exchange event with ref. counter
        CALL    NEGATE          Normalise the refence counter
        CALL    NORMAL           (incrementing the exponent)
        JNZ     .NOT0           Not dividing by 0!
        JMP     RETRY           Reference error! Restart
.NOT0   CALL    EX.ECRC         Move Ec and Rc to original place
        CALL    NEGATE          Restore polarity of exponent

        CALL    DIVIDE          Divide Ec by Rc
        MOV     R6,#-1          Default is 7 digit display (0=>6)
        MOV     R0,#RATE        Get measurement rate
        MOV     A,@R0
        NOP
        NOP
        CALL    SCALE           Adapt the full scale to measure rate

        MOV     A,R6            Add -1 (7 digit) or 0 (6 digit) to
        ADD     A,R7             exponent to place the decimal point
        MOV     R7,A             propperly
        MOV     R5,#6           Left most dp position is digit 6
        MOV     R6,#7           But if it fails, last chance is 7
.DP     CALL    FLOAT           See if exponent is multiple of 3
        JZ      .POINT          It is now! R0 => Hz, kHz or MHz
        DEC     R7              Decrement and shift dp to the right
        DJNZ    R5,.DP          Try again!
.LESS10 INC     R5              Could not position the DP, the only
        INC     R7               chance is next to the left most
        CALL    FLOAT            digit for frequencies below 10Hz
        JZ      .POINT          Found it at last!
        DJNZ    R6,.LESS10      Work your way back MSD!
        JMP     RETRY           Don't know where to put the DP!
.POINT  JMP     .NOP            No use to jump to next instruction
.NOP    MOV     A,R0            3=Hz 2=kHz 1=MHz
        RL      A               Convert 3 to %0000.1101  (Hz)
        XRL     A,#%0000.1010   Convert 2 to %0000.1110  (kHz)
        XCH     A,R0            Convert 1 to %0000.1001  (MHz)
        ANL     A,#1            In all cases don't show Low bat
        ADD     A,R0
        MOV     R0,#DISPBUF     Put result in annunciator part
        MOV     @R0,A            of the display buffer
        MOV     A,#DISPBUF      Set decimal point at propper digit
        ADD     A,R5
        MOV     R0,A
        MOV     A,@R0
        ORL     A,#%1000.0000
        MOV     @R0,A
        CALL    SHOWDSP         Show final value on the display
        JMP     AGAIN           Start new measurement

;---------------------------------------------------------------------------
;  ROUTINE: ROMTST1       ($179)
;  Calculate CRC for ROM page 1
;---------------------------------------------------------------------------

ROMTST1 MOV     A,R0            Get byte from ROM page 2
        MOVP    A,@A
        CALL    CRC             Calculate CRC from it
        DJNZ    R0,ROMTST1      Dot this for all 256 bytes
        JMP     ROMTST2         Continue for ROM Page 2

;---------------------------------------------------------------------------
;  ROUTINE: CHCK.HF       ($181)
;  See if HF status changed. Restart measurement if it changed
;---------------------------------------------------------------------------

CHCK.HF NOP                     Waste some extra time
        NOP                      routine takes 13 cycles to exec.
        IN      A,P1            Get HF status (b6)
        RR      A                and move it to b3
        RR      A
        RR      A
        CPL     A               Did it change?
        XRL     A,R3             (reference still in R3)
        JB3     .CHNGD          The HF status changed!
        RET
.CHNGD  JMP     RETRY           Measurement failed! Restart
        RET

;---------------------------------------------------------------------------
;  ROUTINE: DELAY         ($18F)
;  Decrement delay counter  (counting up until 0)
;---------------------------------------------------------------------------

DELAY   MOV     A,R5            Increment LSB in a very complex
        ADD     A,#1             way (to obtain a carry)
        MOV     R5,A
        CLR     A               Increment MSB if carry is set
        ADDC    A,R4             (done until A=0)
        MOV     R4,A
        RET                      Routine takes 11 cycles to exec.

;---------------------------------------------------------------------------
;  ROUTINE: DIVIDE        ($197)
;  Divide event counter by reference counter (7 digits)
;   Here we can see why the normalisation didn't shift the digits
;   all the way to the left. If the MSD of the event counter is
;   less than the MSD of the reference counter, they can not be
;   subtracted from each other. Therefore the event counter will
;   be multiplied by 10 once more, and the exponent is decre-
;   mented. This will avoid a leading zero on the display.
;  It would have been better to calculate an 8 digit result and 
;   round off to 7 digits. This would increase the accuracy of
;   the least significant digit.
;---------------------------------------------------------------------------

DIVIDE  MOV     R5,#7           Do 7 digits
        MOV     R6,#DISPBUF+7   Start with MSB
        CALL    SUBTRCT         Try to subtract Rc from Ec
        JC      .OK             Subtraction worked!
        CALL    AD.RCEC         Didn't go! Ec<Rc! Restore value
        DEC     R7              Decrement exponent
.NXTDIG CALL    MULEC10         Multiply Ec by 10
.AGAIN  CALL    SUBTRCT         Try to subtract Rc from Ec
        JNC     .NOTOK          Did not go! Ec<Rc!
.OK     MOV     A,R6            Point to propper digit
        MOV     R0,A             and increment it every time the
        INC     @R0              subtraction works
        JMP     .AGAIN          Try to subtract again
.NOTOK  CALL    AD.RCEC         Add reference cntr to event cntr
        DEC     R6              Point to next digit
        DJNZ    R5,.NXTDIG      Do a maximum of 7 digits
        RET

;---------------------------------------------------------------------------
;  ROUTINE: MULEC10       ($1B3)
;  Multiply Ec by 10
;---------------------------------------------------------------------------

MULEC10 CALL    EVE.REF         R0 points to Ec, R2=6
        MOV     R3,#0           Move a 0 in from the right
.LOOP   MOV     A,@R0           Get BCD pair and swap lower and
        SWAP    A                upper digit
        MOV     @R0,A
        MOV     A,R3            Swap highest digit from previous
        XCHD    A,@R0            byte with former highest digit
        MOV     R3,A             of current byte
        INC     R0
        DJNZ    R2,.LOOP        Do all 6 nibble pairs
        RET

;---------------------------------------------------------------------------
;  ROUTINE: AD.RCEC       ($1C1)
;  Add reference counter to event counter
;---------------------------------------------------------------------------

AD.RCEC CALL    EVE.REF         Point to event & reference counter
        CLR     C               (ADDC is used)
.LOOP   MOV     A,@R0           Get byte from event counter
        ADDC    A,@R1            and add it to the reference counter
        DA      A                 in BCD
        MOV     @R0,A           Result is stored in event counter
        INC     R0              Increment both pointers
        INC     R1
        DJNZ    R2,.LOOP        Do all 6 pairs!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: SUBTRCT       ($1CD)
;  Subtract Rc from Ec in BCD format
;---------------------------------------------------------------------------

SUBTRCT CALL    EVE.REF         R0=>Ec, R1=>Rc, R2=6
        CLR     C               Set carry (no borrow state)
        CPL     C
.LOOP   MOV     A,#$99          (-1 in BCD)
        ADDC    A,#$00          Add the borrow
        MOV     R3,A            Save borrow state
        MOV     A,@R1           Form 2s complement value of Rc byte
        CPL     A
        ADD     A,#1
        ADD     A,R3            R3=$9A if no borrow needed
        ADD     A,@R0           Subtract Rc from Ec byte
        DA      A                in BCD form
        MOV     @R0,A
        INC     R0              Increment both pointers
        INC     R1
        DJNZ    R2,.LOOP        Do all 6 bytes
        RET

;---------------------------------------------------------------------------
;  ROUTINE: EVE.REF       ($1E3)
;  Setup pointers to event counter and reference counter
;---------------------------------------------------------------------------

EVE.REF MOV     R1,#REFCNTR     Point to reference counter
        MOV     R0,#EVENT       Point to event counter
        MOV     R2,#6           Both groups contain 6 bytes
        RET

;---------------------------------------------------------------------------
;  ROUTINE: EX.ECRC       ($1EA)
;  Exchange 6 bytes from event counter with reference counter
;---------------------------------------------------------------------------

EX.ECRC CALL    EVE.REF         Point to event & reference counters

;---------------------------------------------------------------------------
;  ROUTINE: EXCHG6        ($1EC)
;  Exchange two groups of 6 bytes with each other (@R0,@R1)
;---------------------------------------------------------------------------

EXCHG6  MOV     R2,#6           Exchange two groups of 6 bytes
.LOOP   MOV     A,@R0           Get byte from first group
        XCH     A,@R1            exchange it with byte of 2nd group
        MOV     @R0,A             store byte of 2nd group in 1st
        INC     R0              Increment both group pointers
        INC     R1
        DJNZ    R2,.LOOP        Do all 6 bytes!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: EXCH.RC       ($1F6)
;  Exchange reference counter with display buffer (to save it)
;---------------------------------------------------------------------------

EXCH.RC MOV     R0,#REFCNTR     Setup pointers for reference counter
        MOV     R1,#DISPBUF+1    and display buffer
        JMP     EXCHG6          Exchange 6 bytes

;---------------------------------------------------------------------------
;  ROUTINE: CLREVEN       ($1FC)
;  Clear 6 bytes of the event counter
;---------------------------------------------------------------------------

CLREVEN MOV     R0,#EVENT       Setup pointer to event counter
        MOV     R4,#6           Setup byte counter
        CLR     A
.LOOP   MOV     @R0,A           Clear the byte
        INC     R0
        DJNZ    R4,.LOOP        Not done!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: NEGATE        ($206)
;  Negate the exponent so decrement becomes increment
;---------------------------------------------------------------------------

NEGATE  MOV     A,R7            Form 2s complement of exponent
        CPL     A                so the decrement instruction
        INC     A                becomes an increment instruction
        MOV     R7,A
        RET

;---------------------------------------------------------------------------
;  ROUTINE: FLOAT         ($20B)
;  See if the exponent is a multiple of 3
;---------------------------------------------------------------------------

FLOAT   MOV     R0,#3           Max exponent is (3-1)x3=6
        MOV     A,R7
.LOOP   JZ      .RET            It is a multiple of 3!
        ADD     A,#-3           Subtract 3 again
        DJNZ    R0,.LOOP        Try again!
        MOV     A,R7            Force non zero value
.RET    RET

;---------------------------------------------------------------------------
;  ROUTINE: FULLSCL       ($216)
;  Check full scale for fast mode (upper hysteresis 2199999)
;  If full scale reached Cy is clear, otherwise it is set
;---------------------------------------------------------------------------

FULLSCL MOV     R0,#DISPBUF+7   Get upper two digits and combine
        MOV     A,@R0            them into one BCD byte
        SWAP    A
        DEC     R0
        ADD     A,@R0
        MOV     R5,A

;---------------------------------------------------------------------------
;  ROUTINE: FULLSCZ       ($21D)
;  Check full scale for fast mode (lower hysteresis 1999999)
;  If full scale reached Cy is clear, otherwise it is set
;---------------------------------------------------------------------------

FULLSCZ MOV     A,R5            Get both digits and see if the
        CPL     A                value is 22 or up
        INC     A               Values of 22 or up clear the Cy
        INC     A               Values below 22 set the Cy
        ADD     A,#$20
        RET

;---------------------------------------------------------------------------
;  ROUTINE: ROMTST2       ($224)
;  Calculate CRC for ROM page 2
;---------------------------------------------------------------------------

ROMTST2 MOV     A,R0            Get byte from ROM page 2
        MOVP    A,@A
        CALL    CRC             Calculate CRC from it
        DJNZ    R0,ROMTST2      Do all 256 bytes!
        JMP     ROMTST3         Continue for ROM page 3

;---------------------------------------------------------------------------
;  ROUTINE: CLRFLAG       ($22C)
;  Clear hysteresis flag (A=%1110.1111)
;---------------------------------------------------------------------------

CLRFLAG SEL     RB1             Access the flag byte
        ANL     A,R2             clear the flag and write it back
        MOV     R2,A            Fall through set flag (no function)

;---------------------------------------------------------------------------
;  ROUTINE: SETFLAG       ($22F)
;  Set hysteresis flag (A=%0001.0000)
;---------------------------------------------------------------------------

SETFLAG SEL     RB1             Access flag byte
        ORL     A,R2            Set the flag and write it back
        MOV     R2,A
        SEL     RB0
        RET

;---------------------------------------------------------------------------
;  ROUTINE: SHIFT         ($234)
;  Shift display right 1 digit to display only 6 digits
;---------------------------------------------------------------------------

SHIFT   INC     R6              R6=0 if only 6 digits are displayed
        MOV     R0,#DISPBUF+7   Start at MSD
        MOV     R2,#7           Do 7 digits
        MOV     A,#PAT.BL       MSD becomes blank
.LOOP   XCH     A,@R0           Exchange current digit with previous
        DEC     R0
        DJNZ    R2,.LOOP        Do all digits!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: NORMAL        ($240)
;  Normalise event counter (Left justify)
;  Everytime the number is multiplied by 10, the exponent is
;  decremented (incremented if reference counter is normalised)
;  The number is normalised if the left most byte doesn't contain
;  $00 anymore. Therefore the upper nibble always remains 0!
;---------------------------------------------------------------------------

NORMAL  MOV     R5,#12          Do a maximum of 12 digits
.LOOP   MOV     R0,#EVENT+5     See if highest byte is <> 0
        MOV     A,@R0
        JNZ     .RET            Yes! Normalised!
        MOV     R0,#EVENT       Will be overwritten again!!!
        CALL    MULEC10         Multiply event counter by 10
        DEC     R7              Decrement exponent counter
        DJNZ    R5,.LOOP        Do until normalised!
        MOV     A,R4            R4 will always be 0! So this test
        JB2     .RET             will always fail! And A will
        CLR     A                always be zero'd!
.RET    RET                     A=0 if counter value is 0

;---------------------------------------------------------------------------
;  ROUTINE: SCALE         ($253)
;  Adapt scale for fast mode and set hysteresis
;---------------------------------------------------------------------------

SCALE   JNZ     .SLOW           Slow mode! Always 7 digits
        CALL    FULLSCL         Full scale reached? (2199999)
        JNC     .DISP6          Yes! Display only 6 digits
        MOV     A,R5            Make shure that the value of R5
        SWAP    A                is 22 or more if the first digit
        ANL     A,#%0000.1111     is a 2
        ADD     A,R5            (21=>23, 20=>22, 19=>20)
        DA      A
        JC      .NEVER          This jump is never taken!
        MOV     R5,A            See if value is above 2199999
        CALL    FULLSCZ
        JNC     .HYST           It is above! See if 6 or 7 digits
.SLOW   MOV     A,#%1110.1111   Clear hysteresis flag
        CALL    CLRFLAG          and display 7 digits
        RET
.NEVER  MOV     @R0,#$94        These three instructions make no
        CALL    FULLSCZ          sence to me, because the program
        JNC     .DISP6           never gets here!
.HYST   MOV     R0,#R2RB1       Get hysteresis bit (b4 of flagbyte)
        MOV     A,@R0
        CPL     A               Compensate for lack of JNB4 instr
        JB4     .RET            Hysteresis=0, 7 digits allowed!
.DISP6  MOV     A,#%0001.0000   Set hysteresis flag
        CALL    SETFLAG
        CALL    SHIFT           Shift all digits right to clear the
.RET    RET                      left most digit

;---------------------------------------------------------------------------
;  ROUTINE: CLEAR         ($27E)
;  Clear specified part of memory
;  R0 contains the begin address, R4 contains the number of bytes
;  to be cleared
;---------------------------------------------------------------------------

CLEAR   CLR     A               Setup fill byte
.LOOP   MOV     @R0,A           Stuff it
        INC     R0              Point to next one
        DJNZ    R4,.LOOP        Do them all!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: GET2DEC       ($284)
;  Get 2 adjacent decades from decade counter
;---------------------------------------------------------------------------

GET2DEC CALL    READDEC         Get least significant decade
        MOV     R2,A
        CALL    READDEC         Get most significant decade
        SWAP    A                and move it in position
        ORL     A,R2            Form a double BCD byte
        MOV     @R1,A            and save it in RAM
        INC     R1              Point to next location
        RET

;---------------------------------------------------------------------------
;  ROUTINE: READDEC       ($28E)
;  Read selected (R0) decade counter value
;---------------------------------------------------------------------------

READDEC ANL     P2,#%1111.0000  Select none of the decade counters
        MOV     A,R0            Get decade counter select mask from
        INC     R0               table and point to the next one
        MOVP    A,@A             for later use
        MOV     R5,A            Combine this mask with rest of
        IN      A,P2             the bits of P2
        ORL     A,R5
        OUTL    P2,A            Select propper decade
        IN      A,P1            Get decade value
        ANL     A,#%0000.1111    and mask off irrelevant bits
        RET

DECADE1 .DA     #%0000.0100     Select LSD (10 of IC107)
DECADE2 .DA     #%0000.0101     Select 2nd digit (100 of IC107)
DECADE3 .DA     #%0000.0110     Select 3d digit (1000 of IC107)
DECADE4 .DA     #%0000.0111     Select 4th digit (10000 of IC107)
DECADE5 .DA     #%0000.1000     Select 5th digit (10 of IC108)
DECADE6 .DA     #%0000.1001     Select 6th digit (100 of IC108)
DECADE7 .DA     #%0000.1010     Select 7th digit (1000 of IC108)
DECADE8 .DA     #%0000.1011     Select 8th digit (10000 of IC108)

;---------------------------------------------------------------------------
;  ROUTINE: SHOWDSP       ($2A3)
;  Show display contents if display update is allowed (R2.0=0)
;---------------------------------------------------------------------------

SHOWDSP MOV     R0,#DISPBUF+7   Point to left digit in buffer
        SEL     RB1             Get flag byte
        MOV     A,R2
        SEL     RB0
        JB0     .RET            Display update not allowed!
        JMP     .ALLOWD         Display update allowed
.RET    RET
.ALLOWD ANL     P2,#%1111.0011  Disable decimal counter outputs
        ORL     P1,#%0010.0000  Select LOW BAT line as input
        IN      A,P1            See if LOW BAT line is active
        JB5     .BATOK          LOW BAT is not active!
        MOV     R1,#DISPBUF     Switch LOW BAT indicator on
        MOV     A,@R1
        ANL     A,#%0000.1000!$FF
        MOV     @R1,A
.BATOK  ANL     P1,#%1111.0000  Clear all 3 display enables and data
        ANL     P2,#%1110.1111  Make display clock low
        ORL     P1,#%0000.0100  Enable display controller EN1
        CLR     A
        CALL    DISP1           Send low start bit
        CALL    DISPDIG         Convert two digits to 7 segment
        CALL    DISPDIG          patterns and display them all
        CALL    GETSEGM         Get next digit and send only the
        CALL    DISP4            4 lsbs to this controller
        MOV     R3,A            The others will go to the next one
        ANL     P1,#%1111.1011  Release EN1
        ORL     P1,#%0000.0010  Enable display controller EN2
        CLR     A
        CALL    DISP1           Send low start bit
        MOV     A,R3            Get rest of 3d digit
        CALL    DISP4           Send 4 bits to the display
        CALL    DISPDIG         Convert two digits to 7 segment
        CALL    DISPDIG          patterns and display them all
        ANL     P1,#%1111.1101  Release EN2
        ORL     P1,#%0000.0001  Enable display controller EN3
        CLR     A
        CALL    DISP1           Send low start bit
        CALL    DISPDIG         Convert two digits to 7 segment
        CALL    DISPDIG          patterns and display them all
        MOV     A,@R0           Send 4 bits of annunciator data
        CALL    DISP4            to the display
        ANL     P1,#%1111.1110  Release EN3
        CLR     A               Send final load pulse to last
        CALL    DISP1            display controller EN3
        RET

;---------------------------------------------------------------------------
;  ROUTINE: DISPDIG       ($2EE)
;  Comvert digit from display buffer to 7 segment pattern and
;  display all bits
;---------------------------------------------------------------------------

DISPDIG CALL    GETSEGM         Convert digit to 7 segment pattern
        CALL    DISP8           Send all 8 bits to the display
        RET

;---------------------------------------------------------------------------
;  ROUTINE: DISP1         ($2F3)
;  Send just 1 bit of the Accu to the display controller
;---------------------------------------------------------------------------

DISP1   MOV     R6,#1           Set bit counter to 1
        JMP     SNDDISP

;---------------------------------------------------------------------------
;  ROUTINE: DISP4         ($2F7)
;  Send 4 bits of the Accu to the display controller
;---------------------------------------------------------------------------

DISP4   MOV     R6,#4           Set bit counter to 4
        JMP     SNDDISP

;---------------------------------------------------------------------------
;  ROUTINE: DISP8         ($2FB)
;  Send all 8 bits of the Accu to the display controller
;---------------------------------------------------------------------------

DISP8   MOV     R6,#8           Set bit counter to 8
        JMP     SNDDISP
        NOP

        .HS     .00.09.01       Never used (maybe CRC adjustment)

;---------------------------------------------------------------------------
;  ROUTINE: SNDDISP       ($303)
;  Send bits over to the display controller(s)
;  The number of bits to be transferred is held in R6
;  The bits itself are held in the Accu (lsb send first)
;---------------------------------------------------------------------------

SNDDISP ORL     P1,#%0000.1000  Make display data line high
        JB0     .HIGH           And keep it that way!
        ANL     P1,#%1111.0111  Sorry! Should have been low
.HIGH   RR      A               Prepare next bit for next time
        ORL     P2,#%0001.0000  Pulse display clock high for 5s
        NOP
        NOP
        NOP
        ANL     P2,#%1110.1111
        DJNZ    R6,SNDDISP      Do all the wanted bits!
        RET

;---------------------------------------------------------------------------
;  ROUTINE: GETSEGM       ($314)
;  Convert digit under digit pointer to 7 segment pattern and
;  decrement digit pointer for later use.
;---------------------------------------------------------------------------

GETSEGM MOV     A,@R0           Get digit to be converted
        DEC     R0              Decrement digit pointer for later
        MOV     R7,A            Save digit
        CPL     A               Invert DP status
        ANL     A,#%1000.0000    and isolate it
        XCH     A,R7            Get digit back
        ANL     A,#%0000.1111   Mask off illegal bits
        ADD     A,#.TABLE       Add offset to segment table
        MOVP    A,@A            Get segment pattern
        ORL     A,R7            Combine with DP status
        RET

        ;segments pcde bafg
.TABLE  .DA     #%1111.1110!$FF    digit 0
        .DA     #%1100.1000!$FF    digit 1
        .DA     #%1011.1101!$FF    digit 2
        .DA     #%1110.1101!$FF    digit 3
        .DA     #%1100.1011!$FF    digit 4
        .DA     #%1110.0111!$FF    digit 5
        .DA     #%1111.0111!$FF    digit 6
        .DA     #%1100.1100!$FF    digit 7
        .DA     #%1111.1111!$FF    digit 8
        .DA     #%1110.1111!$FF    digit 9
        .DA     #%1000.0000!$FF    blank
        .DA     #%1011.0111!$FF    character E
        .DA     #%1001.0001!$FF    character r
        .DA     #%1111.0001!$FF    character o
PAT.BL  .EQ     $0A             Digit value for blank
PAT.E   .EQ     $0B             Digit value for character E
PAT.R   .EQ     $0C             Digit value for character R
PAT.O   .EQ     $0D             Digit value for character O

;---------------------------------------------------------------------------
;  ROUTINE: ROMTST3       ($330)
;  Calculate CRC for ROM page 3
;  Conitnue test afterwards
;---------------------------------------------------------------------------

ROMTST3 SEL     RB1             Clear flag byte in case of an
        MOV     R2,#0            error, otherwise the display might
        SEL     RB0              be blocked
        MOV     R3,#0           Clear the error number
.LOOP   MOV     A,R0            Get byte from ROM page 3
        MOVP    A,@A
        CALL    CRC             Calculate CRC from it
        DJNZ    R0,.LOOP        Do all 256 bytes!
        MOV     A,R1            Both CRC bytes must be 0 now
        ORL     A,R2
        JNZ     ERROR1          It's not! This is Error 1
        MOV     R3,#0           Clear the error number (again)
        MOV     A,#$FF          Test if R0 is working propperly
        MOV     R0,A             before using it as index to test
        MOV     A,R0             the rest of the RAM
        CPL     A
        JNZ     ERROR1          R0 failed!
        CLR     A               Try complemented bits of R0 too
        MOV     R0,A
        MOV     A,R0
        JNZ     ERROR1          R0 failed here!
        MOV     A,#$FF          Do the same with R1 before using
        MOV     R1,A             it as RAM counter to test the
        MOV     A,R1             rest of the RAM
        CPL     A
        JNZ     ERROR1          R1 failed!
        CLR     A               Try complemented bits of R1 too
        MOV     R1,A
        MOV     A,R1
        JNZ     ERROR1          R1 failed here!
        MOV     R0,#2           Start testing rest of RAM from
        MOV     R1,#$3E          address 2 for $3E bytes
.RAMTST MOV     A,#$FF          Test all RAM bytes the same way
        MOV     @R0,A            we tested R0 and R1 before.
        CLR     A               (Note: if RAM fails to work no
        MOV     A,@R0             error code might be displayed!)
        CPL     A
        JNZ     ERROR1          RAM failed here!
        CLR     A               Try complemented bits too
        MOV     @R0,A
        MOV     A,#$FF
        MOV     A,@R0
        JNZ     ERROR1          RAM failed here!
        INC     R0              Point to next byte in RAM
        DJNZ    R1,.RAMTST      Do all available RAM bytes!

        ANL     P2,#%1100.1111  Keep IC106 and the decade counters
        ORL     P2,#%1000.0000   in reset, but apply pulses to IC106
        ORL     P2,#%0110.0000  If everything's OK it should remain
        ANL     P2,#%1000.0000   0, because the reset remains active
        ORL     P1,#%0000.1111  Force decade counter inputs high
        MOV     R0,#DECADE1     Read decade 1 of IC107
        MOV     R1,#4           Read a total of 4 decades from it
.IC107  CALL    READDEC
        JNZ     ERROR2          Decade should have been 0!
        DJNZ    R1,.IC107       Do all 4 decades of IC107!
        MOV     R1,#4           Do all 4 decades of IC108 too
.IC108  CALL    READDEC         Read one decade
        JNZ     ERROR3          Should have been 0!
        DJNZ    R1,.IC108       Not done yet!

        INS     A,BUS           Read reference counter
        MOV     R1,A
        ANL     A,#%0111.1111   MSB is connected to T1
        JNT1    .MSB            MSB is 0!
        ORL     A,#%1000.0000   MSB is 1
.MSB    JNZ     ERROR4          Should have been 0!
        MOV     A,R1            Check the output of IC110
        ANL     A,#%1000.0000
        JNZ     ERROR5          This should have been 0 too!
        ORL     P1,#%0001.0000  Finally check the level of
        IN      A,P1             IC105
        ANL     A,#%0001.0000
        JNZ     ERROR6          Should have been 0!
        MOV     R0,#DISPBUF     Clear annunciators
        MOV     @R0,#%1111.1111
        JMP     RETRY           Start meter

;---------------------------------------------------------------------------
;  ROUTINE: ERROR6        ($3A8)
;  1st flip-flop IC105 failed to indicate 0
;---------------------------------------------------------------------------

ERROR6  INC     R3              Increment error number 6 times

;---------------------------------------------------------------------------
;  ROUTINE: ERROR5        ($3A9)
;  2nd flip-flop IC110 failed to indicate 0
;---------------------------------------------------------------------------

ERROR5  INC     R3              Increment error number 5 times

;---------------------------------------------------------------------------
;  ROUTINE: ERROR4        ($3AA)
;  Reference counter IC106 failed to indicate 0
;---------------------------------------------------------------------------

ERROR4  INC     R3              Increment error number 4 times

;---------------------------------------------------------------------------
;  ROUTINE: ERROR3        ($3AB)
;  Decade counter IC107 failed to indicate 0
;---------------------------------------------------------------------------

ERROR3  INC     R3              Increment error number 3 times

;---------------------------------------------------------------------------
;  ROUTINE: ERROR2        ($3AC)
;  Decade counter IC107 failed to indicate 0
;---------------------------------------------------------------------------

ERROR2  INC     R3              Increment error number twice

;---------------------------------------------------------------------------
;  ROUTINE: ERROR1        ($3AD)
;  An error was found inside the micro processor
;---------------------------------------------------------------------------

ERROR1  INC     R3              Increment error number only once

        MOV     R0,#DISPBUF+1   Point to right most digit in
        MOV     A,R3             the display buffer and put the
        MOV     @R0,A            error number there
        MOV     R0,#DISPBUF+7   Point to the left most digit in
        MOV     @R0,#PAT.E       the buffer and put the word
        DEC     R0               'ERROR ' there
        MOV     @R0,#PAT.R
        DEC     R0
        MOV     @R0,#PAT.R
        DEC     R0
        MOV     @R0,#PAT.O
        DEC     R0
        MOV     @R0,#PAT.R
        DEC     R0
        MOV     @R0,#PAT.BL
        DEC     R0              Skip error number digit
        DEC     R0
        MOV     @R0,#%1111.1111 Switch all annunciators off
        CALL    SHOWDSP
        SEL     RB1             Don't allow the display to be
        INC     R2               updated anymore
        SEL     RB0
        JMP     RETRY           Start meter anyway

;---------------------------------------------------------------------------
;  ROUTINE: INIT          ($3D0)
;  Get things started
;---------------------------------------------------------------------------

INIT    MOV     A,#8+%1000.0000  Show 8. on all digits (lamptest)
        MOV     R0,#DISPBUF+7
        MOV     R1,#7           There are 7 digits to be filled
.LAMP   MOV     @R0,A           Fill them
        DEC     R0
        DJNZ    R1,.LAMP        Do them all!
        MOV     @R0,#%0000.0000 All annunciators are on
        SEL     RB1             Allow display to be updated
        MOV     R2,#0
.WAIT   DJNZ    R2,.WAIT        Wait a while!
        SEL     RB0             Flag byte will be cleared here
        CALL    SHOWDSP         Show the display contents
        JMP     ROMTEST         Do a complete selftest (useless)

;---------------------------------------------------------------------------
;  ROUTINE: CRC           ($3E6)
;  Calculate CRC of all ROM bytes
;  CRC value is stored in R1 and R2
;---------------------------------------------------------------------------

CRC     MOV     R3,#8           Do 8 bits
.LOOP   RLC     A               Get left most digit of what's left
        MOV     R4,A             of the ROM byte in the carry
        MOV     A,R2            Complement previous value of CRC
        CPL     A
        JB6     .BIT6           Bit 6 is set!
        CPL     C               If not complement new bit
.BIT6   JB7     .BIT7           Bit 7 is set!
        CPL     C               If not complement new bit (again)
.BIT7   MOV     A,R1            Complement 2nd byte of CRC
        CPL     C
        JB1     .BIT1           Bit 1 is now set
        CPL     C               If not complement new bit (again)
.BIT1   RLC     A               Roll new bit into old CRC
        MOV     R1,A
        MOV     A,R2
        RLC     A
        MOV     R2,A
        MOV     A,R4            Get rest of ROM byte back
        DJNZ    R3,.LOOP        Do this for all 8 bits!
        RET

        .LI     OFF

