;---------------------------------------------------------------------------
;
;   BUS.ASM
;
;   Include file for SB6668 Frequency counter
;
;   Contains SB-Bus related routines
;
;   Remark: 1st 2 instructions of each SB-Bus command handling routine
;           should contain 2 LCALL instructions because a fixed instruc-
;           tion length is expected!
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
;   ROUTINE    :  POLL_SOUT
;   FUNCTION   :  Get the serial output started
;---------------------------------------------------------------------------

POLL_SOUT       JB      SOUT_ACTIVE,.RET Already active!
                JNB     SB_SELECT,.RET   SB-Bus not selected!
                JB      SB_GENERAL,.RET  Selected through general call!
                JB      SOUT_HALTED,.RET XOFF active! Don't transmit
                MOV     A,SOUT_HEAD      Is there anything to transmit?
                XRL     A,SOUT_TAIL
                JZ      .RET             No!
                MOV     A,CR_DELAY       CR delay over yet?
                JNZ     .RET             No! Keep waiting
                SETB    SOUT_ACTIVE      Set the active flag
                MOV     DPSEL,#2         Get character to transmit
                MOV     DPL,SOUT_TAIL
                INC     SOUT_TAIL
                MOVX    A,@DPTR
                MOV     SBUF,A            and transmit it
                MOV     DPSEL,#0
                JNB     SLOW_MODE,.RET   We're not in slow mode!
                CJNE    A,#CR,.RET       It was not a CR!
                MOV     CR_DELAY,#CR_DELAY_TIME Start CR delay counter
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  COUT
;   FUNCTION   :  Send character to SB-Bus
;---------------------------------------------------------------------------

COUT            JNB     SB_SELECT,.RET  Nothing to be done!
                JB      SB_GENERAL,.RET Nothing allowed to do!
                PUSH    DPSEL
                MOV     DPSEL,#2        Select output buffer
                MOV     DPL,SOUT_HEAD
                PUSH    ACC
                ANL     A,#%0111.1111   Get rid of end flag
                MOVX    @DPTR,A
                MOV     A,DPL           Increment pointer
                INC     A
                XRL     A,SOUT_TAIL
                JZ      .SKIP           But not if buffer is full!
                INC     SOUT_HEAD
.SKIP           POP     ACC
                POP     DPSEL
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  SEND_STRING
;   FUNCTION   :  Send string, pointed to by DPTR0 to SB-Bus
;---------------------------------------------------------------------------

SEND_STRING     >GETC
                INC     DPTR
                CALL    COUT            Send character to the bus
                JNB     ACC.7,SEND_STRING Send until end marker is found
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  GET_UC
;   FUNCTION :  Get next upper case character and increment pointer
;   OUPUT    :  A=This upper-case character
;               DPTR3=Pointer to next character
;               C=1 if blank or end of line
;---------------------------------------------------------------------------

GET_UC          PUSH    DPSEL           Save current datapointer
                MOV     DPSEL,#3        Get character from input string
                MOVX    A,@DPTR
                INC     DPL             Increment data pointer
                CJNE    A,#CR,.NOTCR    It's not a CR
                DEC     DPL             Don't increment if it's a CR
.NOTCR          >CMP    A,"#'a'"        Is it lower case?
                JC      .NO_LC          No!
                >CMP    A,"#'z'+1"
                JNC     .NO_LC          No!
                ANL     A,#%1101.1111   Make it upper case
.NO_LC          MOV     LAST_CHAR,A     Save last character
                POP     DPSEL
                CJNE    A,#' ',.NOTBLANK It's not a space!
                SJMP    .BLANK
.NOTBLANK       CJNE    A,#CR,.RET      It's not a CR
.BLANK          SETB    C               Mark end of string
                RET
.RET            CLR     C               Not end of string
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  CLR_SIN
;   FUNCTION :  Clear current string from serial input buffer
;---------------------------------------------------------------------------

CLR_SIN         MOV     A,CR_COUNT      Is there a string in the buffer?
                JZ      .RET            No!
                MOV     DPSEL,#3        Point to begin of string
                MOV     DPH,/SIN_BUFFER
                MOV     DPL,SIN_TAIL
.CLR_LOOP       MOVX    A,@DPTR         Get character
                INC     DPL
                CJNE    A,#CR,.CLR_LOOP End not found yet!
                DEC     CR_COUNT        Decrement CR counter
                MOV     SIN_TAIL,DPL
.RET            MOV     DPSEL,#0        Restore original pointer
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  POLL_SBBUS
;   FUNCTION :  Poll SB-Bus input status and decode if command completed
;---------------------------------------------------------------------------

POLL_SBBUS      MOV     A,CR_COUNT      Has a complete commande arrived yet?
                JNZ     .COMMAND_IN     Yes it has!
                RET

.COMMAND_IN     MOV     DPSEL,#3          Load parse data pointer
                MOV     DPH,/SIN_BUFFER   Point to begin of command
                MOV     DPL,SIN_TAIL
                CALL    NEXT_NON_SPACE    Find first non space character
                JC      .REPEAT           Repeat command!
                JNB     SB_GENERAL,.DO_IT Selected by private call!
                CJNE    A,#'*',.CLR_SIN   General call, no normal commands

.DO_IT          MOV     DPSEL,#0        Setup command table parse pointer
                MOV     DPTR,#COMMAND_TABLE
                CALL    PARSE           Start parsing the command
                JNC     .SYNTAX_ERROR   Exit with syntax error!
                MOV     DPSEL,#0        Point to command table
                MOV     A,R0            Multiply string number by 2
                RL      A
                ADD     A,#CMD_ADDRESS  Get address to calculate the
                MOV     DPL,A            repeat address which is 3 bytes
                MOV     A,/CMD_ADDRESS   above the normal address
                ADDC    A,#0
                MOV     DPH,A
                >GETC
                MOV     R0,A            Save high byte of routine address
                INC     DPTR
                >GETC
                MOV     R1,A            Save low byte of routine address
                ADD     A,#3
                MOV     REPEAT_CMD+1,A
                MOV     A,R0
                ADDC    A,#0
                MOV     REPEAT_CMD,A
                JNB     SLOW_MODE,.FAST          We're in fast mode!
                MOV     CR_DELAY,#CR_DELAY_TIME  Setup delay time
.FAST           PUSH    B0_R1           Start the routine by pushing
                PUSH    B0_R0            it's address on the stack
                RET                      followed by a RET

.REPEAT         MOV     A,ERROR         Was it syntax error last time?
                DEC     A
                JZ      .SYNTAX_ERROR   Yes! Than it will be this time too
                MOV     DPSEL,#0
                PUSH    REPEAT_CMD+1    Push the repeat command on the
                PUSH    REPEAT_CMD       stack to execute it
                RET

.CLR_SIN        JMP     CLR_SIN         Clear this line from buffer

.SYNTAX_ERROR   MOV     ERROR,#1        Send syntax error status
                JMP     SEND_STATUS     Send syntax error status

;---------------------------------------------------------------------------
;   ROUTINE  :  PARSE
;   FUNCTION :  Try to parse a string
;   INPUT    :  DPTR0=Parse table pointer
;               DPTR3=Begin of string to be parsed
;   OUTPUT   :  R0=String number (Starting from 0)
;               C=0 string not found, C=1 string is found
;               DPTR3=Begin of string if C=0, end of string if C=1
;---------------------------------------------------------------------------

PARSE           MOV     DPSEL,#3        Save begin of parse string
                MOV     PARSE_SAVE,DPH
                MOV     PARSE_SAVE+1,DPL
                MOV     R0,#0           Clear parse pointer

.TRY_NEXT       MOV     DPSEL,#3        Restore parse string begin
                MOV     DPH,PARSE_SAVE
                MOV     DPL,PARSE_SAVE+1
.LOOP           >GETC   0               End of parse table?
                JZ      .NOT_FOUND      Yes! String not found!
                CALL    GET_UC          Get an upper case character
                JC      .NOT_THIS_ONE   Premature end of parse string!
                CJNE    A,#',',.NOCOMMA It's not a comma!
                SJMP    .NOT_THIS_ONE   A comma is the end of the string
.NOCOMMA        MOV     B,A
                >GETC   0               Get parse table character
                XRL     A,B             Compare characters together
                CLR     C
                RLC     A               Ignore end flag
                JNZ     .NOT_THIS_ONE   It's not this one!
                JC      .LAST_CHAR      It's the last character of table!
                INC     DPTR            Increment parse pointer
                JMP     .LOOP           Do next character

.LAST_CHAR      MOV     DPSEL,#3        Next character must be a blank
                MOVX    A,@DPTR
                CJNE    A,#' ',.NEXT1   It's not!
                SJMP    .FOUND_STRING   Found string!
.NEXT1          CJNE    A,#',',.NEXT2   Or a comma!
                SJMP    .FOUND_STRING   It was a comma
.NEXT2          CJNE    A,#CR,.NEXT3    Or a CR!
                SJMP    .FOUND_STRING   It was a CR
.NEXT3          INC     R0              Increment string counter
                MOV     DPSEL,#0
                INC     DPTR            Point to begin of next string
                JMP     .TRY_NEXT

.NOT_THIS_ONE   INC     R0              Increment string counter
                MOV     DPSEL,#0        Find end of this string
.FIND_END       CLR     A               Get character
                MOVC    A,@A+DPTR       Is this the end?
                INC     DPTR
                JB      ACC.7,.TRY_NEXT Found end of the string!
                SJMP    .FIND_END       Keep looking

.NOT_FOUND      MOV     DPSEL,#0        Always select default DPTR
                CLR     C               Indicate string not found!
                RET

.FOUND_STRING   MOV     PARSE_SAVE,DPH  Save last parse pointer
                MOV     PARSE_SAVE+1,DPL
                MOV     DPSEL,#0        Select default DPTR
                SETB    C               Indicate string is found
                RET

;---------------------------------------------------------------------------
;   TABLE    :  COMMAND_TABLE
;   FUNCTION :  Table with all SB-Bus commands
;---------------------------------------------------------------------------

COMMAND_TABLE   .AT     /*CATALOG?/
                .AT     /*ERROR?/
                .AT     /*FAST/
                .AT     /*FLOW/
                .AT     /*FLOW?/
                .AT     /*HOLD/
                .AT     /*ID?/
                .AT     /*LOCS/
                .AT     /*REMS/
                .AT     /*RST/
                .AT     /*SLAVE/
                .AT     /*SLOW/
                .AT     /*TRIG/
                .AT     /*TST?/
                .AT     /CALC?/
                .AT     /DISPLAY?/
                .AT     /DUMP?/
                .AT     /FORMAT/
                .AT     /FORMAT?/
                .AT     /FREQ?/
                .AT     /HOLD/
                .AT     /OFFSET/
                .AT     /OFFSET?/
                .AT     /OPTION/
                .AT     /OPTION?/
                .AT     /RATE/
                .AT     /RATE?/
                .AT     /READ?/
                .AT     /REFERENCE/
                .AT     /REFERENCE?/
                .AT     /RESET/
                .AT     /SCALE/
                .AT     /SCALE?/
                .AT     /SPEED/
                .AT     /SPEED?/
                .AT     /SYNC/
                .DA     #0

;---------------------------------------------------------------------------
;   TABLE    :  CMD_ADDRESS
;   FUNCTION :  Pointers to command routines
;---------------------------------------------------------------------------

CMD_ADDRESS     .DA     CATALOG_REQ     *CATALOG?
                .DA     ERROR_REQ       *ERROR?
                .DA     FAST_CMD        *FAST
                .DA     FLOW_CMD        *FLOW
                .DA     FLOW_REQ        *FLOW?
                .DA     HOLD_CMD        *HOLD
                .DA     ID_REQ          *ID?
                .DA     LOCS_CMD        *LOCS
                .DA     REMS_CMD        *REMS
                .DA     RST_CMD         *RST
                .DA     SLAVE_CMD       *SLAVE
                .DA     SLOW_CMD        *SLOW
                .DA     TRIG_CMD        *TRIG
                .DA     TST_REQ         *TST?
                .DA     CALC_REQ        CALC?
                .DA     DISP_REQ        DISPLAY?
                .DA     DUMP_REQ        DUMP?
                .DA     FRMT_CMD        FORMAT
                .DA     FRMT_REQ        FORMAT?
                .DA     FREQ_REQ        FREQ?
                .DA     HOLDREAD_CMD    HOLD
                .DA     OFFSET_CMD      OFFSET
                .DA     OFFSET_REQ      OFFSET?
                .DA     OPTION_CMD      OPTION
                .DA     OPTION_REQ      OPTION?
                .DA     RATE_CMD        RATE
                .DA     RATE_REQ        RATE?
                .DA     DISP_REQ        READ?
                .DA     REF_CMD         REFERENCE
                .DA     REF_REQ         REFERENCE?
                .DA     RESET_CMD       RESET
                .DA     SCALE_CMD       SCALE
                .DA     SCALE_REQ       SCALE?
                .DA     SPEED_CMD       SPEED
                .DA     SPEED_REQ       SPEED?
                .DA     SYNC_CMD        SYNC

;---------------------------------------------------------------------------
;   ROUTINE  :  NEXT_NON_SPACE
;   FUNCTION :  Let DPTR point to next non space
;   OUPUT    :  C=0 not end of line, C=1 end of line
;---------------------------------------------------------------------------

NEXT_NON_SPACE  MOV     DPSEL,#3
                MOVX    A,@DPTR         Get character
                CJNE    A,#' ',.NEXT    Not a space!
                INC     DPL             Increment pointer
                SJMP    NEXT_NON_SPACE  Try next character
.NEXT           CJNE    A,#CR,.NOTEOL   Not end of line!
                SETB    C               Mark end of line
                RET
.NOTEOL         CLR     C               Not end of line
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  SEND_STATUS
;   FUNCTION :  Send status string (=>, ?> or !>), depending on last error
;---------------------------------------------------------------------------

SEND_STATUS     MOV     DPSEL,#0        Setup string pointers
                MOV     DPTR,#.OK_TEXT  In case of no error
                MOV     A,ERROR         What is the last error?
                JZ      .THIS_ONE       It's no error!
                MOV     DPTR,#.SYN_TEXT In case of syntax error
                DEC     A               Was it syntax error?
                JZ      .THIS_ONE       It's syntax error!
                MOV     DPTR,#.EXE_TEXT It must be execute error now
.THIS_ONE       CALL    SEND_STRING     Send this string
                CALL    CLR_SIN         Clear string from input buffer
                MOV     SP,#STACK_BEGIN
                JMP     MAIN            Dirty return to main routine

.OK_TEXT        .AT     /=>/,#CR
.SYN_TEXT       .AT     /?>/,#CR
.EXE_TEXT       .AT     /!>/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  NO_PARAMS
;   FUNCTION :  See if really no parameters are given
;---------------------------------------------------------------------------

NO_PARAMS       MOV     ERROR,#0        Clear error state
                MOV     DPSEL,#3        (Last parse position still here)
                CALL    NEXT_NON_SPACE  See if anything follows
                JC      .RET            No! O.K.!
                MOV     ERROR,#4        Exit with no parameters allowed
                JMP     SEND_STATUS
.RET            MOV     DPSEL,#0
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  NO_MORE_PARS
;   FUNCTION :  No more parameters allowed
;---------------------------------------------------------------------------

NO_MORE_PARS    MOV     ERROR,#0        Start without an error
                CALL    NEXT_NON_SPACE
                JC      .RET            No!
                CJNE    A,#',',.ILLEGAL It's not a comma!
                MOV     ERROR,#6        Exit with too many parameters error
                JMP     SEND_STATUS
.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter
                JMP     SEND_STATUS
.RET            MOV     DPSEL,#0        Select main pointer
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  ACK_WAIT
;   FUNCTION :  Wait for acknowledge if it's enabled
;               R2 is used to count the acknowledge errors
;---------------------------------------------------------------------------

ACK_WAIT        JNB     FLOW_MODE,.XOFF Xon/Xoff, so don't clear input bffr
.CLR_SIN        CALL    CLR_SIN         Clear string from input buffer
                JNZ     .CLR_SIN        Not all strings done!
.XOFF           JB      STOP_ACT,QUIT   Stop all activities!
                >PUSH   B0_R2,DPL,DPH
                CALL    DO_EVENTS       Service other routines
                >POP    DPH,DPL,B0_R2
                MOV     A,SOUT_HEAD     Output buffer empty yet?
                XRL     A,SOUT_TAIL
                JNZ     .XOFF           Not yet!
                JB      FLOW_MODE,.ACKN Acknowledge flow mode active!
                SETB    C
                RET
.ACKN           JB      STOP_ACT,QUIT   Stop all activities!
                >PUSH   B0_R2,DPL,DPH
                CALL    DO_EVENTS       Service other routines
                >POP    DPH,DPL,B0_R2
                MOV     A,CR_COUNT      Is a CR received already?
                JZ      .ACKN           Not yet!
                MOV     DPSEL,#3        Get pointer in input buffer
                MOV     DPL,SIN_TAIL
                CALL    NEXT_NON_SPACE  Get 1st non space character
                JC      .ABORT          No acknowledge! Abort
                CALL    GET_UC          Get the 1st character
                CJNE    A,#'=',.NOT_OK  Not OK status!

                CALL    NEXT_NON_SPACE  No more characters allowed
                JNC     .ABORT          There are more!
                MOV     DPSEL,#0
                SETB    C               Indicate that everything's O.K.
                RET

.NOT_OK         CJNE    A,#'!',.NOT_ACK Not acknowledge error!
                SJMP    .ACK_ERR
.NOT_ACK        CJNE    A,#'?',.ABORT   Not syntax error!

.ACK_ERR        CALL    NEXT_NON_SPACE  No more characters allowed
                JNC     .ABORT          There are more!
                INC     R2              Increment acknowledge error
                MOV     A,R2            Maximum number of errors found?
                >CMP    A,#MAX_ACK_ERRORS+1
                JNC     .MAX_ERROR      Maximum number of errors found!
                MOV     DPSEL,#0
                CLR     C               Exit with abort error
                RET

.ABORT          MOV     ERROR,#3        Exit with aborted error
                JMP     SEND_STATUS

.MAX_ERROR      MOV     ERROR,#8        Exit with too many errors
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  QUIT
;   FUNCTION :  Quit because of a stop activity command
;---------------------------------------------------------------------------

QUIT            JNB     SB_SELECT,.MAIN Not selected anymore! Nothing to do
                MOV     A,LAST_SEND     Stopped by a received address?
                INC     A
                JZ      .MAIN           Yes! Nothing to do
                CLR     EA              Temporary disable interrupts
                MOV     SIN_HEAD,#0     Clear output buffer
                MOV     SIN_TAIL,#0
                MOV     SOUT_HEAD,#0
                MOV     SOUT_TAIL,#0
                MOV     CR_COUNT,#0
                SETB    EA              Allow them again
                CJNE    A,#CR+1,.DO_CR  Last char was not a CR
                SJMP    .CR_DONE        Last char was a CR!
.DO_CR          MOV     A,#CR           Send an extra CR
                CALL    COUT
.CR_DONE        MOV     ERROR,#3        Exit with aborted error
                JMP     SEND_STATUS     Quit everything
.MAIN           JMP     MAIN

;---------------------------------------------------------------------------
;   ROUTINE  :  TEST_ERROR
;   FUNCTION :  Test last error results
;---------------------------------------------------------------------------

TEST_ERROR      MOV     A,ERROR         Did an error occur?
                JZ      .RET            No! OK
                CJNE    A,#3,.FAIL1     Others than Aborted error not allowed!
.RET            MOV     ERROR,#0        Clear last error
                RET
.FAIL1          CJNE    A,#8,.FAIL2     Too many errors is last allowed error
                SJMP    .RET
.FAIL2          JMP     SEND_STATUS     Send error status again

;---------------------------------------------------------------------------
;   ROUTINE  :  TEST_HOLD
;   FUNCTION :  Test HOLD mode status
;---------------------------------------------------------------------------

TEST_HOLD       JNB     HOLD_ACTIVE,.RET Is hold mode active?
                MOV     A,HOLD_FUNCTION  Is hold function loaded yet?
                ORL     A,HOLD_FUNCTION+1
                JNZ     .HOLD_ERROR      Yes! Error! Can't have 2 in hold
                POP     HOLD_FUNCTION    Don't execute command now, but put
                POP     HOLD_FUNCTION+1   it's addr. in hold function
                JMP     SEND_STATUS      Exit without error
.HOLD_ERROR     MOV     ERROR,#9         Exit with hold mode deactivated
                CLR     HOLD_ACTIVE       and realy deactivate it
                MOV     HOLD_FUNCTION,#0
                MOV     HOLD_FUNCTION+1,#0
                JMP     SEND_STATUS
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE  :  REP_ERROR
;   FUNCTION :  There is nothing to repeat
;---------------------------------------------------------------------------

REP_ERROR       MOV     ERROR,#2        Exit with repeat error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  CATALOG_REQ
;   FUNCTION :  Request command catalog
;---------------------------------------------------------------------------

CATALOG_REQ     LCALL   NO_PARAMS       No parameters allowed here!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.THE_END Not if selected generally
                MOV     DPTR,#COMMAND_TABLE Setup pointer to command table
.CAT_LOOP       MOV     SAVE_TEXT,DPH   Save current text pointer in case
                MOV     SAVE_TEXT+1,DPL  we have to repeat the string
                MOV     R2,#0           Clear error counter
                >GETC
                JZ      .THE_END        End of table reached!
.RETRY          CALL    SEND_STRING     Send the string
                MOV     A,#CR           Terminate with a CR
                CALL    COUT
                CALL    ACK_WAIT        Wait for acknowledge
                JC      .CAT_LOOP       Acknowledged OK!
                MOV     DPH,SAVE_TEXT   Repeat this string for it went wrong
                MOV     DPL,SAVE_TEXT+1
                JMP     .RETRY
.THE_END        JMP     SEND_STATUS     Exit without error

;---------------------------------------------------------------------------
;   ROUTINE  :  ERROR_REQ
;   FUNCTION :  Request last error
;---------------------------------------------------------------------------

ERROR_REQ       LCALL   .PARAMETERS     Handle parameter fetch different
                LCALL   TEST_ERROR      Test execution error
                NOP                     Skip HOLD test
                NOP
                NOP
                JB      SB_GENERAL,.SKIP Nothing to do if general
                MOV     A,PARAMETERS    Get last error number
                MOV     R0,A
                MOV     DPTR,#ERROR_TEXT Setup error text pointer
.LOOP           MOV     A,R0            This string?
                JZ      .FOUND_STRING   No!
                DEC     R0              Decrement string counter
.FIND_NEXT      >GETC                   ; find end of this string
                INC     DPTR
                JNB     ACC.7,.FIND_NEXT Not end of string yet
                JMP     .LOOP           Try this one!
.FOUND_STRING   CALL    SEND_STRING     Send the found string
.SKIP           JMP     SEND_STATUS
.PARAMETERS     MOV     PARAMETERS,ERROR Make copy of original error #
                JMP     NO_PARAMS       No parameters allowed

;---------------------------------------------------------------------------
;   ROUTINE  :  FAST_CMD
;   FUNCTION :  Fast command
;---------------------------------------------------------------------------

FAST_CMD        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                CLR     SLOW_MODE       Select fast mode
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
;   ROUTINE  :  FLOW_CMD
;   FUNCTION :  Setup flow control mode
;---------------------------------------------------------------------------

FLOW_CMD        LCALL   .PARAMETER      Get parameter
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     A,PARAMETERS    Get parameter and store it
                MOV     C,ACC.0
                MOV     FLOW_MODE,C
                JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /XOFF/
                .AT     /ACK/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  FLOW_REQ
;   FUNCTION :  Flow mode request
;---------------------------------------------------------------------------

FLOW_REQ        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.XON_XOFF  In case of Xon/Xoff mode
                JNB     FLOW_MODE,.SEND It is Xon/Xoff
                MOV     DPTR,#.ACKNOWLEDGE It must be acknowledge mode now
.SEND           CALL    SEND_STRING
.END            JMP     SEND_STATUS     Exit with no error

.XON_XOFF       .AT     \XON/XOFF\,#CR
.ACKNOWLEDGE    .AT     /ACKNOWLEDGE/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  HOLD_CMD
;   FUNCTION :  Start hold mode
;---------------------------------------------------------------------------

HOLD_CMD        LCALL   NO_PARAMS       No parameters allowed here!
                LCALL   TEST_ERROR      Test previous errors
                JB      SB_GENERAL,.NO_ERROR Ignore command if general
                JB      HOLD_ACTIVE,.HOLD_ERROR Hold already activated!
                MOV     HOLD_FUNCTION,#0 Clear hold function pointer
                MOV     HOLD_FUNCTION+1,#0
                SETB    HOLD_ACTIVE     Activate Hold function
.NO_ERROR       JMP     SEND_STATUS     Exit with no error

.HOLD_ERROR     MOV     ERROR,#9        Exit with hold mode deactivated
                CLR     HOLD_ACTIVE     Clear hold mode too
                MOV     HOLD_FUNCTION,#0
                MOV     HOLD_FUNCTION+1,#0
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  ID_REQ
;   FUNCTION :  ID string request
;---------------------------------------------------------------------------

ID_REQ          LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.ID6668   In case of SB6668
                JNB     PM6667,.PM6668  It is!
                MOV     DPTR,#.ID6667   It's an SB6668
.PM6668         CALL    SEND_STRING     Send 7 or 8 as last digit
.END            JMP     SEND_STATUS     Exit with no error

.ID6667         .AT     /SB-6667 FREQUENCY COUNTER V1.1/,#CR
.ID6668         .AT     /SB-6668 FREQUENCY COUNTER V1.1/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  LOCS_CMD
;   FUNCTION :  Local command
;---------------------------------------------------------------------------

LOCS_CMD        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                CLR     REMOTE_MODE     Select local mode
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
;   ROUTINE  :  REMS_CMD
;   FUNCTION :  Remote command
;---------------------------------------------------------------------------

REMS_CMD        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                SETB    REMOTE_MODE     Select remote mode
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
;   ROUTINE  :  RST_CMD
;   FUNCTION :  Reset command
;---------------------------------------------------------------------------

RST_CMD         LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JMP     RESET           Restart the machine

;---------------------------------------------------------------------------
;   ROUTINE  :  SLAVE_CMD
;   FUNCTION :  Setup new slave address, only during general call
;---------------------------------------------------------------------------

SLAVE_CMD       LCALL   .PARAMETER      Get the parameter
                LCALL   TEST_ERROR      Test previous errors
                LCALL   TEST_HOLD       Test hold mode
                MOV     MY_ADDRESS,PARAMETERS Setup own address
                MOV     NVM_BUFFER,#1   Send my address to the NVM
                MOV     NVM_BUFFER+1,#NVM_MY_ADDRESS
                MOV     NVM_BUFFER+2,PARAMETERS
                CALL    WRITE_NVM
                JNC     .NO_IIC_ERROR   No IC error occurred!
                SETB    IIC_ERROR       Set IIC Error bit
.NO_IIC_ERROR   JMP     SEND_STATUS     Done OK!

.PARAMETER      CALL    NEXT_NON_SPACE  Get new slave address
                JC      .MISSING        Missing parameter error
                CALL    GET_HEX         Is it a hex number?
                JC      .RANGE          Overflow error!
                JNZ     .CHECK_RANGE    Check for range
                CALL    GET_DECIMAL     Get a decimal number
                JC      .RANGE          Overflow error!
                JZ      .ILLEGAL        Illegal parameter error
.CHECK_RANGE    MOV     A,R0            Get value and make it easier to
                ANL     A,#%0111.1111    check the range
                JZ      .RANGE          0 is not allowed
                INC     A
                JB      ACC.7,.RANGE    255 is not allowed
                DEC     A
                DEC     A
                JZ      .RANGE          1 is not allowed
                MOV     A,R0            Make SB-Bus address from it
                ORL     A,#%1000.0000
                MOV     PARAMETERS,A     and save it as parameter
                JMP     NO_MORE_PARS    Nothing may follow now

.RANGE          MOV     ERROR,#12       Exit with range error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  SLOW_CMD
;   FUNCTION :  Slow command
;---------------------------------------------------------------------------

SLOW_CMD        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                SETB    SLOW_MODE       Select slow mode
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
;   ROUTINE  :  TRIG_CMD
;   FUNCTION :  Trigger an in HOLD function
;---------------------------------------------------------------------------

TRIG_CMD        LCALL   NO_PARAMS       No parameters allowed here!
                LCALL   TEST_ERROR      Test previous errors
                JNB     HOLD_ACTIVE,.NO_HOLD Hold mode is not active!
                MOV     A,HOLD_FUNCTION Is hold function available?
                ORL     A,HOLD_FUNCTION+1
                JZ      .HOLD_EMPTY     No! Nothing to do!
                CLR     HOLD_ACTIVE     Clear hold flag
                PUSH    HOLD_FUNCTION+1 Execute the function in hold
                PUSH    HOLD_FUNCTION
                RET

.NO_HOLD        MOV     ERROR,#10       Exit with hold mode not active
                SJMP    .CLR_HOLD

.HOLD_EMPTY     MOV     ERROR,#11       Exit with nothing in hold error

.CLR_HOLD       CLR     HOLD_ACTIVE     Clear hold mode
                MOV     HOLD_FUNCTION,#0
                MOV     HOLD_FUNCTION+1,#0
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  TST_REQ
;   FUNCTION :  *TST? command
;---------------------------------------------------------------------------

TST_REQ         LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.MEM_OK   In case memory is OK
                MOV     NVM_BUFFER,#1   Read option byte from NVM
                MOV     NVM_BUFFER+1,#NVM_OPTIONS
                CALL    READ_NVM        Read the byte
                MOV     A,NVM_BUFFER+2
                JB      ACC.2,.NOT_MEM_LOST Not memory lost!
                MOV     DPTR,#.MEM_LOST Send memory lost string
                MOV     NVM_BUFFER,#1   Set memory lost reported flag
                MOV     NVM_BUFFER+1,#NVM_OPTIONS
                MOV     A,NVM_BUFFER+2
                ORL     A,#%0000.0100
                MOV     NVM_BUFFER+2,A
                CALL    WRITE_NVM       Write it to NVM
                JNC     .NOT_MEM_LOST   No write error occurred!
                SETB    IIC_ERROR       Set IIC error flag
.NOT_MEM_LOST   CALL    SEND_STRING     Send memory status string
                MOV     DPTR,#.IIC_GOOD In case of no IC error
                JNB     IIC_ERROR,.GOOD No error found!
                MOV     DPTR,#.IIC_BAD  IC write errors found
.GOOD           CALL    SEND_STRING
                MOV     A,DOG_BARKED    Say number of watchdog resets
                CALL    SEND_DEC
                MOV     DPTR,#.DOG_TEXT
                CALL    SEND_STRING
                MOV     DOG_BARKED,#0   Reset watchdog counter
.END            JMP     SEND_STATUS     Exit with no error

.MEM_OK         .AT     /NV MEMORY OK/,#CR
.MEM_LOST       .AT     /NV MEMORY LOST/,#CR
.DOG_TEXT       .AT     / WATCHDOG RESETS/,#CR
.IIC_GOOD       .AT     /IIC BUS OK/,#CR
.IIC_BAD        .AT     /IIC BUS FAILED/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  GET_HEX
;   FUNCTION :  Get HEX value from SB-Bus
;   OUTPUT   :  R0 is value, A=0 if not legal number, C=1 if overflow
;---------------------------------------------------------------------------

GET_HEX         MOV     DPSEL,#3        Select serial input buffer pointer
                MOV     R0,#0           Clear result
                MOV     R1,#0           Clear nothing input flag
                MOV     R3,DPL          Save current tail pointer
                CALL    GET_UC          Get 1st character of the line
                CJNE    A,#'$',.NO_DIGITS  It's not a hex digit
.LOOP           MOV     R4,DPL          In case we're too far
                CALL    GET_UC          Get next character
                JC      .END
                CLR     C               Strip ASCII part
                SUBB    A,#'0'
                >CMP    A,#9+1          Was it a digit?
                JC      .09             Yes!
                SUBB    A,#7            It can also be A to F
                >CMP    A,#$F+1
                JNC     .END            It's not!
.09             MOV     R2,A            Save new digit
                MOV     A,R0            Test range of current value
                ANL     A,#%1111.0000    (must be 0 now!)
                JNZ     .OVERFLOW       Exit with overflow
                MOV     A,R0            Combine old value * 16 and new
                SWAP    A                value
                ORL     A,R2
                MOV     R0,A
                INC     R1              Increment digit counter
                SJMP    .LOOP           Do next digit!
.OVERFLOW       SETB    C               Set C as sign of overflow
                MOV     DPL,R3          Restore tail pointer
                RET
.END            MOV     DPL,R4          Backup if not eol
                MOV     A,R1            Get digit counter in A
                JNZ     .RET            There are digits found!
.NO_DIGITS      MOV     DPL,R3          Restore tail pointer
                MOV     A,#0            Get digit counter
.RET            CLR     C               No overflow occurred
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  GET_DECIMAL
;   FUNCTION :  Get a decimal number from input line
;   OUTPUT   :  R0 is value, A=0 if not legal number, C=1 if overflow
;---------------------------------------------------------------------------

GET_DECIMAL     MOV     DPSEL,#3        Select serial input buffer pointer
                MOV     R0,#0           Clear result
                MOV     R1,#0           Clear digit counter
                MOV     R3,DPL          Save tail pointer
.LOOP           MOV     R4,DPL          In case we're too far
                CALL    GET_UC          Get next digit
                JC      .END            End of string
                CLR     C               Strip ASCII part
                SUBB    A,#'0'
                >CMP    A,#9+1          Was it a digit?
                JNC     .END            No! End of number found
                MOV     R2,A            Save new digit
                INC     R1              Increment digit counter
                MOV     A,R0            Multiply previous result by 10
                MOV     B,#10
                MUL     AB
                MOV     R0,A            Save result
                MOV     A,B             See if overflow occurred
                JNZ     .OVERFLOW       Yes! Must remain 0
                MOV     A,R2            Add new digit to result
                ADD     A,R0
                MOV     R0,A
                JC      .OVERFLOW       Overflow occurred
                SJMP    .LOOP
.OVERFLOW       MOV     DPL,R3          Restore tail pointer
                SETB    C               Indicate overflow
                RET
.END            MOV     DPL,R4          Backup if not eol
                MOV     A,R1            Any digit found?
                JNZ     .RET            Yes!
                MOV     DPL,R3          Restore tail pointer
                MOV     A,R1            Clear digit found flag
.RET            CLR     C               No overflow error!
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  FRMT_CMD
;   FUNCTION :  Set the requested format  (1=xxxxxxxkHz, 2=x.xxxxxxE+3)
;---------------------------------------------------------------------------

FRMT_CMD        LCALL   .PARAMETER      Get parameter
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     A,PARAMETERS    Get parameter and store it
                MOV     C,ACC.1
                MOV     FORMAT,C
                JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                CALL    GET_UC          Get the parameter
                CLR     C               Subtract ASCII part of it
                SUBB    A,#'0'
                JZ      .RANGE          Range error!
                >CMP    A,#10           Is it a digit?
                JNC     .ILLEGAL        Illegal parameter
                >CMP    A,#3            Range Ok?
                JNC     .RANGE          No!
                MOV     PARAMETERS,A    Save parameter
                JMP     NO_MORE_PARS    No more parameters please

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.RANGE          MOV     ERROR,#12       Exit with range error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /1/
                .AT     /2/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  FRMT_REQ
;   FUNCTION :  Request current format setting
;---------------------------------------------------------------------------

FRMT_REQ        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                CLR     A
                MOV     C,FORMAT        Get format number
                ADDC    A,#'1'          Add carry to the ASCII value
                CALL    COUT             and send it to the SB-Bus
                MOV     A,#CR
                CALL    COUT
.END            JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
;   ROUTINE  :  FREQ_REQ
;   FUNCTION :  Request for the measured frequency (not modified)
;---------------------------------------------------------------------------

FREQ_REQ        LCALL   .PARAMETER      See if a parameter follows
                LCALL   .ERRORS         Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#HOLD_FREQ In case hold value is requested
                MOV     A,PARAMETERS    See if hold value requested
                JNZ     .HOLD           Yes!
                JB      MENU_MODE,.MENU_ERROR  Can't measure if menu active!
                MOV     DPTR,#SB_FREQUENCY Setup pointer to the value
.HOLD           CALL    SEND_FREQ       Send frequency to SB-Bus
                MOV     A,#CR
                CALL    COUT
.END            JMP     SEND_STATUS     Exit with no error

.ERRORS         MOV     A,ERROR         See if last error was 13
                XRL     A,#13
                JZ      .RET            Yes! May be OK now
                JMP     TEST_ERROR      Do normal error test
.RET            MOV     ERROR,#0        Clear error
                RET

.MENU_ERROR     MOV     ERROR,#13       Exit with menu error
                JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     PARAMETERS,#0   In case no parameter given
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .RET            No parameter given!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                INC     A
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /H/
                .AT     /HOLD/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  CALC_REQ
;   FUNCTION :  Request for calculation method
;---------------------------------------------------------------------------

CALC_REQ        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.DISP_TEXT Send 1st part of string
                CALL    SEND_STRING
                JB      FUNC_PRIORITY,.SCALE1ST Scaling first!
                JB      OFFSET_ACTIVE,.OFFSET   Offset is active
                MOV     DPTR,#.FREQ_TEXT        Simply print frequency
                CALL    SEND_STRING
                SJMP    .SKIP_OFFS
.OFFSET         JB      SCALE_ACTIVE,.OFFS_SCALE Both offset and scale!
                MOV     DPTR,#.FREQ_TEXT Send freq+/-offset
                CALL    SEND_STRING
                CALL    .SEND_OFFSET    Send freq+/-offset
                SJMP    .DONE
.OFFS_SCALE     MOV     A,#'('          Print (freq+/-offset)
                CALL    COUT
                MOV     DPTR,#.FREQ_TEXT
                CALL    SEND_STRING
                CALL    .SEND_OFFSET
                MOV     A,#')'
                CALL    COUT
.SKIP_OFFS      JNB     SCALE_ACTIVE,.DONE No scale needed!
                CALL    .SEND_SCALE     Send scaling
.DONE           MOV     A,#CR           Terminate string with CR
                CALL    COUT
.END            JMP     SEND_STATUS     Exit with no error

.SCALE1ST       JB      SCALE_ACTIVE,.SCALE Scale is active!
                MOV     DPTR,#.FREQ_TEXT    Simply print Frequency
                CALL    SEND_STRING
                SJMP    .SKIP_SCALE
.SCALE          JB      OFFSET_ACTIVE,.SCAL_OFFS Do both scale and offset
                MOV     DPTR,#.FREQ_TEXT Print frequency*//scale
                CALL    SEND_STRING
                CALL    .SEND_SCALE
                SJMP    .DONE
.SCAL_OFFS      MOV     A,#'('          Print (FREQ*//SCALE)
                CALL    COUT
                MOV     DPTR,#.FREQ_TEXT
                CALL    SEND_STRING
                CALL    .SEND_SCALE
                MOV     A,#')'
                CALL    COUT
.SKIP_SCALE     JNB     OFFSET_ACTIVE,.DONE No offset given
                CALL    .SEND_OFFSET
                SJMP    .DONE

.SEND_OFFSET    MOV     A,#'+'          In case of positive offset
                JNB     OFFSET_POL,.POS1 It is positive
                MOV     A,#'-'          It must be negative now
.POS1           CALL    COUT
                MOV     DPTR,#.OFFS_TEXT
                CALL    SEND_STRING
                RET

.SEND_SCALE     MOV     A,#'*'          In case of multiplication
                JNB     SCALE_POL,.MUL1 It is multiplication
                MOV     A,#'/'          It must be division now
.MUL1           CALL    COUT
                MOV     DPTR,#.SCAL_TEXT
                CALL    SEND_STRING
                RET

.DISP_TEXT      .AT     /DISPLAY=/
.FREQ_TEXT      .AT     /FREQUENCY/
.OFFS_TEXT      .AT     /OFFSET/
.SCAL_TEXT      .AT     /SCALE/

;---------------------------------------------------------------------------
;   ROUTINE  :  DISP_REQ
;   FUNCTION :  Request for the displayed value (modified)
;---------------------------------------------------------------------------

DISP_REQ        LCALL   .PARAMETER      Only the HOLD parameter allowed!
                LCALL   .ERRORS         Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#HOLD_DISP In case the hold value is requested
                MOV     A,PARAMETERS    Is the hold value requested?
                JNZ     .HOLD_DISP      Yes!
                JB      MENU_MODE,.MENU_ERROR  Can't measure if menu active!
                MOV     DPTR,#SB_DISPLAY Setup pointer to the value
.HOLD_DISP      CALL    SEND_FREQ       Send frequency to SB-Bus
                JNB     SPEED_FUNCTION,.CROUT  Not speed function!
                CALL    SEND_PERCENT    Send percentage to SB-Bus
.CROUT          MOV     A,#CR           Send terminating CR
                CALL    COUT
.END            JMP     SEND_STATUS     Exit with no error

.ERRORS         MOV     A,ERROR         See if last error was 13
                XRL     A,#13
                JZ      .RET            Yes! May be OK now
                JMP     TEST_ERROR      Do normal error test
                MOV     ERROR,#0        Clear error
.RET            RET

.MENU_ERROR     MOV     ERROR,#13       Exit with menu error
                JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     PARAMETERS,#0   In case no parameter given
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .RET            No parameter given!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                INC     A
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /H/
                .AT     /HOLD/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  SEND_PERCENT
;   FUNCTION :  Send the speed deviation percentage to the SB-Bus
;---------------------------------------------------------------------------

SEND_PERCENT    MOV     A,#','          Send comma, followed by percentage
                CALL    COUT
                MOV     DPTR,#HOLD_PERC In case the hold value is requested
                MOV     A,PARAMETERS    Is the hold value requested?
                JNZ     .HOLD_PERC      Yes!
                MOV     DPTR,#PERCENT_VALUE  Get the percentage value
.HOLD_PERC      MOV     R0,#FP_ACCU_A
                MOV     R2,#7           Copy 7 bytes
.GET_VALUE      MOVX    A,@DPTR         Get byte
                MOV     @R0,A            and copy it
                INC     DPTR            Increment pointers
                INC     R0
                DJNZ    R2,.GET_VALUE   Do all 7!
                MOV     A,FP_ACCU_A+5   Get sign of percentage
                MOV     C,ACC.7
                CLR     ACC.7
                MOV     FP_ACCU_A+5,A
                MOV     A,#'+'          In case it's positive!
                JC      .EXP_POS        It is positive!
                MOV     A,#'-'
.EXP_POS        CALL    COUT
                MOV     A,FP_EXP_A      Exponent must be <=1
                MOV     C,ACC.6         Sign extend to erase 6/7 digit
                MOV     ACC.7,C          flag
                JB      ACC.7,.EXP_OK   It is! (for it's negative)
                >CMP    A,#1+1
                JNC     .PERC_EXP_OL    It's not! It's too high
.EXP_OK         DEC     A               Was exponent 1?
                CLR     F0
                JZ      .PERC_EXP_1     Yes! Show Tens and Units then
                INC     A               Was exponent 0?
                SETB    F0
                JZ      .PERC_EXP_1     Yes! Show Units and Tenths
                INC     A               Was exponent -1? (0.x)
                JNZ     .PERC_EXP_0     No! Number is too small show 0.0
                MOV     A,FP_ACCU_A+5   Shift MSD 1 position right
                SWAP    A
                MOV     FP_ACCU_A+4,A
                MOV     FP_ACCU_A+5,#0
                SJMP    .PERC_EXP_1     Skip 0 load
.PERC_EXP_0     MOV     FP_ACCU_A+5,#0  Show 0.0
                MOV     FP_ACCU_A+4,#0
.PERC_EXP_1     MOV     A,FP_ACCU_A+5   Get MSD and send it to the bus
                ORL     A,#'0'
                CALL    COUT
                JNB     F0,.NO_POINT    No decimal point needed!
                MOV     A,#'.'          Send decimal point
                CALL    COUT
.NO_POINT       MOV     A,FP_ACCU_A+4   Get LSD and send it out
                SWAP    A
                ANL     A,#%0000.1111
                ORL     A,#'0'
                CALL    COUT
                JB      FORMAT,.RET2    Don't show % on format 2
                MOV     A,#'%'
                CALL    COUT
.RET2           RET

.PERC_EXP_OL    JB      FORMAT,.FORMAT2 Show 99 on format 2
                MOV     A,#'O'          Show OL on format 1
                CALL    COUT
                MOV     A,#'L'
                CALL    COUT
                RET
.FORMAT2        MOV     A,#'9'
                CALL    COUT
                CALL    COUT
                RET

;---------------------------------------------------------------------------
;   ROUTINE  :  DUMP_REQ
;   FUNCTION :  Dump every modified reading to the SB-Bus until cancelled
;---------------------------------------------------------------------------

DUMP_REQ        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   .ERRORS         Test previous errors
                JB      SB_GENERAL,.RET Don't do anything if general
.AGAIN          JB      MENU_MODE,.MENU_ERROR  Can't measure if menu active!

                SETB    SYNC_FLAG       Wait until this flag is clear again
.LOOP           CALL    DO_EVENTS       Keep doing all other things
                JB      MENU_MODE,.MENU_ERROR  Don't wait if user is in menu
                JB      STOP_ACT,.ABORT ESC is pressed!
                JB      SYNC_FLAG,.LOOP Flag still set!

                MOV     DPTR,#SB_DISPLAY Setup pointer to the value
                CALL    SEND_FREQ       Send frequency to SB-Bus
                JNB     SPEED_FUNCTION,.CROUT  Not speed function!
                CALL    SEND_PERCENT    Send percentage to SB-Bus
.CROUT          MOV     A,#CR           Send terminating CR
                CALL    COUT
                JMP     .AGAIN          Keep doing this, until aborted

.ERRORS         MOV     A,ERROR         See if last error was 13
                XRL     A,#13            (menu error)
                JZ      .RET            Yes! May be OK now
                MOV     A,ERROR         See if last error was 3
                XRL     A,#3             (aborted error)
                JZ      .RET            Yes! May be OK now
                JMP     TEST_ERROR      Do normal error test
                MOV     ERROR,#0        Clear error
.RET            RET

.ABORT          MOV     ERROR,#3        Exit with aborted error
                JMP     SEND_STATUS

.MENU_ERROR     MOV     ERROR,#13       Exit with menu error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  HOLDREAD_CMD
;   FUNCTION :  HOLD command, copy current values to hold values
;---------------------------------------------------------------------------

HOLDREAD_CMD    LCALL   NO_PARAMS       No parameters allowed!
                LCALL   .TEST_ERROR     Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      MENU_MODE,.MENU_ACTIVE  Can't hold if menu's active!
                MOV     DPH,/SB_FREQUENCY  Setup high byte of pointer
                MOV     R0,#SB_FREQUENCY   Setup low byte of source
                MOV     R1,#HOLD_FREQ      Setup low byte of destination
                MOV     R2,#7              Set number of bytes to copy
                CALL    .COPY              Copy this value
                MOV     R0,#SB_DISPLAY     Setup low byte of source
                MOV     R1,#HOLD_DISP      Setup low byte of destination
                MOV     R2,#7              Set number of bytes to copy
                CALL    .COPY              Copy this value
                MOV     R0,#PERCENT_VALUE  Setup low byte of source
                MOV     R1,#HOLD_PERC      Setup low byte of destination
                MOV     R2,#7              Set number of bytes to copy
                CALL    .COPY
                JMP     SEND_STATUS     Exit with no error

.COPY           MOV     DPL,R0          Get source byte
                MOVX    A,@DPTR
                MOV     DPL,R1          Copy to destination
                MOVX    @DPTR,A
                INC     R0              Increment pointers
                INC     R1
                DJNZ    R2,.COPY        Do all bytes
                RET

.TEST_ERROR     MOV     A,ERROR         Was Menu Active the last error?
                CJNE    A,#13,.TEST_REST No!
                MOV     ERROR,#0        Clear last error
                RET                     Maybe menu's not active anymore
.TEST_REST      JMP     TEST_ERROR

.MENU_ACTIVE    MOV     ERROR,#13       Exit with menu active error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  OFFSET_CMD
;   FUNCTION :  Set offset or change NVM offset value
;---------------------------------------------------------------------------

OFFSET_CMD      LCALL   GET_OFFS_PAR    Get parameters
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     DPSEL,#0        Select main DP bank
                MOV     A,PARAMETERS    Get function parameter
                JNZ     .NOT_POS_OFFSET Not a positive offset
                CLR     OFFSET_POL      Indicate positive offset
.COPY_VALUE     MOV     R0,#PARAMETERS+1 Copy value to offset value
                MOV     DPTR,#OFFSET_VALUE
                MOV     R2,#5           Do 5 bytes
                MOV     A,PARAMETERS+4  Is the value 0?
                JZ      .VALUE_0        Yes!
.COPY_VLOOP     MOV     A,@R0
                MOVX    @DPTR,A
                INC     R0
                INC     DPTR
                DJNZ    R2,.COPY_VLOOP  Do all bytes
                SETB    OFFSET_ACTIVE   Indicate offset is active
                JB      SCALE_ACTIVE,.DONE Don't change priority!
                CLR     FUNC_PRIORITY   Offset has priority!
.DONE           JMP     SEND_STATUS
.VALUE_0        CLR     OFFSET_ACTIVE   Offset no longer active
                JNB     SCALE_ACTIVE,.DONE Scale's not active either
                SETB    FUNC_PRIORITY   Scale has now priority!
                JMP     SEND_STATUS


.NOT_POS_OFFSET DEC     A               See if negative offset
                JNZ     .NOT_NEG_OFFSET It's not!
                SETB    OFFSET_POL      Indicate negative offset
                SJMP    .COPY_VALUE     Set the offset

.NOT_NEG_OFFSET DEC     A               See if S parameter
                JNZ     .DELETE         It's not! Must be delete now
                MOV     A,PARAMETERS+4  Is the value 0?
                JZ      .RANGE          Yes! Not allowed here
                MOV     A,PARAMETERS+5  Set b7 of exponent to indicate
                PUSH    ACC              offset value
                SETB    ACC.7
                MOV     PARAMETERS+5,A
                CALL    INS_VALUE       Insert value to memory
                POP     PARAMETERS+5
                CALL    SORT_VALUES     Sort the values
                JMP     SEND_STATUS

.DELETE         MOV     A,PARAMETERS+4  Is the value 0?
                JZ      .RANGE          Yes! Not allowed here
                MOV     A,PARAMETERS+5  Set b7 of exponent to indicate
                PUSH    ACC              offset value
                SETB    ACC.7
                MOV     PARAMETERS+5,A
                CALL    DEL_VALUE       Delete value from memory
                POP     PARAMETERS+5    Restore original parameter value
                JMP     SEND_STATUS

.RANGE          MOV     ERROR,#12       Exit with range error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  GET_SCAL_PAR
;   FUNCTION :  Get Scale parameter
;               parameter   =>  Set new scale value (including polarity)
;               S,parameter =>  Save scale value behind comma
;               D,parameter =>  Delete scale value behind comma
;---------------------------------------------------------------------------

GET_SCAL_PAR    MOV     ERROR,#0        Clear error flag
                MOV     PARAMETERS,#0   Clear parameter flag
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     R1,DPL          Save begin pointer
                CALL    GET_UC          Get 1st character
                CJNE    A,#'*',.NOTMUL  It's not a multiplication sign!
                SJMP    .GET_NUM        Next must follow a number!
.NOTMUL         INC     PARAMETERS      It can be a division sign now
                CJNE    A,#'/',.NOTDIV  It's not a division sign
                SJMP    .GET_NUM        Next must follow a number!
.NOTDIV         INC     PARAMETERS      It can be an S
                CJNE    A,#'S',.NOTS    It's not an S!
                SJMP    .COMMA          Next must be a comma!
.NOTS           INC     PARAMETERS      It can be a D
                CJNE    A,#'D',.ILLEGAL It's not a D!
.COMMA          CALL    GET_UC          Next character must be a comma!
                CJNE    A,#',',.ILLEGAL It's not!
                CALL    GET_OFFS_SCAL   Get a floating point number
                MOV     A,PARAMETERS+4  Value of 0 is not allowed!
                JZ      .RANGE          It is 0! (Avoid division by 0)
                RET

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS
.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.GET_NUM        CALL    GET_OFFS_SCAL   Get an FP value
                MOV     A,PARAMETERS+5  Check value range
                >CMP    A,#6+1          Valid range is EXP0 to EXP6
                JNC     .RANGE          It's out of range!
                MOV     A,PARAMETERS+4
                JZ      .RANGE          But value 0 is not allowed!
                RET

.RANGE          MOV     ERROR,#12       Exit with range error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  GET_OFFS_PAR
;   FUNCTION :  Get Offset parameter
;               parameter   =>  Set new offset value (including sign)
;               S,parameter =>  Save offset value behind comma
;               D,parameter =>  Delete offset value behind comma
;---------------------------------------------------------------------------

GET_OFFS_PAR    MOV     ERROR,#0        Clear error flag
                MOV     PARAMETERS,#0   Clear parameter flag
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     R1,DPL          Save begin pointer
                CALL    GET_UC          Get 1st character
                CJNE    A,#'+',.NOTPLUS It's not a plus sign!
                SJMP    .GET_NUM        Next must follow a number!
.NOTPLUS        INC     PARAMETERS      It can be a minus sign now
                CJNE    A,#'-',.NOTMIN  It's not a minus sign
                SJMP    .GET_NUM        Next must follow a number!
.NOTMIN         INC     PARAMETERS      It can be an S
                CJNE    A,#'S',.NOTS    It's not an S!
                SJMP    .COMMA          Next must be a comma!
.NOTS           INC     PARAMETERS      It can be a D
                CJNE    A,#'D',.NOTD    It's not a D!
.COMMA          CALL    GET_UC          Next character must be a comma!
                CJNE    A,#',',.ILLEGAL It's not!
                SJMP    .GET_NUM        Get a floating point number
.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS
.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS
.NOTD           MOV     PARAMETERS,#0   Default value polarity is +
                MOV     DPL,R1          Decrement input pointer

.GET_NUM        CALL    GET_OFFS_SCAL   Get an FP value
                MOV     A,PARAMETERS+5  Check value range
                JB      ACC.7,.NEG_EXP  It's a negative exponent
                >CMP    A,#10+1         Max value is 99.99999GHz
                JNC     .RANGE          Range is not OK!
                RET
.NEG_EXP        >CMP    A,#-3           Min value is 1mHz
                JC      .RANGE          It's not OK!
                MOV     A,PARAMETERS    See if value is 0
                ORL     A,PARAMETERS+1
                ORL     A,PARAMETERS+2
                ORL     A,PARAMETERS+3
                ORL     A,PARAMETERS+4
                JZ      .RANGE          Value 0 is not allowed!
                RET

.RANGE          MOV     ERROR,#12       Exit with range error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  GET_OFFS_SCAL
;   FUNCTION :  Get Offset or Scale value
;---------------------------------------------------------------------------

GET_OFFS_SCAL   CALL    GET_FP_VALUE    Get an FP value!
                MOV     R0,#FP_ACCU_C+2 Copy FP accu to parameter table
                MOV     R1,#PARAMETERS+1
                MOV     R2,#5           Do 5 bytes
.COPY_PAR       MOV     A,@R0
                MOV     @R1,A
                INC     R0
                INC     R1
                DJNZ    R2,.COPY_PAR    Do all bytes
                JMP     NO_MORE_PARS    No more parameters may follow

;---------------------------------------------------------------------------
;   ROUTINE  :  OFFSET_REQ
;   FUNCTION :  Request current Offset setting or list all Offset values
;               if an L parameter follows the OFFSET? command.
;---------------------------------------------------------------------------

OFFSET_REQ      LCALL   .PARAMETER      See if the L parameter is given
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     A,PARAMETERS    See if parameter is given
                JNZ     .LIST           Do a list of all Offset values
                JB      OFFSET_ACTIVE,.SHOW_OFFSET Offset mode is active!
                MOV     DPTR,#.NO_OFFSET_TXT Send no offset mode text
                CALL    SEND_STRING
                JMP     SEND_STATUS
.SHOW_OFFSET    MOV     A,#'+'          In case of addition
                JNB     OFFSET_POL,.ADD It is!
                MOV     A,#'-'          It must be subtraction now
.ADD            CALL    COUT            Send this character to the bus
                MOV     DPTR,#OFFSET_VALUE Copy offset value to FP Accu C
                MOV     R0,#FP_ACCU_C   Copy value to FP Accu C
                CALL    MEM_2_FP_ACCU
                CALL    SEND_OFFSET     Send offset frequency to the bus
.END            JMP     SEND_STATUS     Exit with no error

.LIST           MOV     DPTR,#VALUE_TABLE  Point to value table
                MOV     R6,#50          Do a total of 50 values
.LIST_LOOP      MOV     R2,#0           Clear acknowledge error counter
.LIST_AGAIN     MOV     R0,#FP_ACCU_C   Copy value to FP Accu C
                CALL    MEM_2_FP_ACCU
                JNC     .LIST_SKIP      It's not a Offset value!
                JB      ACC.6,.LIST_SKIP It's not a value (empty memory)
                >PUSH   DPL,DPH,B0_R2
                CALL    SEND_OFFSET     Print this value
                >POP    B0_R2,DPH,DPL
                >PUSH   B0_R6           Save loop counter
                CALL    ACK_WAIT        Wait for acknowledge
                >POP    B0_R6
                JC      .LIST_SKIP      Acknowledge was OK!
                CLR     C
                MOV     A,DPL           Subtract 5 from DPTR to print
                SUBB    A,#5             the value again
                MOV     DPL,A
                MOV     A,DPH
                SUBB    A,#0
                MOV     DPH,A
                SJMP    .LIST_AGAIN
.LIST_SKIP      DJNZ    R6,.LIST_LOOP   Try all 50 values!
                JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     PARAMETERS,#0   Clear parameter flag
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                INC     A               Make parameter <> 0
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!
.MISSING        RET                     No parameter given!

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /L/             List parameter
                .AT     /LIST/          Can also be spelled out completely
                .DA     #0

.NO_OFFSET_TXT  .AT     /NOT ACTIVE/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  OPTION_CMD
;   FUNCTION :  Set options in NVM
;---------------------------------------------------------------------------

OPTION_CMD      LCALL   .PARAMETER      Get parameter
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     NVM_BUFFER,#1   Read 1 byte from NVM
                MOV     NVM_BUFFER+1,#NVM_OPTIONS    Read option byte
                CALL    READ_NVM
                DEC     NVM_BUFFER+1    Point to NVM_OPTIONS again
                MOV     A,PARAMETERS    Get parameter and store it
                CJNE    A,#4,.STORE      if it's not RESET!
                MOV     NVM_BUFFER,#0   Clear 1st byte in NVM memory
                MOV     NVM_BUFFER+1,#1  to delete the system password
                MOV     NVM_BUFFER+2,#$FF
                CALL    WRITE_NVM       Erase the password
                JMP     RESET           Restart the system
.STORE          MOV     C,ACC.0
                JB      ACC.1,.REF      Set/clr reference option
                MOV     PM6667,C        Set/clr PM6667/PM6668 flag
                MOV     A,NVM_BUFFER+2  Get options byte
                MOV     ACC.0,C          and set current option
                MOV     NVM_BUFFER+2,A
                CALL    WRITE_NVM
                JMP     SEND_STATUS     Done
.REF            MOV     OPTION_EXT_REF,C Set/clr external reference option
                MOV     A,NVM_BUFFER+2  Get options byte
                MOV     ACC.1,C          and set current option
                MOV     NVM_BUFFER+2,A
                CALL    WRITE_NVM
.CONTINUE       JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more parameters are allowed!

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /SB6668/
                .AT     /SB6667/
                .AT     /EXTREF/
                .AT     /NOEXTREF/
                .AT     /RESETNVM/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  OPTION_REQ
;   FUNCTION :  Show current options
;---------------------------------------------------------------------------

OPTION_REQ      LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.PM6667   In case it's a PM6667
                JB      PM6667,.SKIP1   It is!
                MOV     DPTR,#.PM6668   It must be a PM6668 now
.SKIP1          CALL    SEND_STRING
                MOV     DPTR,#.EXTREF   In case there's an external ref
                JNB     OPTION_EXT_REF,.SKIP2 It is!
                MOV     DPTR,#.NOEXTREF There's no external reference
.SKIP2          CALL    SEND_STRING
.END            JMP     SEND_STATUS     Exit with no error

.PM6667         .AT     /SB6667/,#CR
.PM6668         .AT     /SB6668/,#CR
.EXTREF         .AT     /EXTREF/,#CR
.NOEXTREF       .AT     /NOEXTREF/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  RATE_CMD
;   FUNCTION :  Set the requested measuring rate (fast or slow)
;---------------------------------------------------------------------------

RATE_CMD        LCALL   .PARAMETER      Get parameter
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     A,PARAMETERS    Get parameter and store it
                MOV     C,ACC.0
                MOV     MEASURE_RATE,C
                CLR     ONLY6           Show 7 digits by default
                MOV     A,COUNTER_STAT  Is counter busy?
                CJNE    A,#2,.CONTINUE  No!
                MOV     COUNTER_STAT,#4 Stop counting, and restart
.CONTINUE       JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /SLOW/
                .AT     /FAST/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  RATE_REQ
;   FUNCTION :  Measuring rate request
;---------------------------------------------------------------------------

RATE_REQ        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.SLOW     In case of Slow
                JNB     MEASURE_RATE,.SEND It is Slow
                MOV     DPTR,#.FAST     It must be Fast now
.SEND           CALL    SEND_STRING
.END            JMP     SEND_STATUS     Exit with no error

.SLOW           .AT     /SLOW/,#CR
.FAST           .AT     /FAST/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  REF_CMD
;   FUNCTION :  Set the requested reference source (internal, external)
;---------------------------------------------------------------------------

REF_CMD         LCALL   .PARAMETER      Get parameter
                LCALL   TEST_ERROR      Test errors
                CALL    TEST_HOLD       Test hold mode
                MOV     A,PARAMETERS    Get parameter and store it
                MOV     C,ACC.2
                MOV     REF_SOURCE,C
                CPL     C                and set output pint
                MOV     P3.2,C
.CONTINUE       JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                JB      OPTION_EXT_REF,.NO_EXT No Ext reference available
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.NO_EXT         MOV     ERROR,#18       Exit with no ext reference input
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /I/
                .AT     /INT/
                .AT     /INTERN/
                .AT     /INTERNAL/
                .AT     /E/
                .AT     /EXT/
                .AT     /EXTERN/
                .AT     /EXTERNAL/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  REF_REQ
;   FUNCTION :  Request reference source
;---------------------------------------------------------------------------

REF_REQ         LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.NO_SWITCH In case no reference switch installed
                JB      OPTION_EXT_REF,.SEND   Switch not installed!
                MOV     DPTR,#.INTERNAL  In case of Internal
                JNB     REF_SOURCE,.SEND It is Internal
                MOV     DPTR,#.EXTERNAL  It must be External now
.SEND           CALL    SEND_STRING
.END            JMP     SEND_STATUS     Exit with no error

.INTERNAL       .AT     /INTERNAL/,#CR
.EXTERNAL       .AT     /EXTERNAL/,#CR
.NO_SWITCH      .AT     /NO REFERENCE SWITCH/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  RESET_CMD
;   FUNCTION :  Reset counter command
;---------------------------------------------------------------------------

RESET_CMD       LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     FUNCTION_FLAGS,#0 Clear all function flags
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
;   ROUTINE  :  SCALE_CMD
;   FUNCTION :  Set Scale or change NVM Scale value
;---------------------------------------------------------------------------

SCALE_CMD       LCALL   GET_SCAL_PAR    Get parameters
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     DPSEL,#0        Select main DP bank
                MOV     A,PARAMETERS    Get function parameter
                JNZ     .NOT_MUL        Not a multiplication scale
                CLR     SCALE_POL       Indicate multiplication scale
.COPY_VALUE     MOV     R0,#PARAMETERS+1 Copy value to scale value
                MOV     DPTR,#SCALE_VALUE
                MOV     R2,#5           Do 5 bytes
.COPY_VLOOP     MOV     A,@R0
                MOVX    @DPTR,A
                INC     R0
                INC     DPTR
                DJNZ    R2,.COPY_VLOOP  Do all bytes
                MOV     A,PARAMETERS    See if value is 1
                ORL     A,PARAMETERS+1
                ORL     A,PARAMETERS+2
                ORL     A,PARAMETERS+3
                ORL     A,PARAMETERS+5
                JNZ     .NOT1           It's not!
                MOV     A,PARAMETERS+4
                DEC     A
                JZ      .VALUE_1        It is!
.NOT1           SETB    SCALE_ACTIVE    Indicate scale is active
                JB      OFFSET_ACTIVE,.DONE Don't change priority!
                SETB    FUNC_PRIORITY   Scale has priority!
.DONE           JMP     SEND_STATUS
.VALUE_1        CLR     SCALE_ACTIVE    Offset no longer active
                JNB     OFFSET_ACTIVE,.DONE Offset's not active either
                CLR     FUNC_PRIORITY   Offset has now priority!
                JMP     SEND_STATUS

.NOT_MUL        DEC     A               See if division scale
                JNZ     .NOT_DIV        It's not!
                SETB    SCALE_POL       Indicate division scale
                SJMP    .COPY_VALUE     Set the scale

.NOT_DIV        DEC     A               See if S parameter
                JNZ     .DELETE         It's not! Must be delete now
                CALL    INS_VALUE       Insert value to memory
                CALL    SORT_VALUES     Sort the values
                JMP     SEND_STATUS

.DELETE         CALL    DEL_VALUE       Delete value from memory
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  SCALE_REQ
;   FUNCTION :  Request current Scale setting or list all Scale factors
;               if an L parameter follows the SCALE? command.
;---------------------------------------------------------------------------

SCALE_REQ       LCALL   .PARAMETER      See if the L parameter is given
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     A,PARAMETERS    See if parameter is given
                JNZ     .LIST           Do a list of all Scaling factors
                JB      SCALE_ACTIVE,.SHOW_SCALE Scale mode is active!
                MOV     DPTR,#.NO_SCALE_TXT Send no scale mode text
                CALL    SEND_STRING
                JMP     SEND_STATUS
.SHOW_SCALE     MOV     A,#'*'          In case of multiply
                JNB     SCALE_POL,.MUL  It is!
                MOV     A,#'/'          It must be divide now
.MUL            CALL    COUT            Send this character to the bus
                MOV     DPTR,#SCALE_VALUE  Copy scaling value to FP Accu C
                MOV     R0,#FP_ACCU_C
                CALL    MEM_2_FP_ACCU
                CALL    SCALE_PRINT     Print the scaling value
                MOV     A,#CR           Terminate with CR
                CALL    COUT
.END            JMP     SEND_STATUS     Exit with no error

.LIST           MOV     DPTR,#VALUE_TABLE  Point to value table
                MOV     R6,#50          Do a total of 50 values
.LIST_LOOP      MOV     R2,#0           Clear acknowledge error counter
.LIST_AGAIN     MOV     R0,#FP_ACCU_C   Copy value to FP Accu C
                CALL    MEM_2_FP_ACCU   Get current value from memory
                JC      .LIST_SKIP      It's not a Scale value!
                CALL    SCALE_PRINT     Print this value
                MOV     A,#CR            followed by CR
                CALL    COUT
                >PUSH   B0_R6           Save loop counter
                CALL    ACK_WAIT        Wait for acknowledge
                >POP    B0_R6
                JC      .LIST_SKIP      Acknowledge was OK!
                CLR     C
                MOV     A,DPL           Subtract 5 from DPTR to print
                SUBB    A,#5             the value again
                MOV     DPL,A
                MOV     A,DPH
                SUBB    A,#0
                MOV     DPH,A
                SJMP    .LIST_AGAIN
.LIST_SKIP      DJNZ    R6,.LIST_LOOP   Try all 50 values!
                JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     PARAMETERS,#0   Clear parameter flag
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                INC     A               Make parameter <> 0
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!
.MISSING        RET                     No parameter given!

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /L/             List parameter
                .AT     /LIST/          Can also be spelled out completely
                .DA     #0

.NO_SCALE_TXT   .AT     /NOT ACTIVE/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  SPEED_CMD
;   FUNCTION :  Setup the Speed function for CCIR or DIN norm
;---------------------------------------------------------------------------

SPEED_CMD       LCALL   .PARAMETER      Get parameter
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                MOV     A,PARAMETERS    Get parameter and store it
                MOV     C,ACC.0
                MOV     SPEED_REFERENCE,C
                SETB    SPEED_FUNCTION  Activate speed function
                MOV     A,COUNTER_STAT  Is counter busy?
                CJNE    A,#2,.CONTINUE  No!
                MOV     COUNTER_STAT,#4 Stop counting, and restart
.CONTINUE       JMP     SEND_STATUS

.PARAMETER      MOV     ERROR,#0        Clear error flag
                MOV     DPSEL,#3        Select parse pointer
                CALL    NEXT_NON_SPACE  Is a parameter following?
                JC      .MISSING        Parameter is missing!
                MOV     DPSEL,#0
                MOV     DPTR,#.PAR_TABLE Setup parameter table
                CALL    PARSE           Parse parameter
                JNC     .ILLEGAL        Illegal parameter error!
                MOV     A,R0            Save parameter
                MOV     PARAMETERS,A
                JMP     NO_MORE_PARS    No more paraemters are allowed!

.MISSING        MOV     ERROR,#5        Exit with missing parameter error
                JMP     SEND_STATUS

.ILLEGAL        MOV     ERROR,#7        Exit with illegal parameter error
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /CCIR/
                .AT     /DIN/
                .DA     #0

;---------------------------------------------------------------------------
;   ROUTINE  :  SPEED_REQ
;   FUNCTION :  Request current Speed norm
;---------------------------------------------------------------------------

SPEED_REQ       LCALL   NO_PARAMS       No parameters allowed!
                LCALL   TEST_ERROR      Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      SB_GENERAL,.END Don't do anything if general
                MOV     DPTR,#.NONE     In case of Speed mode not active
                JNB     SPEED_FUNCTION,.SEND Speed mode not active!
                MOV     DPTR,#.CCIR     In case of CCIR norm
                JNB     SPEED_REFERENCE,.SEND It was CCIR norm
                MOV     DPTR,#.DIN      It must be DIN norm now
.SEND           CALL    SEND_STRING
.END            JMP     SEND_STATUS     Exit with no error

.NONE           .AT     /SPEED IS NOT ACTIVE/,#CR
.CCIR           .AT     /CCIR/,#CR
.DIN            .AT     /DIN/,#CR

;---------------------------------------------------------------------------
;   ROUTINE  :  SYNC_CMD
;   FUNCTION :  Synchronise reading. Wait for measurement to complete
;---------------------------------------------------------------------------

SYNC_CMD        LCALL   NO_PARAMS       No parameters allowed!
                LCALL   .TEST_ERROR     Test previous errors
                CALL    TEST_HOLD       Test hold mode
                JB      MENU_MODE,.MENU_ACTIVE  Can't sync if menu's active!
                SETB    SYNC_FLAG       Wait until this flag is clear again
.LOOP           CALL    DO_EVENTS       Keep doing all other things
                JB      MENU_MODE,.EXIT Don't wait if user is in menu
                JB      STOP_ACT,.EXIT  ESC is pressed!
                JB      SYNC_FLAG,.LOOP Flag still set!
.EXIT           JMP     SEND_STATUS     Exit with no error

.TEST_ERROR     MOV     A,ERROR         It's ok if last error was the
                CJNE    A,#13,.TEST_REST  Menu active error
                MOV     ERROR,#0        Clear last error
                RET
.TEST_REST      JMP     TEST_ERROR

.MENU_ACTIVE    MOV     ERROR,#13       Exit with Menu active error
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
;   ROUTINE  :  SEND_FREQ
;   FUNCTION :  Send the frequency to the SB-Bus
;---------------------------------------------------------------------------

SEND_FREQ       MOV     R0,#FP_ACCU_A   Get the value in FP_ACCU_A
                MOV     R1,#6           Do 6 digit pairs
                MOV     R2,#0           Clear the 0 test
.READ_LOOP      MOVX    A,@DPTR         Copy a digit pair
                MOV     @R0,A
                ORL     A,R2            Test if total number is 0
                MOV     R2,A
                INC     DPTR            Increment pointers
                INC     R0
                DJNZ    R1,.READ_LOOP   Do all pairs
                MOV     A,R2            See if mantissa is 0
                JZ      .ZERO           Send 0 to the bus
                MOVX    A,@DPTR         Get exponent too
                MOV     C,ACC.7         Get 6/7 digit flag
                MOV     F0,C
                MOV     C,ACC.6         Sign extend to bit 7
                MOV     ACC.7,C
                MOV     FP_EXP_A,A      Setup the exponent

                MOV     R7,A            Shift right until exponent is pos.
.FRAC_LOOP      JNB     ACC.7,.EXP_POS  Exponent is positive! OK to display
                CLR     F0              Less digits are shown anyway!
                MOV     R0,#FP_ACCU_A   Shift ACCU 1 digit right
                CALL    FP_SHIFT_RIGHT
                INC     R7              Increment exponent
                MOV     A,R7
                SJMP    .FRAC_LOOP      Try again!
.EXP_POS        ADD     A,#6            Minimum value was -6, is now 0

                JB      FORMAT,.SCIENTIFIC Use scientific format

; Format 1 (xxxxxxxkHz)
                MOV     DPTR,#.DP_TABLE Setup pointer to lookup table
                CALL    .SEND_MANTISSA  Send mantissa to SB-Bus

                MOV     A,R2            Get annunciator values
                SWAP    A
                ANL     A,#%0000.0111
                JZ      .NO_KM          No k, M, G or T!
                MOV     R0,#'k'         In case it's kilo
                DEC     A
                JZ      .FOUND          It is!
                MOV     R0,#'M'         In case it's Mega
                DEC     A
                JZ      .FOUND          It is!
                MOV     R0,#'G'         In case it's Giga
                DEC     A
                JZ      .FOUND          It is!
                MOV     R0,#'T'         Now it must be Tera
.FOUND          MOV     A,R0            Send k, M, G, or T
                CALL    COUT
.NO_KM          MOV     DPTR,#.HZ       Send Hz to SB-Bus
                CALL    SEND_STRING
                RET

.HZ             .AT     /Hz/

.ZERO           MOV     A,#'0'          Send a 0 to the bus
                CALL    COUT
                RET

; Format 2 (x.xxxxxxE+y)
.SCIENTIFIC     MOV     DPTR,#.SC_TABLE Setup pointer to lookup table
                CALL    .SEND_MANTISSA  Send the mantissa to SB-Bus

                MOV     A,#'E'          Send the exponent indicator
                CALL    COUT
                MOV     A,#'+'
                CALL    COUT
                MOV     A,R2            Get exponent value
                SWAP    A
                ANL     A,#%0000.1111
                >CMP    A,#12           2 digit exponent?
                JC      .ONE_DIGIT      No! Only 1
                MOV     B,A
                MOV     A,#'1'          First digit is always 1
                CALL    COUT
                MOV     A,B
                CLR     C
                SUBB    A,#10           Get 2nd digit
.ONE_DIGIT      ORL     A,#'0'          Make it ASCII
                CALL    COUT             and send it to the SB-Bus
                RET

.SEND_MANTISSA  MOVC    A,@A+DPTR       Get DP position, and exponent
                MOV     R2,A            Save exponent
                ANL     A,#%0000.0111   Keep only DP position
                MOV     R3,A

                MOV     R0,#FP_ACCU_A+5 Setup the pointers
                CLR     F0              (F0 clear means low digit of byte)
                MOV     R4,#7
                JNB     ONLY6,.7DIGITS
                MOV     R4,#6           Do only 6 digits
.7DIGITS        MOV     A,@R0           Get digit
                SWAP    A               Use higher nibble as default
                JB      F0,.HIGH_NIBBLE
                SWAP    A               Use lower nibble instead
                DEC     R0              Get value from next byte next time
.HIGH_NIBBLE    CPL     F0
                ANL     A,#%0000.1111   Convert this to an ASCII digit
                ORL     A,#'0'
                CALL    COUT             and send it to SB-Bus
                DEC     R3
                MOV     A,R3            Show decimal point here?
                JNZ     .NO_POINT       No!
                MOV     A,#'.'          Send decimal point
                CALL    COUT
.NO_POINT       DJNZ    R4,.7DIGITS     Do them all
                RET

.DP_TABLE       .DA     #%0000.0000+1   0.00000x Hz     E-6
                .DA     #%0000.0000+1   0.0000xx Hz     E-5
                .DA     #%0000.0000+1   0.000xxx Hz     E-4
                .DA     #%0000.0000+1   0.00xxxx Hz     E-3
                .DA     #%0000.0000+1   0.0xxxxx Hz     E-2
                .DA     #%0000.0000+1   0.xxxxxx Hz     E-1
                .DA     #%0000.0000+1   x.xxxxxx Hz     E 0
                .DA     #%0000.0000+2   xx.xxxxx Hz     E+1
                .DA     #%0000.0000+3   xxx.xxxx Hz     E+2
                .DA     #%0001.0000+1   x.xxxxxx kHz    E+3
                .DA     #%0001.0000+2   xx.xxxxx kHz    E+4
                .DA     #%0001.0000+3   xxx.xxxx kHz    E+5
                .DA     #%0010.0000+1   x.xxxxxx MHz    E+6
                .DA     #%0010.0000+2   xx.xxxxx MHz    E+7
                .DA     #%0010.0000+3   xxx.xxxx MHz    E+8
                .DA     #%0011.0000+1   x.xxxxxx GHz    E+9
                .DA     #%0011.0000+2   xx.xxxxx GHz    E+10
                .DA     #%0011.0000+3   xxx.xxxx GHz    E+11
                .DA     #%0100.0000+1   x.xxxxxx THz    E+12
                .DA     #%0100.0000+2   xx.xxxxx THz    E+13
                .DA     #%0100.0000+3   xxx.xxxx THz    E+14
                .DA     #%0100.0000+4   xxxx.xxx THz    E+15
                .DA     #%0100.0000+5   xxxxx.xx THz    E+16
                .DA     #%0100.0000+6   xxxxxx.x THz    E+17

.SC_TABLE       .DA     #%0000.0000+1   0.00000x E+0    E-6
                .DA     #%0000.0000+1   0.0000xx E+0    E-5
                .DA     #%0000.0000+1   0.000xxx E+0    E-4
                .DA     #%0000.0000+1   0.00xxxx E+0    E-3
                .DA     #%0000.0000+1   0.0xxxxx E+0    E-2
                .DA     #%0000.0000+1   0.xxxxxx E+0    E-1
                .DA     #%0000.0000+1   x.xxxxxx E+0    E 0
                .DA     #%0000.0000+2   xx.xxxxx E+0    E+1
                .DA     #%0000.0000+3   xxx.xxxx E+0    E+2
                .DA     #%0011.0000+1   x.xxxxxx E+3    E+3
                .DA     #%0011.0000+2   xx.xxxxx E+3    E+4
                .DA     #%0011.0000+3   xxx.xxxx E+3    E+5
                .DA     #%0110.0000+1   x.xxxxxx E+6    E+6
                .DA     #%0110.0000+2   xx.xxxxx E+6    E+7
                .DA     #%0110.0000+3   xxx.xxxx E+6    E+8
                .DA     #%1001.0000+1   x.xxxxxx E+9    E+9
                .DA     #%1001.0000+2   xx.xxxxx E+9    E+10
                .DA     #%1001.0000+3   xxx.xxxx E+9    E+11
                .DA     #%1100.0000+1   x.xxxxxx E+12   E+12
                .DA     #%1100.0000+2   xx.xxxxx E+13   E+13
                .DA     #%1100.0000+3   xxx.xxxx E+14   E+14
                .DA     #%1111.0000+1   x.xxxxxx E+15   E+15
                .DA     #%1111.0000+1   xx.xxxxx E+16   E+16
                .DA     #%1111.0000+1   xxx.xxxx E+17   E+17

;---------------------------------------------------------------------------
;   ROUTINE  :  SEND_OFFSET
;   FUNCTION :  Send the offset frequency to the SB-Bus
;---------------------------------------------------------------------------

SEND_OFFSET     MOV     A,FP_EXP_C      Get exponent
                ADD     A,#6            Minimum value was -6, is now 0

                MOV     DPTR,#.SC_TABLE Setup pointer to lookup table
                CALL    .SEND_MANTISSA  Send the mantissa to SB-Bus

                MOV     A,#'E'          Send the exponent indicator
                CALL    COUT
                MOV     A,R2            Get exponent sign
                JB      ACC.3,.EXP_NEG  It's negative!
                MOV     A,#'+'
                SJMP    .EXP_POS
.EXP_NEG        MOV     A,#'-'
.EXP_POS        CALL    COUT
                MOV     A,R2            Get exponent value
                SWAP    A
                ANL     A,#%0000.1111
                ORL     A,#'0'          Make it ASCII
                CALL    COUT             and send it to the SB-Bus
                MOV     A,#CR
                CALL    COUT
                RET

.SEND_MANTISSA  MOVC    A,@A+DPTR       Get DP position, and exponent
                MOV     R2,A            Save exponent
                ANL     A,#%0000.0111   Keep only DP position
                MOV     R3,A

                MOV     R0,#FP_ACCU_C+5 Setup the pointers
                CLR     F0              (F0 clear means low digit of byte)
                MOV     R4,#7
.7DIGITS        MOV     A,@R0           Get digit
                SWAP    A               Use higher nibble as default
                JB      F0,.HIGH_NIBBLE
                SWAP    A               Use lower nibble instead
                DEC     R0              Get value from next byte next time
.HIGH_NIBBLE    CPL     F0
                ANL     A,#%0000.1111   Convert this to an ASCII digit
                ORL     A,#'0'
                CALL    COUT             and send it to SB-Bus
                DEC     R3
                MOV     A,R3            Show decimal point here?
                JNZ     .NO_POINT       No!
                MOV     A,#'.'          Send decimal point
                CALL    COUT
.NO_POINT       DJNZ    R4,.7DIGITS     Do them all
                RET

.SC_TABLE       .DA     #%0110.1000+1   x.xxxxxx E-6    E-6
                .DA     #%0110.1000+2   xx.xxxxx E-6    E-5
                .DA     #%0110.1000+3   xxx.xxxx E-6    E-4
                .DA     #%0011.1000+1   x.xxxxxx E-3    E-3
                .DA     #%0011.1000+2   xx.xxxxx E-3    E-2
                .DA     #%0011.1000+3   xxx.xxxx E-3    E-1
                .DA     #%0000.0000+1   x.xxxxxx E+0    E 0
                .DA     #%0000.0000+2   xx.xxxxx E+0    E+1
                .DA     #%0000.0000+3   xxx.xxxx E+0    E+2
                .DA     #%0011.0000+1   x.xxxxxx E+3    E+3
                .DA     #%0011.0000+2   xx.xxxxx E+3    E+4
                .DA     #%0011.0000+3   xxx.xxxx E+3    E+5
                .DA     #%0110.0000+1   x.xxxxxx E+6    E+6
                .DA     #%0110.0000+2   xx.xxxxx E+6    E+7
                .DA     #%0110.0000+3   xxx.xxxx E+6    E+8
                .DA     #%1001.0000+1   x.xxxxxx E+9    E+9
                .DA     #%1001.0000+2   xx.xxxxx E+9    E+10
                .DA     #%1001.0000+3   xxx.xxxx E+9    E+11
                .DA     #%1001.0000+4   xxxx.xxx E+9    E+12

;---------------------------------------------------------------------------
;   ROUTINE  :  SCALE_PRINT
;   FUNCTION :  Print scaling value in FP Accu C
;---------------------------------------------------------------------------

SCALE_PRINT     MOV     A,FP_EXP_C      Get Exponent of scaling factor
                INC     A
                MOV     R7,A
                MOV     R0,#FP_ACCU_C+2 Point to LSD of scaling factor
                CLR     F0              Point to 2nd BCD number in byte
                MOV     R1,#7           Set meaningful digits counter
.MEAN_DIG_LOOP  MOV     A,@R0           Get digit
                JNB     F0,.LOW_DIGIT1  Get low digit
                SWAP    A               Get upper digit
.LOW_DIGIT1     ANL     A,#%0000.1111   Throw away high part
                JNZ     .MEAN_DIGIT     This is a meaningful digit
                CPL     F0              Complement digit flag
                JB      F0,.SKIP1       Skip incrementing pointer
                INC     R0              Increment pointer
.SKIP1          DJNZ    R1,.MEAN_DIG_LOOP Find 1st meaningful digit!
                INC     R1              Must be 1 digit at least!
.MEAN_DIGIT     MOV     R0,#FP_ACCU_C+5 Point to MSD
                CLR     F0              Point to 2nd BCD number in byte

.PRINT_LOOP     MOV     A,@R0           Get digit
                JNB     F0,.LOW_DIGIT2  Use low digit
                SWAP    A               Use high digit
.LOW_DIGIT2     ANL     A,#%0000.1111   Use only low nibble
                ORL     A,#'0'          Make it ASCII
                CPL     F0              Complement digit flag
                JNB     F0,.SKIP2       Don't increment pointer in ACCU!
                DEC     R0              Decrement pointer in ACCU
.SKIP2          CALL    COUT            Send character out
                DEC     R1              Decrement meaningful digit counter
                DEC     R7              Show decimal point here?
                MOV     A,R7
                JNZ     .NO_POINT       No!
                MOV     A,R1            No meaningful digits after DP?
                JZ      .DONE           No!
                JB      ACC.7,.DONE     No!
                MOV     A,#'.'          Send decimal point
                CALL    COUT
.NO_POINT       MOV     A,R7            Is decimal point printed yet?
                JNB     ACC.7,.PRINT_LOOP Not yet! Go on!
                MOV     A,R1            More meaningful digits?
                JNZ     .PRINT_LOOP     Yes! Do them all
.DONE           RET

;---------------------------------------------------------------------------
;   ROUTINE    :  GET_FP_VALUE
;   FUNCTION   :  Get FP value from SB-Bus buffer
;---------------------------------------------------------------------------

GET_FP_VALUE    MOV     R0,#FP_ACCU_C   Clear result accu
                CALL    CLR_FP_ACCU
                MOV     FP_EXP_C,#-1    Give exponent a negative offset
                CLR     DP_FLAG         Clear decimal point flag
                CLR     VALID_FP        Clear valid floating point flag
                CLR     LEAD_0          Start with leading zeroflag clear
.MANTISSA_LOOP  MOV     R1,DPL          Save current pointer
                CALL    GET_UC          Get new digit from buffer
                CJNE    A,#'.',.NOT_DP  It's not a decimal point!
                JB      DP_FLAG,.ILLEGAL Illegal value if more than one dp!
                SETB    DP_FLAG         Set decimal point flag
                SETB    VALID_FP        At least something was entered
                SJMP    .MANTISSA_LOOP  Get next digit
.NOT_DP         >CMP    A,#'0'          Is it a digit?
                JC      .NOT_DIGIT      It's not a digit!
                >CMP    A,#'9'+1
                JNC     .NOT_DIGIT      It's not a digit!
                SETB    VALID_FP
                XRL     A,#'0'          Strip off the ASCII part
                JZ      .ZERO           It's a 0!
                SETB    LEAD_0          No leading 0 anymore
.ZERO           MOV     R7,A            Save this number
                MOV     A,FP_ACCU_C+5   Accept more digits?
                JNZ     .DONT_ACCEPT    No! We've got enough!
                MOV     R0,#FP_ACCU_C   Shift Accu left
                CALL    FP_SHIFT_LEFT
                MOV     A,FP_ACCU_C+2   Include new digit
                ORL     A,R7
                MOV     FP_ACCU_C+2,A
.DONT_ACCEPT    JB      DP_FLAG,.FRAC   Handle Exp different if DP is given!
                JNB     LEAD_0,.MANTISSA_LOOP  Still leading zeros!
                INC     FP_EXP_C        Increment Exponent
                SJMP    .MANTISSA_LOOP
.FRAC           JB      LEAD_0,.MANTISSA_LOOP Don't change exp if DP given!
                DEC     FP_EXP_C        Decrement as long as leading 0
                SJMP    .MANTISSA_LOOP

.ILLEGAL        MOV     ERROR,#7        Exit with illegal value error
                RET

.NOT_DIGIT      MOV     R7,A            Save new character
                JNB     LEAD_0,.MANTISSA_0 Mantissa is 0
                MOV     R2,#7           Try to shift number all the way
                MOV     R0,#FP_ACCU_C    to the left
.SHIFT_LOOP     MOV     A,FP_ACCU_C+5   Is most significant digit <> 0?
                JNZ     .SHIFT_DONE     Yes!
                CALL    FP_SHIFT_LEFT   Shift result to the left
                DJNZ    R2,.SHIFT_LOOP  Do max 7 times! Otherwise # is 0

.MANTISSA_0     MOV     FP_EXP_C,#0     Clear exponent for mantissa is 0
.SHIFT_DONE     MOV     A,R7            Get character back
                MOV     R7,#0           Clear temporary exponent value
                CJNE    A,#'E',.DONE    It's not the exponent!
                SETB    VALID_FP
                MOV     R1,DPL          Save current position
                CALL    GET_UC          Get next character
                CLR     DP_FLAG         Abuse this flag as sign flag
                CJNE    A,#'+',.NOT_POS It's not positive!
                MOV     R1,DPL          Save current position
                CALL    GET_UC          Get next character
                SJMP    .GET_EXP        It's positive!
.NOT_POS        CJNE    A,#'-',.GET_EXP It's not negative either!
                MOV     R1,DPL          Save current position
                CALL    GET_UC          Get next character
                SETB    DP_FLAG         Indicate negative sign
.GET_EXP        >CMP    A,#'0'          Is it a digit?
                JC      .EXP_DONE       It's not a digit!
                >CMP    A,#'9'+1
                JNC     .EXP_DONE       It's not a digit!
                JNB     LEAD_0,.NEXT_EXP Mantissa is 0, don't bother range!
                XRL     A,#'0'          Strip off the ASCII part
                MOV     R2,A            Save this byte
                MOV     A,R7            Get previous value
                MOV     B,#10            and multiply it by 10
                MUL     AB
                ADD     A,R2            Add new digit to it
                JC      .RANGE          Range error!
                MOV     R7,A            Save new exponent value
                MOV     A,B             See if high byte of result still 0
                JNZ     .RANGE          It's not! So range error
.NEXT_EXP       MOV     R1,DPL          Save current position
                CALL    GET_UC          Get next digit
                SJMP    .GET_EXP        Continue until it's not a number
.EXP_DONE       MOV     A,R7            Get exponent value
                JB      ACC.7,.RANGE    Range error if b7 <>0
                JB      DP_FLAG,.NEG_EXP It's a negative exponent
                ADD     A,FP_EXP_C      Add to existing exponent
                MOV     FP_EXP_C,A      Save new exponent
                JNC     .DONE           No range error!
                JB      ACC.7,.RANGE    Range error if result is neg again!
                SJMP    .DONE
.NEG_EXP        XCH     A,FP_EXP_C      Subtract from exisiting exponent
                CLR     C
                SUBB    A,FP_EXP_C
                MOV     FP_EXP_C,A      Save new exponent
                JNC     .DONE           Definatly no overflow
                JNB     ACC.7,.RANGE    Range error if result is pos again!
.DONE           MOV     DPL,R1          Restore previous position
                MOV     DPSEL,#0        Restore DP selector
                RET

.RANGE          MOV     ERROR,#12       Exit with range error
                RET

;---------------------------------------------------------------------------
;   ERROR TEXT table
;---------------------------------------------------------------------------

ERROR_TEXT      .AT     /NO ERROR/,#CR                          Error 0
                .AT     /SYNTAX ERROR/,#CR                      Error 1
                .AT     /NOTHING TO REPEAT ERROR/,#CR           Error 2
                .AT     /ABORTED ERROR/,#CR                     Error 3
                .AT     /NO PARAMETERS ALLOWED/,#CR             Error 4
                .AT     /MISSING PARAMETER ERROR/,#CR           Error 5
                .AT     /TOO MANY PARAMETERS ERROR/,#CR         Error 6
                .AT     /ILLEGAL PARAMETER ERROR/,#CR           Error 7
                .AT     /TOO MANY ERRORS/,#CR                   Error 8
                .AT     /HOLD MODE DEACTIVATED/,#CR             Error 9
                .AT     /HOLD NOT ACTIVE ERROR/,#CR             Error 10
                .AT     /NOTHING IN HOLD ERROR/,#CR             Error 11
                .AT     /RANGE ERROR/,#CR                       Error 12
                .AT     /MENU IS ACTIVE ERROR/,#CR              Error 13
                .AT     /VALUE NOT FOUND ERROR/,#CR             Error 14
                .AT     /DUPLICATE VALUE ERROR/,#CR             Error 15
                .AT     /NVM WRITE ERROR/,#CR                   Error 16
                .AT     /MEMORY FULL ERROR/,#CR                 Error 17
                .AT     /NO REFERENCE SWITCH ERROR/,#CR         Error 18


