;---------------------------------------------------------------------------
;
;   COUNTER.ASM
;
;   Include file for SB6668 Frequency counter
;
;   These routines handle the actual frequency counting
;
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
;   ROUTINE    :  COUNTER
;   FUNCTION   :  Main counter entry routine
;---------------------------------------------------------------------------

COUNTER         JB      MENU_MODE,.RET  Don't do anything if menu active!
                MOV     A,COUNTER_STAT  Get current counter status
                RL      A               Create table address to fetch dest.
                ADD     A,#.COUNT_JUMPS
                MOV     DPL,A
                MOV     A,/.COUNT_JUMPS
                ADDC    A,#0
                MOV     DPH,A
                >GETC
                MOV     R0,A            Save high byte of routine address
                INC     DPTR
                >GETC
                MOV     R1,A            Save low byte of routine address
                PUSH    B0_R1           Start the routine by pushing
                PUSH    B0_R0            its address on the stack
.RET            RET                      followed by a RET

.COUNT_JUMPS    .DA     COUNTER_IDLE  0 Counting has stopped
                .DA     COUNTER_TRIG  1 Wait for trigger pulse from input
                .DA     COUNTER_MEAS  2 Counting commences
                .DA     COUNTER_STOP  3 Wait for trigger pulse to stop
                .DA     RETRY_MEASURE 4 In case SB-BUS wants to interrupt

;---------------------------------------------------------------------------
;   ROUTINE    :  COUNTER_IDLE
;   FUNCTION   :  Counting has stopped, wait to start new run
;---------------------------------------------------------------------------

COUNTER_IDLE    JB      MENU_MODE,.RET  Don't bother if menu is active
                MOV     A,TIME_DELAY    Is the timer still running?
                ORL     A,TIME_DELAY+1
                JZ      START_COUNTER   Not anymore! Start the counter
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  START_COUNTER
;   FUNCTION   :  Get the counter running
;---------------------------------------------------------------------------

START_COUNTER   MOV     COUNTER_STAT,#1 Set counter status 1 (wait for trig)
                MOV     TIME_DELAY,#TRIG_DELAY  Setup trigger delay
                MOV     TIME_DELAY+1,/TRIG_DELAY

                CPL     GATE_FLAG       Complement gate flag everey new start
                ANL     P1,#%1111.0000  Disable display controllers
                CLR     P0.4            Reset Flip-flop of IC110
                MOV     C,P1.6          Copy input select bit to flag
                MOV     INPUT_SELECT,C
                SETB    P0.5            Activate Reset and
                CLR     P0.6             -Reset
                MOV     REF_EXTEND,#0   Clear reference extension
                MOV     C,INPUT_SELECT  Select propper input
                MOV     P1.7,C
                CLR     A               Clear hardware timer/counter T0
                MOV     TL0,A
                MOV     TH0,A
                SETB    TR0             Start timer/counter T0
                SETB    ET0             Enable T0 ineterrupt
                CLR     P0.5            Release Reset
                ORL     P0,#%0101.0000  Release -Reset and IC110
                CLR     P0.7            Start measurement
                RET                     Wait for trigger pulse now

;---------------------------------------------------------------------------
;   ROUTINE    :  COUNTER_TRIG
;   FUNCTION   :  Wait for 1st input pulse to trigger the start flip-flop
;---------------------------------------------------------------------------

COUNTER_TRIG    CALL    CHECK_HF        Abort if HF status has changed
                JB      P3.0,START_MEASURE Trigger pulse received!
                MOV     A,TIME_DELAY    See if we've waited long enough
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Not yet!
                JMP     RETRY_MEASURE   It took too long!
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  START_MEASURE
;   FUNCTION   :  1st pulse is detected. Now start measure delay time
;---------------------------------------------------------------------------

START_MEASURE   MOV     COUNTER_STAT,#2 Set counter status 2 (measure time)
                MOV     TIME_DELAY,#SLOW_DELAY  Setup measuring time in
                MOV     TIME_DELAY+1,/SLOW_DELAY  case of slow rate
                JNB     MEASURE_RATE,.SLOW      It is slow mode!
                MOV     TIME_DELAY,#FAST_DELAY  Setup measuring time for
                MOV     TIME_DELAY+1,/FAST_DELAY   fast rate
.SLOW           MOV     C,MEASURE_RATE          Copy measure rate to actual
                MOV     ACTUAL_RATE,C            rate (in case it changes)
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  COUNTER_MEAS
;   FUNCTION   :  Wait for measuring time to expire
;---------------------------------------------------------------------------

COUNTER_MEAS    CALL    CHECK_HF        Abort if HF status changed
                MOV     A,TIME_DELAY    Counter timed out?
                ORL     A,TIME_DELAY+1
                JZ      STOP_MEASURE    Yes! Stop measuring
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  STOP_MEASURE
;   FUNCTION   :  Signal end of measuring periode
;---------------------------------------------------------------------------

STOP_MEASURE    SETB    P0.7            Stop measuring (at next input edge)
                MOV     COUNTER_STAT,#3 Set counter status 3 (wait for stop)
                MOV     TIME_DELAY,#STOP_DELAY  Setup stop trigger delay
                MOV     TIME_DELAY+1,/STOP_DELAY
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  COUNTER_STOP
;   FUNCTION   :  Signal end of measuring periode
;---------------------------------------------------------------------------

COUNTER_STOP    CALL    CHECK_HF        Abort if HF status changed
                JNB     P3.0,CALC_RESULT  Counter stopped! Calculate result
                MOV     A,TIME_DELAY    See if delay time is over yet
                ORL     A,TIME_DELAY+1
                JZ      RETRY_MEASURE   Yes! Restart measuring
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  CHECK_HF
;   FUNCTION   :  See if HF status has changed
;---------------------------------------------------------------------------

CHECK_HF        MOV     C,P1.6          Get actual status
                MOV     ACC.0,C
                XRL     A,COUNTER_FLAGS See if it changed since the start
                JB      ACC.0,.CHANGED  It changed!
                RET
.CHANGED        >POP    ACC,ACC         Clear return stack and fall through

;---------------------------------------------------------------------------
;   ROUTINE    :  RETRY_MEASURE                        (Fall through)
;   FUNCTION   :  Clear display to 0, and restart measurement
;---------------------------------------------------------------------------
; Display is cleared to 0, even if an offset is selected. This is done to
; avoid confusing results on the display. The unit indicators are also not
; displayed.

RETRY_MEASURE   SETB    P0.7            Stop measurement, no matter what!
                CLR     SYNC_FLAG       Quit sync command
                MOV     R0,#FP_ACCU_C   Clear FP_Accu_C
                CALL    CLR_FP_ACCU
                MOV     DPTR,#SB_FREQUENCY Clear SB-Bus values
                CALL    FP_C_2_SBBUS
                MOV     DPTR,#SB_DISPLAY
                CALL    FP_C_2_SBBUS
                MOV     FP_EXP_C,#3     Make percentage display OL
                MOV     DPTR,#PERCENT_VALUE
                CALL    FP_C_2_SBBUS
                MOV     R0,#7           Display all 0's on the display
                MOV     A,#CHR_0
                MOV     R1,#DSP_BUFFER
.CLR_DISPLAY    MOV     @R1,A           Do all 7 displays
                INC     R1
                DJNZ    R0,.CLR_DISPLAY
                MOV     @R1,#0          Clear all annunciators
                JNB     SPEED_FUNCTION,.SKIP Speed mode not active
                MOV     DSP_BUFFER+4,#SG    Show -oL in % area
                MOV     DSP_BUFFER+5,#CHR_O
                MOV     DSP_BUFFER+6,#CHR_L
.SKIP           DEC     R1              Copy gate flag to rightmost DP
                MOV     A,@R1
                MOV     C,GATE_FLAG
                MOV     ACC.7,C
                MOV     @R1,A
                CALL    UPDATE_DSP      Update the display
                JMP     START_COUNTER   Start the counter

;---------------------------------------------------------------------------
;   ROUTINE    :  CALC_RESULT
;   FUNCTION   :  Calculate result of measurement
;---------------------------------------------------------------------------

CALC_RESULT     CLR     TR0             Stop timer/counter T0
                CLR     ET0             Disable T0 ineterrupt
                CLR     SYNC_FLAG       Release SYNC command

; The reference counter is made up from 3 different sources:
;  - The least significant 8 bits come from IC106 in binary format
;  - Then come the 16 bits from T0 in binary format
;  - Then come 2 BCD digits from REF_EXTEND.
; It is not very likely that REF_EXTEND > 0, only at very low frequencies
; this counter will increment. So a test on 0 of this value will increase
; the normal calculation speed.

                MOV     R0,#FP_ACCU_B   Clear FP ACCU B
                CALL    CLR_FP_ACCU
                MOV     R0,#FP_ACCU_A   Clear FP ACCU A (leaving R0 point
                CALL    CLR_FP_ACCU      to FP_ACCU_A!)
                MOV     A,REF_EXTEND    Was REF_EXTEND incremented?
                JZ      .SKIP           No!
                MOV     FP_ACCU_A,A     Multiply REF_EXTEND with 2^24
                MOV     R1,R0           Add FP_ACCU_A 24 times to itself
                MOV     R3,#24
.DOUBLE_EXTEND  CALL    ADD_FP          Add FP ACCU A to FP ACCU A
                DJNZ    R3,.DOUBLE_EXTEND  Do this 24 times!
.SKIP           MOV     A,TH0           Convert most significant binary
                CALL    BIN2BCD          byte to BCD
                MOV     A,TL0           Convert middle significant binary
                CALL    BIN2BCD          byte to BCD
                MOV     A,P2            Convert least significant binary
                CALL    BIN2BCD          byte to BCD
                MOV     R0,#FP_ACCU_A   Add converted BCD to the extension
                MOV     R1,#FP_ACCU_B    counter in FP ACCU A
                CALL    ADD_FP          Reference counter is now in FP A!

; The event counter is made up of 2 different sources
;  - The 8 event BCD digits counted in IC107 and IC108
;  - The 2 bits from IC106 and IC110
; Therefore the BCD counters from IC107 and IC108 must be multiplied by 4
; to be able to include the 2 extra bits.

                MOV     R0,#FP_ACCU_B   Clear FP accu B
                CALL    CLR_FP_ACCU
                ORL     P1,#%0000.1111  Enable the event counter inputs
                MOV     R0,#0           Select hardware decade 1
                MOV     R1,#FP_ACCU_B   Put result in FP ACCU B
                MOV     R4,#4           Do 4 pairs of decimal digits
.GET_EVENT_BCD  CALL    GET_2_DECIMALS  Get 2 decimals from BCD counters
                DJNZ    R4,.GET_EVENT_BCD Do all 4 pairs
                ANL     P0,#%1111.0000  Disable decade counter outputs
                MOV     R0,#FP_ACCU_B   Double FP ACCU B twice to include
                MOV     R1,#FP_ACCU_B    the 2 extra flip-flops
                CALL    ADD_FP
                MOV     A,@R0           Include 2nd LSB flip-flop
                MOV     C,P1.5
                MOV     ACC.0,C
                MOV     @R0,A
                CALL    ADD_FP
                MOV     A,@R0           Include LSB flip-flop
                MOV     C,P1.4
                MOV     ACC.0,C
                MOV     @R0,A           Put result back

; Now we have the counters in FP_A (reference) and FP_B (events).
; Next we must set the exponent correctly.
; For LF input the exponent must be 8 (10MHz x 10(prescaler) = 10^7 x 10)
; For HF input the event counter must be multiplied with 256 and the
; exponent set to 10 (10MHz reference)

                MOV     R7,#8           Temporary exponent for LF input
                JB      INPUT_SELECT,.EXP_OK  LF input was used!
                MOV     R0,#FP_ACCU_B   Multiply event counter with 256
                MOV     R1,#FP_ACCU_B    by doubling it 8 times
.X256           CALL    ADD_FP
                DJNZ    R7,.X256        The LF exponent is mis-used here
                MOV     R7,#7           Set exponent to 7 now
.EXP_OK

; Now we must normalise both counters. This means that the values are
; shifted left until the left digit reaches the 2nd MSD. The most signifi-
; cant digit is left 0 for easy division later on.
; Each time the event value is shifted (multiplied by 10), the exponent
; will be decremented. Each time the reference counter is shifted, the
; exponent will be incremented.

                MOV     R0,#FP_ACCU_B   Normalise event counter first
                CALL    CNT_NORMALISE
                MOV     A,R7            Negate the exponent, so it will
                CPL     A                be incremented during next
                INC     A                normalising process
                MOV     R7,A
                MOV     R0,#FP_ACCU_A   Normalise reference counter
                CALL    CNT_NORMALISE
                JNZ     .NOT_0          Divider was not 0!
                JMP     RETRY_MEASURE   Something's wrong with reference!
.NOT_0          MOV     A,R7            Give exponent original polarity
                CPL     A
                INC     A
                MOV     FP_EXP_B,A
                MOV     FP_EXP_A,#0     Will be subtracted by division

; Counters are normalised, now it's time to divide the event counter
; by the reference counter.
                CALL    DIVIDE_FP       Divide B by A, result in C

; Now let's see if the result must be displayed as a 7 digit or 6 digit
; result. The 6 digit result applies to Fast measuring rate values of
; 200000 and up. To avoid a jumping display a hysteresis of 10% is
; introduced in the switching range. Above 219999 we switch back to 6
; digit display. Below 200000 we switch back to 7 digit display.

                JNB     ACTUAL_RATE,.SLOW   Slow rate is always 7 digits!
                MOV     A,FP_ACCU_C+5   Get 2 most significant digits and
                SWAP    A                combine them to 1 BCD byte
                MOV     R2,A
                MOV     A,FP_ACCU_C+4
                SWAP    A
                ANL     A,#%0000.1111
                ORL     A,R2
                JB      ONLY6,.HYST     Showing 6, may we show 7?
                >CMP    A,#$22          Range below 219999? (showing 7)
                JC      .SLOW           Yes! Don't change flag
                SETB    ONLY6           Set the 6 digit flag
                SJMP    .SLOW
.HYST           >CMP    A,#$20          Range below 200000
                JNC     .SLOW           No! Keep showing 6!
                CLR     ONLY6           Clear the 6 digit flag
.SLOW

; Frequency value is known now! First round it to 6 or 7 digits, and then
; copy the frequency value to the SB-Bus buffer, in case the user requests
; the frequency.

                MOV     R0,#FP_ACCU_C   Round Accu C to Accu B
                MOV     R1,#FP_ACCU_B
                MOV     R2,#7           Round at 7 digits as default
                JNB     ONLY6,.ROUND7   It's Ok!
                DEC     R2              Round at 6 digits
.ROUND7         CALL    ROUND
                MOV     DPTR,#SB_FREQUENCY Setup pointers
                MOV     R0,#FP_ACCU_B
                CALL    FP_2_SBBUS      Copy the value to the SB-Bus buffer

; Now we know the exact frequency, its exponent and its accuracy.
; We can modify the values now to include the new counter features.

                CALL    MODIFY_VALUE    Modify the value

; Finally FP_ACCU_C holds the frequency and exponent value of the measured
; and modified value. This value must be rounded and copied to the display
; and SB-Bus display value.
; Copying to the display includes setting of the decimal point, showing the
; right units (Hz, kHZ and MHz) and showing the appropriate accuracy.

                MOV     R0,#FP_ACCU_C   Round Accu C to Accu B
                MOV     R1,#FP_ACCU_B
                MOV     R2,#7           Round at 7 digits as default
                JNB     ONLY6,.ROUND_7  It's Ok!
                DEC     R2              Round at 6 digits
.ROUND_7        CALL    ROUND
                MOV     DPTR,#SB_DISPLAY Setup pointers
                MOV     R0,#FP_ACCU_B
                CALL    FP_2_SBBUS      Copy the value to the SB-Bus buffer
                JB      SPEED_FUNCTION,.DSP_SPEED  Display speed and %
                MOV     R0,#FP_ACCU_B   Copy Accu B to Accu C
                MOV     R1,#FP_ACCU_C
                CALL    COPY_FP_ACCU
                CALL    FP_C_2_DISPLAY  Send FP Accu B to the display
                JMP     START_COUNTER   Restart the counter

.DSP_SPEED      CALL    DISPLAY_SPEED   Display the speed and %
                JMP     START_COUNTER   Restart the counter

;---------------------------------------------------------------------------
;   ROUTINE    :  FP_C_2_SBBUS
;   FUNCTION   :  Copy Floating point Accu C to SB-Bus value buffer
;---------------------------------------------------------------------------

FP_C_2_SBBUS    MOV     R0,#FP_ACCU_C   Copy Accu C
FP_2_SBBUS      MOV     R1,#6           Do 6 digits pairs
.LOOP           MOV     A,@R0           Copy a digit pair
                MOVX    @DPTR,A
                INC     R0              Increment pointers
                INC     DPTR
                DJNZ    R1,.LOOP        Do all digit pairs!
                MOV     A,@R0           Finally copy the exponent
                MOV     C,ONLY6         Copy 6/7 digit flag to b7
                MOV     ACC.7,C
                MOVX    @DPTR,A
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  FP_C_2_DISPLAY
;   FUNCTION   :  Copy modified value to the display
;---------------------------------------------------------------------------

FP_C_2_DISPLAY  MOV     R0,#FP_ACCU_C   Point to propper Accu
                MOV     R2,#6           See if value is 0
                CLR     A
.ZERO_TEST      ORL     A,@R0
                INC     R0
                DJNZ    R2,.ZERO_TEST   Do 6 digit pairs!
                JZ      .ZERO           Display only 0000000
                MOV     A,FP_EXP_C      Get the exponent
                MOV     R7,A            Shift right until exponent is pos.
.FRAC_LOOP      JNB     ACC.7,.EXP_POS  Exponent is positive! OK to display
                CLR     ONLY6           Less digits are shown anyway!
                MOV     R0,#FP_ACCU_C   Shift ACCU 1 digit right
                CALL    FP_SHIFT_RIGHT
                INC     R7              Increment exponent
                MOV     A,R7
                SJMP    .FRAC_LOOP      Try again!

.EXP_POS        MOV     A,FP_EXP_C      Minimum exponent is -6, make it 0
                ADD     A,#6
                MOV     DPTR,#.7DIGIT_TABLE In case of 7 digit display
                JNB     ONLY6,.SKIP_TABLE  Display 7 digits!
                MOV     DPTR,#.6DIGIT_TABLE Use different table in case of
.SKIP_TABLE     MOVC    A,@A+DPTR       Get flag byte
                JZ      .OL             Overload!
                MOV     R2,A            Save flags
                ANL     A,#%0000.0111   Keep only decimal point position
                MOV     R3,A

                MOV     R0,#FP_ACCU_C+5 Setup the pointers
                MOV     R1,#DSP_BUFFER
                CLR     F0              (F0 clear means low digit of byte)
                MOV     R4,#7
                JNB     ONLY6,.DO_7_DIGITS
                MOV     R4,#6           Do only 6 digits
                MOV     @R1,#0          Clear 7th digit
                MOV     A,@R0           Display a 0 here?
                JNZ     .NO_LEAD0       No!
                MOV     @R1,CHR_0       Show leading 0
.NO_LEAD0       INC     R1              Point to next digit
.DO_7_DIGITS    MOV     A,@R0           Get digit
                SWAP    A               Use higher nibble as default
                JB      F0,.HIGH_NIB
                SWAP    A               Use lower nibble instead
                DEC     R0              Get value from next byte next time
.HIGH_NIB       CPL     F0
                CALL    BCD_2_7SEGM     Convert this to 7 segment value
                MOV     @R1,A           Save it
                DEC     R3
                MOV     A,R3            Show decimal point here?
                JNZ     .NO_POINT       No!
                MOV     A,@R1           Set decimal point
                ORL     A,#%1000.0000
                MOV     @R1,A
.NO_POINT       INC     R1              Point to next digit
                DJNZ    R4,.DO_7_DIGITS Do them all
                MOV     A,R2            Get annunciator values
                SWAP    A
                ANL     A,#%0000.0111
                MOV     @R1,A           And switch on the annunciators
                DEC     R1              Point to rightmost digit
                MOV     C,GATE_FLAG     Copy gate flag to right most point
                MOV     A,@R1
                MOV     ACC.7,C
                MOV     @R1,A
                CALL    UPDATE_DSP      Update display
                RET

.ZERO           MOV     R0,#7           Display all 0's on the display
                MOV     A,#CHR_0
                MOV     R1,#DSP_BUFFER
.CLR_DISPLAY    MOV     @R1,A           Do all 7 displays
                INC     R1
                DJNZ    R0,.CLR_DISPLAY
                MOV     @R1,#0          Clear all annunciators
                DEC     R1              Copy gate flag to rightmost DP
                MOV     A,@R1
                MOV     C,GATE_FLAG
                MOV     ACC.7,C
                MOV     @R1,A
                CALL    UPDATE_DSP      Update display
                RET

.OL             MOV     DPTR,#.OVERLOAD Put overload in display
                CALL    SHOW_TEXT
                RET

.OVERLOAD       .DA     #BLANK,#CHR_O,#CHR_L,#BLANK,#BLANK,#BLANK,#BLANK

.7DIGIT_TABLE   .DA     #%0010.0000+1   0.00000x Hz     E-6
                .DA     #%0010.0000+1   0.0000xx Hz     E-5
                .DA     #%0010.0000+1   0.000xxx Hz     E-4
                .DA     #%0010.0000+1   0.00xxxx Hz     E-3
                .DA     #%0010.0000+1   0.0xxxxx Hz     E-2
                .DA     #%0010.0000+1   0.xxxxxx Hz     E-1
                .DA     #%0010.0000+1   x.xxxxxx Hz     E 0
                .DA     #%0010.0000+2   xx.xxxxx Hz     E+1
                .DA     #%0010.0000+3   xxx.xxxx Hz     E+2
                .DA     #%0010.0000+4   xxxx.xxx Hz     E+3
                .DA     #%0001.0000+2   xx.xxxxx kHz    E+4
                .DA     #%0001.0000+3   xxx.xxxx kHz    E+5
                .DA     #%0001.0000+4   xxxx.xxx kHz    E+6
                .DA     #%0110.0000+2   xx.xxxxx MHz    E+7
                .DA     #%0110.0000+3   xxx.xxxx MHz    E+8
                .DA     #%0110.0000+4   xxxx.xxx MHz    E+9
                .DA     #%0110.0000+5   xxxxx.xx MHz    E+10
                .DA     #%0110.0000+6   xxxxxx.x MHz    E+11
                .DA     #%0110.0000+0   xxxxxxx. MHz    E+12
                .DA     #%0000.0000+0      OL           E+13
                .DA     #%0000.0000+0      OL           E+14
                .DA     #%0000.0000+0      OL           E+15
                .DA     #%0000.0000+0      OL           E+16
                .DA     #%0000.0000+0      OL           E+17
                .DA     #%0000.0000+0      OL           E+18

.6DIGIT_TABLE   .DA     #%0010.0000+1   0.00000x  Hz     E-6
                .DA     #%0010.0000+1   0.0000xx  Hz     E-5
                .DA     #%0010.0000+1   0.000xxx  Hz     E-4
                .DA     #%0010.0000+1   0.00xxxx  Hz     E-3
                .DA     #%0010.0000+1   0.0xxxxx  Hz     E-2
                .DA     #%0010.0000+1   0.xxxxxx  Hz     E-1
                .DA     #%0010.0000+1    x.xxxxx  Hz     E 0
                .DA     #%0010.0000+2    xx.xxxx  Hz     E+1
                .DA     #%0010.0000+3    xxx.xxx  Hz     E+2
                .DA     #%0001.0000+1    x.xxxxx  kHz    E+3
                .DA     #%0001.0000+2    xx.xxxx  kHz    E+4
                .DA     #%0001.0000+3    xxx.xxx  kHz    E+5
                .DA     #%0110.0000+1    x.xxxxx  MHz    E+6
                .DA     #%0110.0000+2    xx.xxxx  MHz    E+7
                .DA     #%0110.0000+3    xxx.xxx  MHz    E+8
                .DA     #%0110.0000+4    xxxx.xx  MHz    E+9
                .DA     #%0110.0000+5    xxxxx.x  MHz    E+10
                .DA     #%0110.0000+0    xxxxxx.  MHz    E+11
                .DA     #%0000.0000+0      OL           E+12
                .DA     #%0000.0000+0      OL           E+13
                .DA     #%0000.0000+0      OL           E+14
                .DA     #%0000.0000+0      OL           E+15
                .DA     #%0000.0000+0      OL           E+16
                .DA     #%0000.0000+0      OL           E+17
                .DA     #%0000.0000+0      OL           E+18

;---------------------------------------------------------------------------
;   ROUTINE    :  GET_2_DECIMALS
;   FUNCTION   :  Get 2 decimals from BCD counters
;---------------------------------------------------------------------------

GET_2_DECIMALS  CALL    .READ_DECIMAL   Read 1 BCD digit from counter
                MOV     R2,A            Save it for a while
                CALL    .READ_DECIMAL   Read next BCD digit from counter
                SWAP    A                and move it in position
                ORL     A,R2            Combine with previous BCD digit
                MOV     @R1,A            and save it in FP ACCU B
                INC     R1
                RET

.READ_DECIMAL   ANL     P0,#%1111.0000  Select none of the decade counters
                MOV     A,R0            Get decade counter select mask from
                INC     R0                 table and point to next one for
                MOV     DPTR,#.DECADE_MASK  later use
                MOVC    A,@A+DPTR
                ORL     P0,A            Combine this mask with rest of
                NOP                     We're going too fast for some of
                NOP                      decade counters. One NOP fixed
                NOP                      it for me. But I'll put in 4,
                NOP                      just to be on the save side.
                MOV     A,P1            Get the BCD bits from the counter
                ANL     A,#%0000.1111   Mask off irrelevant bits
                RET

.DECADE_MASK    .DA     #%0000.0100     Select LSD (10 of IC107)
                .DA     #%0000.0101     Select 2nd digit (100 of IC107)
                .DA     #%0000.0110     Select 3d digit (1000 of IC107)
                .DA     #%0000.0111     Select 4th digit (10000 of IC107)
                .DA     #%0000.1000     Select 5th digit (10 of IC108)
                .DA     #%0000.1001     Select 6th digit (100 of IC108)
                .DA     #%0000.1010     Select 7th digit (1000 of IC108)
                .DA     #%0000.1011     Select 8th digit (10000 of IC108)

;---------------------------------------------------------------------------
;   ROUTINE    :  CNT_NORMALISE
;   FUNCTION   :  Normalise both counters
;---------------------------------------------------------------------------

CNT_NORMALISE   MOV     R2,#12          Normalise a maximum of 12 digits
                MOV     B0_R3,B0_R0     Save begin address
                MOV     A,R0            Point to left most digit pair
                ADD     A,#5
                MOV     R1,A
.LOOP           MOV     B0_R0,B0_R3     Restore Accu address
                MOV     A,@R1           See if left most digit pair <> 0
                JNZ     .RET            Yes! Normalising done!
                CALL    FP_SHIFT_LEFT   Shift the number 1 digit to the left
                DEC     R7              Decrement exponent (keep same value)
                DJNZ    R2,.LOOP        Do this a maximum of 12 times
                CLR     A               Indicate that normalising failed
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  MODIFY_VALUE
;   FUNCTION   :  Modify the reading according to scaling and offset
;---------------------------------------------------------------------------

MODIFY_VALUE    JB      FUNC_PRIORITY,.SCALE1ST Do Scale calculation first!
                JNB     OFFSET_ACTIVE,.SCALE    Do only Scale, if given!
                CALL    ADD_OFFSET              Add offset to C
                JB      SUB_ZERO,.RET           Result is now 0! We're done
.SCALE          JNB     SCALE_ACTIVE,.RET       Scale is not active!
                CALL    CALC_SCALE              Calculate scale to C
.RET            RET
.SCALE1ST       JNB     SCALE_ACTIVE,.OFFSET    Do only offset, if given!
                CALL    CALC_SCALE              Calculate scale to C
.OFFSET         JNB     OFFSET_ACTIVE,.RET      Offset is not active!
                CALL    ADD_OFFSET              Add offset to C
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  ADD_OFFSET
;   FUNCTION   :  Add or subtract the given offset to Accu C
;---------------------------------------------------------------------------

ADD_OFFSET      MOV     DPTR,#OFFSET_VALUE      Copy offset value to Accu A
                MOV     R0,#FP_ACCU_A
                CALL    MEM_2_FP_ACCU
                MOV     A,FP_EXP_A              Add 10 to exponent to avoid
                ADD     A,#10                    negative values
                MOV     FP_EXP_A,A
                MOV     A,FP_EXP_C
                ADD     A,#10
                MOV     FP_EXP_C,A
                JB      OFFSET_POL,.SUB         Subtract A from C
                MOV     R0,#FP_ACCU_C           Add A to C
                MOV     R1,#FP_ACCU_A
                CALL    ADD_FP_NORM
                SJMP    .RET
.SUB            MOV     R0,#FP_ACCU_C           Subtract A from C
                MOV     R1,#FP_ACCU_A
                CALL    SUB_FP_NORM
                JB      SUB_SIGN,.RET           Result is positive!
                SETB    SUB_ZERO                Make negative results =0
                CALL    CLR_FP_ACCU
.RET            CLR     C                       Subtract epxonent offset
                MOV     A,FP_EXP_C               to get propper value back
                SUBB    A,#10
                MOV     FP_EXP_C,A
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  CALC_SCALE
;   FUNCTION   :  Multiply or divide Scale to Accu C
;---------------------------------------------------------------------------

CALC_SCALE      MOV     DPTR,#SCALE_VALUE       Copy scale value to Accu A
                MOV     R0,#FP_ACCU_A
                CALL    MEM_2_FP_ACCU
                JB      SCALE_POL,.DIV          Divide C through A?
                CALL    MUL_FP_NORM             Multiply Accu A to C
                RET
.DIV            MOV     R0,#FP_ACCU_C           Divide C through A
                MOV     R1,#FP_ACCU_A
                CALL    DIV_FP_NORM
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  DISPLAY_SPEED
;   FUNCTION   :  Display the speed (4 digits) and the % deviation
;---------------------------------------------------------------------------

DISPLAY_SPEED   MOV     A,FP_EXP_C      See if value is within range
                JB      ACC.7,.TOO_LOW  Value is too low
                >CMP    A,#3+1          Exponent must be 0...3
                JNC     .TOO_HIGH       Value too high
.EXP3           MOV     A,FP_EXP_C      Make exponent 3
                XRL     A,#3
                JZ      .EXP3_DONE      Ready
                MOV     R0,#FP_ACCU_C   Shift Accu C right 1 digit
                CALL    FP_SHIFT_RIGHT
                INC     FP_EXP_C        Increment exponent
                SJMP    .EXP3
.EXP3_DONE      CALL    .FREQ2DSP       Put frequency in display

                CALL    CALC_PERCENTAGE Calculate percentage deviation
                MOV     DSP_BUFFER+4,#0 In case it's a positive number
                MOV     A,FP_ACCU_C+5   Get sign
                JB      ACC.7,.POS      It is positive!
                MOV     DSP_BUFFER+4,#SG Show minus sign
.POS            MOV     A,FP_EXP_C      Exponent must be <=1
                JB      ACC.7,.PERC_EXP_OK  It is! (for it's negative)
                >CMP    A,#1+1
                JNC     .PERC_EXP_OL    It's not! Now it's too high
.PERC_EXP_OK    DEC     A               Was exponent 1?
                CLR     C
                JZ      .PERC_EXP_1     Yes! Show Tens and Units then
                INC     A               Was exponent 0?
                SETB    C
                JZ      .PERC_EXP_1     Yes! Show Units and Tenths
                INC     A               Was exponent -1? (0.x)
                JNZ     .PERC_EXP_0     No! Number is too small show 0.0
                MOV     A,FP_ACCU_C+5   Shift MSD 1 position right
                SWAP    A
                MOV     FP_ACCU_C+4,A
                MOV     FP_ACCU_C+5,#0
                SJMP    .PERC_EXP_1     Skip 0 load
.PERC_EXP_0     MOV     FP_ACCU_C+5,#0  Show 0.0
                MOV     FP_ACCU_C+4,#0
.PERC_EXP_1     MOV     A,FP_ACCU_C+5   Get MSD and put it in the display
                CALL    BCD_2_7SEGM
                MOV     ACC.7,C         Set or clear DP
                MOV     DSP_BUFFER+5,A
                MOV     A,FP_ACCU_C+4   Get LSD and put it in the display
                SWAP    A
                CALL    BCD_2_7SEGM
                MOV     DSP_BUFFER+6,A
.PERC_OK        MOV     DSP_BUFFER+7,#0 Clear annunciators
                MOV     C,GATE_FLAG     Copy gate flag to DP7
                MOV     A,DSP_BUFFER+6
                MOV     ACC.7,C
                MOV     DSP_BUFFER+6,A
                CALL    UPDATE_DSP
                RET

.TOO_LOW        MOV     DPTR,#.TOO_LOW_DSP      Show LO in display
                SJMP    .SKIP_HIGH
.TOO_HIGH       MOV     DPTR,#.TOO_HIGH_DSP     Show OL in display
.SKIP_HIGH      MOV     C,GATE_FLAG             Show gate flag
                MOV     A,DSP_BUFFER+6
                MOV     ACC.7,C
                MOV     DSP_BUFFER+6,A
                CLR     A
                JMP     SHOW_TEXT

.PERC_EXP_OL    MOV     DSP_BUFFER+5,#CHR_O     Show oL in perc. display
                MOV     DSP_BUFFER+6,#CHR_L
                SJMP    .PERC_OK

.FREQ2DSP       MOV     R0,#FP_ACCU_C+3 Round value to 4 digits, not with
                MOV     A,@R0            normal round routine for that
                ADD     A,#$05           normalises
                DA      A
                MOV     @R0,A
                INC     R0
                MOV     A,@R0
                ADDC    A,#0
                DA      A
                MOV     @R0,A
                INC     R0
                MOV     A,@R0
                ADDC    A,#0
                DA      A
                MOV     @R0,A
                ANL     A,#%1111.0000   Is high nibble of result 0?
                JZ      .NOSHIFT        Yes!
                MOV     R0,#FP_ACCU_C   Shift FP accu to the right
                CALL    FP_SHIFT_RIGHT
.NOSHIFT        MOV     R0,#DSP_BUFFER  Put frequency in display
                MOV     R1,#FP_ACCU_C+5 Starting with leftmost digit
                MOV     R2,#4           Do 4 digits
                CLR     F0              Start with LSD of byte
.COPY_DSP       MOV     A,@R1           Get digit
                JNB     F0,.SKIP_SWAP   Don't swap now
                SWAP    A
.SKIP_SWAP      CALL    BCD_2_7SEGM     Convert to 7 segments
                MOV     @R0,A
                INC     R0
                JBC     F0,.SKIP_INC    MSD done! Now do LSD of byte
                SETB    F0              Point to MSD of next pair
                DEC     R1
.SKIP_INC       DJNZ    R2,.COPY_DSP
                RET

.TOO_LOW_DSP    .DA     #CHR_0,#CHR_0,#CHR_0,#CHR_0,#SG,#CHR_O,#CHR_L
.TOO_HIGH_DSP   .DA     #BLANK,#CHR_O,#CHR_L,#BLANK,#BLANK,#CHR_O,#CHR_L
