;---------------------------------------------------------------------------
;
;   DEC.ASM
;
;   Include file for SB6668 Frequency counter
;
;   Contains all memeory, constants and macro definitions
;
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
;  Constants
;---------------------------------------------------------------------------

TUNE_TIME       .EQ     209             Tune timer for 1ms
CR_DELAY_TIME   .EQ     5               Delaytime after CR is 5ms
BAUD_RELOAD     .EQ     -13             Reload value T1 for 9600 baud
STACK_BEGIN     .EQ     $7F             Begin address of stack

BS              .EQ     $08             Back space
LF              .EQ     $0A             Line feed
CR              .EQ     $0D             Carriage return
XON             .EQ     $11             Xon
XOFF            .EQ     $13             Xoff
CANCEL          .EQ     $18             Cancel last transmission
ESC             .EQ     $1B             Escape
DEL             .EQ     $7F             Delete

NVM_WRITE_ADDR  .EQ     $A0               NVM write address
NVM_READ_ADDR   .EQ     NVM_WRITE_ADDR+1  NVM read address

MAX_ACK_ERRORS  .EQ     10              Maximum acknowledge errors

TRIG_DELAY      .EQ     350   ms        Time to wait for 1st trigger pulse
FAST_DELAY      .EQ     200   ms        Time to wait for fast measure rate
SLOW_DELAY      .EQ     1000  ms        Time to wait for slow measure rate
STOP_DELAY      .EQ     1500  ms        Time to wait for last trigger pulse

;---------------------------------------------------------------------------
;  Display constants
;---------------------------------------------------------------------------

SA              .EQ     %0000.0100      Display segment a
SB              .EQ     %0000.1000      Display segment b
SC              .EQ     %0100.0000      Display segment c
SD              .EQ     %0010.0000      Display segment d
SE              .EQ     %0001.0000      Display segment e
SF              .EQ     %0000.0010      Display segment f
SG              .EQ     %0000.0001      Display segment g
SDP             .EQ     %1000.0000      Display segment dp

CHR_0           .EQ     SA+SB+SC+SD+SE+SF
CHR_1           .EQ     SB+SC
CHR_2           .EQ     SA+SB+SD+SE+SG
CHR_3           .EQ     SA+SB+SC+SD+SG
CHR_4           .EQ     SB+SC+SF+SG
CHR_5           .EQ     SA+SC+SD+SF+SG
CHR_6           .EQ     SA+SC+SD+SE+SF+SG
CHR_7           .EQ     SA+SB+SC
CHR_8           .EQ     SA+SB+SC+SD+SE+SF+SG
CHR_9           .EQ     SA+SB+SC+SD+SF+SG

CHR_A           .EQ     SA+SB+SC+SE+SF+SG
CHR_B           .EQ     SC+SD+SE+SF+SG
CHR_C           .EQ     SD+SE+SG
CHR_D           .EQ     SB+SC+SD+SE+SG
CHR_E           .EQ     SA+SD+SE+SF+SG
CHR_F           .EQ     SA+SE+SF+SG
CHR_G           .EQ     SA+SC+SD+SE+SF
CHR_H           .EQ     SC+SE+SF+SG
CHR_I           .EQ     SE
CHR_J           .EQ     SB+SC+SD
CHR_K           .EQ     SA+SC+SE+SF+SG
CHR_L           .EQ     SD+SE+SF
CHR_M           .EQ     SA+SB+SC+SE+SF
CHR_N           .EQ     SC+SE+SG
CHR_O           .EQ     SC+SD+SE+SG
CHR_P           .EQ     SA+SB+SE+SF+SG
CHR_Q           .EQ     SA+SB+SC+SF+SG
CHR_R           .EQ     SE+SG
CHR_S           .EQ     SA+SC+SD+SF+SG
CHR_T           .EQ     SD+SE+SF+SG
CHR_U           .EQ     SC+SD+SE
CHR_V           .EQ     SC+SD+SE
CHR_W           .EQ     SB+SC+SD+SE+SF
CHR_X           .EQ     SB+SC+SE+SF+SG
CHR_Y           .EQ     SB+SC+SD+SF+SG
CHR_Z           .EQ     SA+SB+SD+SE+SG
BLANK           .EQ     0

CHR_IS          .EQ     SD+SG           =
CHR_QST         .EQ     SA+SB+SE+SG     Question mark
CHR_SL          .EQ     SB+SE+SG        /

;---------------------------------------------------------------------------
;  Absolute addresses of registers
;---------------------------------------------------------------------------

B0_R0           .EQ     $00
B0_R1           .EQ     $01
B0_R2           .EQ     $02
B0_R3           .EQ     $03
B0_R4           .EQ     $04
B0_R5           .EQ     $05
B0_R6           .EQ     $06
B0_R7           .EQ     $07
B1_R0           .EQ     $08
B1_R1           .EQ     $09
B1_R2           .EQ     $0A
B1_R3           .EQ     $0B
B1_R4           .EQ     $0C
B1_R5           .EQ     $0D
B1_R6           .EQ     $0E
B1_R7           .EQ     $0F
B2_R0           .EQ     $10
B2_R1           .EQ     $11
B2_R2           .EQ     $12
B2_R3           .EQ     $13
B2_R4           .EQ     $14
B2_R5           .EQ     $15
B2_R6           .EQ     $16
B2_R7           .EQ     $17
B3_R0           .EQ     $18
B3_R1           .EQ     $19
B3_R2           .EQ     $1A
B3_R3           .EQ     $1B
B3_R4           .EQ     $1C
B3_R5           .EQ     $1D
B3_R6           .EQ     $1E
B3_R7           .EQ     $1F

;---------------------------------------------------------------------------
;  Bit addresses
;---------------------------------------------------------------------------

; byte $20      SB-Bus flags (part 1)
SB_SELECT       .EQ     $00             Selected if 1
SB_GENERAL      .EQ     $01             Selected by general call if 1
SOUT_ACTIVE     .EQ     $02             Serial output active if 1
STOP_ACT        .EQ     $03             Stop all activities if 1
SOUT_HALTED     .EQ     $04             Serial output halted if 1
HOLD_ACTIVE     .EQ     $05             Hold mode active if 1
SLOW_MODE       .EQ     $06             Slow mode if 1
FLOW_MODE       .EQ     $07             Acknowledge mode if 1

; byte $21      SB_Bus flags (part 2)
REMOTE_MODE     .EQ     $08             Remote mode if 1
FORMAT          .EQ     $09             0=normal, 1=scientific notation
IIC_ERROR       .EQ     $0A             IC write error if 1
SYNC_FLAG       .EQ     $0B             Used for SYNC command

; byte $22      Counter flags
COUNTER_FLAGS   .EQ     $22
INPUT_SELECT    .EQ     $10             LF input if 1, HF input if 0
MEASURE_RATE    .EQ     $11             Measureing rate slow=0, fast=1
ACTUAL_RATE     .EQ     $12             Actual measuring rate of reading
ONLY6           .EQ     $13             Only 6 digits displayed if set
GATE_FLAG       .EQ     $14             Flag inverts every new start
PM6667          .EQ     $15             It's a PM6667 if set (No HF input)
REF_SOURCE      .EQ     $16             Reference source (0 is internal)
OPTION_EXT_REF  .EQ     $17             External reference possible if 0

; byte $23      Keyboard & misc. flags
KEY_FLAGS       .EQ     $23             Keyboard flags
LAST_KEY_STATE  .EQ     $18             Last key state (0=out, 1=in)
KEY_STATE       .EQ     $19             Debounced keystate (0=out, 1=in)
MENU_MODE       .EQ     $1A             Menu is on if 1
DP_FLAG         .EQ     $1B             DP entered in FP number
VALID_FP        .EQ     $1C             At least something valid was entered
LEAD_0          .EQ     $1D             Still leading 0 if 0
SUB_SIGN        .EQ     $1E             Sign after subtraction
SUB_ZERO        .EQ     $1F             =1 If subtraction result is 0

; byte 24       Function flags
FUNCTION_FLAGS  .EQ     $24             Counter function flags
SPEED_FUNCTION  .EQ     $20             Speed function active if 1
SPEED_REFERENCE .EQ     $21             Speed reference; 0=CCIR, 1=DIN
OFFSET_ACTIVE   .EQ     $22             Offset calculation active if 1
OFFSET_POL      .EQ     $23             Offset polarity; 0=ADD, 1=SUB
SCALE_ACTIVE    .EQ     $24             Scaling calculation active if 1
SCALE_POL       .EQ     $25             Scaling polarity; 0=MUL, 1=DIV
FUNC_PRIORITY   .EQ     $26             0=Offset:Scale, 1=Scale:Offset

;---------------------------------------------------------------------------
;  Data memory allocation
;---------------------------------------------------------------------------

                .OR     $0030           Start behind bit addressable bytes
                .DU
MY_ADDRESS      .BS     1               My own address
SIN_HEAD        .BS     1               Serial input head pointer
SIN_TAIL        .BS     1               Serial input tail pointer
SOUT_HEAD       .BS     1               Serial output head pointer
SOUT_TAIL       .BS     1               Serial output tail pointer
CR_COUNT        .BS     1               CR counter
CR_DELAY        .BS     1               CR delay counter
INLENGTH        .BS     1               Input line length
LAST_SEND       .BS     1               Last character send over bus
ERROR           .BS     1               Last error number
HOLD_FUNCTION   .BS     2               Hold function number
REPEAT_CMD      .BS     2               Repeat function
DOG_BARKED      .BS     1               Number of watchdog resets
LAST_CHAR       .BS     1               Last character fetched from input
PARSE_SAVE      .BS     2               Temporary save of parse string
SAVE_TEXT       .BS     2               Used to save the text pointer
PARAMETERS      .BS     6               Parameter stack
NVM_BUFFER      .BS     8               NVM memory buffer
DSP_BUFFER      .BS     8               Display buffer (1,2,3,4,5,6,7,ann)
TIME_DELAY      .BS     2               Gerenal delay timer (LSB 1st)
COUNTER_STAT    .BS     1               Counter status register
REF_EXTEND      .BS     1               Reference counter extend byte
FP_ACCU_A       .BS     6               Floating point ACCU A (LSD first)
FP_EXP_A        .BS     1               Exponent of ACCU A
FP_ACCU_B       .BS     6               Floating point ACCU B
FP_EXP_B        .BS     1               Exponent of ACCU B
FP_ACCU_C       .BS     6               Floating point ACCU C
FP_EXP_C        .BS     1               Exponent of ACCU C
MENU_PNTR       .BS     2               Menu pointer (index to routine)
MENU_INDEX      .BS     1               Step in menu carousel

                .DO     *>$7F
                .ER     Out of direct accessible RAM memory!
                .FI

                .ED

;---------------------------------------------------------------------------
;  SFR definition
;---------------------------------------------------------------------------

P0              .EQ     $80             I/O port 0
SP              .EQ     $81             Stack pointer
DPL             .EQ     $82             Data pointer Low
DPH             .EQ     $83             Data pointer High
;                                       DP0 is general purpose
;                                       DP1 is serial input buffer
;                                       DP2 is serial output buffer
;                                       DP3 is serial input parse pointer
WDTL            .EQ     $84             Watchdog timer Low
WDTH            .EQ     $85             Watchdog timer High
WDTREL          .EQ     $86             Watchdog timer reload
PCON            .EQ     $87             Power control
TCON            .EQ     $88             Timer control
TMOD            .EQ     $89             Timer mode
TL0             .EQ     $8A             Timer 0 Low
TL1             .EQ     $8B             Timer 1 Low
TH0             .EQ     $8C             Timer 0 High
TH1             .EQ     $8D             Timer 1 High
P1              .EQ     $90             I/O port 1
MEX1            .EQ     $94             Memory extension control 1
MEX2            .EQ     $95             Memory extension control 2
SCON            .EQ     $98             Serial interface control
SBUF            .EQ     $99             Serial interface doorway
P2              .EQ     $A0             I/O port 2
DPSEL           .EQ     $A2             Data pointer selector
AFR             .EQ     $A6             Advanced function register
IE              .EQ     $A8             Interrupt enable flags
IP              .EQ     $A9             Interrupt priority control
P3              .EQ     $B0             I/O port 3
WDSTART         .EQ     $B8             Watchdog timer start
PWME            .EQ     $C0             PWM enable bits
PWMC            .EQ     $C8             PWM control
PSW             .EQ     $D0             Program status word
ADCON           .EQ     $D8             ADC control
ADDAT           .EQ     $D9             ADC data
DAPR            .EQ     $DA             ADC start register
ACC             .EQ     $E0             The Accumulator
P4              .EQ     $E8             I/O port 4
B               .EQ     $F0             The auxiliary Accumulotor B
PWCOMP0         .EQ     $F1             PWM comparator for bit 0
PWCOMP1         .EQ     $F2             PWM comparator for bit 1
PWCOMP2         .EQ     $F3             PWM comparator for bit 2
PWCOMP3         .EQ     $F4             PWM comparator for bit 3
PWCOMP4         .EQ     $F5             PWM comparator for bit 4
PWCOMP5         .EQ     $F6             PWM comparator for bit 5
PWCOMP6         .EQ     $F7             PWM comparator for bit 6
PWCOMP7         .EQ     $F8             PWM comparator for bit 7
PWCOUNT         .EQ     $F9             PWM reference counter

;---------------------------------------------------------------------------
;  Special function flags
;---------------------------------------------------------------------------

; TCON
IT0             .EQ     $88             INT0 trigger mode; edge=1 / level=0
IE0             .EQ     $89             INT0 was triggered by falling edge
IT1             .EQ     $8A             INT1 trigger mode; edge=1 / level=0
IE1             .EQ     $8B             INT1 was triggered by falling edge
TR0             .EQ     $8C             Timer 0 run flag
TF0             .EQ     $8D             Timer 0 overflow flag
TR1             .EQ     $8E             Timer 1 run flag
TF1             .EQ     $8F             Timer 1 overflow flag

; SCON
RI              .EQ     $98             Receive interrupt flag
TI              .EQ     $99             Transmit interrupt flag
RB8             .EQ     $9A             9th received databit
TB8             .EQ     $9B             9th transmitted databit
REN             .EQ     $9C             Receive enable
SM2             .EQ     $9D             Serial mode bit 2
SM1             .EQ     $9E             Serial mode bit 1
SM0             .EQ     $9F             Serial mode bit 0

; IE
EX0             .EQ     $A8             Enable external interrupt 0
ET0             .EQ     $A9             Enable timer 0 overflow interrupt
EX1             .EQ     $AA             Enable external interrupt 1
ET1             .EQ     $AB             Enable timer 1 overflow interrupt
ES              .EQ     $AC             Enable serial interrupt
WDT             .EQ     $AE             Watchdog timer refresh flag
EA              .EQ     $AF             Global interrupt enable flag

; WDSTART
SWDT            .EQ     $BE             Watchdog timer start flag

; PWME
E0              .EQ     $C0             PWM bit 0 enabled
E1              .EQ     $C1             PWM bit 1 enabled
E2              .EQ     $C2             PWM bit 2 enabled
E3              .EQ     $C3             PWM bit 3 enabled
E4              .EQ     $C4             PWM bit 4 enabled
E5              .EQ     $C5             PWM bit 5 enabled
E6              .EQ     $C6             PWM bit 6 enabled
E7              .EQ     $C7             PWM bit 7 enabled

; PWMC
R               .EQ     $CC             PWM counter Run/Stop flag
M0              .EQ     $CD             PWM mode control bit 0
M1              .EQ     $CE             PWM mode control bit 1
S               .EQ     $CF             Overflow state of PWM timer

; PSW
P               .EQ     $D0             Parity flag
F1              .EQ     $D1             User flag
OV              .EQ     $D2             Overflow flag
RS0             .EQ     $D3             Register select bit 0
RS1             .EQ     $D4             Register select bit 1
F0              .EQ     $D5             User flag
AC              .EQ     $D6             Auxiliary carry flag
CY              .EQ     $D7             Carry flag

; ADCON
MX0             .EQ     $D8             ADC channel select bit 0
MX1             .EQ     $D9             ADC channel select bit 1
ADM             .EQ     $DB             ADC mode; 0=single, 1=coninuous
BSY             .EQ     $DC             ADC busy flag; 1=busy
PSC             .EQ     $DF             Prescaler control

;---------------------------------------------------------------------------
;  XRAM addresses
;---------------------------------------------------------------------------

SIN_BUFFER      .EQ     $F800,$F8FF     256 bytes of serial input buffer
SOUT_BUFFER     .EQ     $F900,$F9FF     256 bytes of serial output buffer
VALUE_TABLE     .EQ     $FA00,$FAF9     50 times 5 bytes of values
SB_FREQUENCY    .EQ     $FB00,$FB06     Unmodified measured frequency
SB_DISPLAY      .EQ     $FB07,$FB0D     Modified measured frequency
OFFSET_VALUE    .EQ     $FB0E,$FB12     Preset offset value (LSD,,,MSD,exp)
SCALE_VALUE     .EQ     $FB13,$FB17     Preset scale value  (LSD,,,MSD,exp)
PERCENT_VALUE   .EQ     $FB18,$FB1E     Percentage  (sign in MSD bit 7)
HOLD_FREQ       .EQ     $FB1F,$FB25     Hold unmodified frequency
HOLD_DISP       .EQ     $FB26,$FB2C     Hold modified frequency
HOLD_PERC       .EQ     $FB2D,$FB33     Hold percentage value

;---------------------------------------------------------------------------
;  NVM memory addresses
;---------------------------------------------------------------------------

                .OR     $0000           Start with 1st address
                .DU
NVM_ID          .BS     4               ID filled with '6668'
NVM_MY_ADDRESS  .BS     1               My SB-Bus address
NVM_OPTIONS     .BS     1               Options byte
;                                       b0; SB6668 if 0, SB6667 if 1
;                                       b1; ext.ref if 0, no ext.ref = 1
;                                       b2; memory lost reported if 1
NVM_ARRAY       .BS     5*50            50 Scaling/Offset values
;                                       byte 0..3 = 7 digit mantissa
;                                        D6D7,D4D5,D2D3,00D1
;                                       byte 4 is exponent and flag byte
;                                        b7=0 scaling, b7=1 offset
;                                       no entry if all bytes are $FF

                .ED

;---------------------------------------------------------------------------
;  MACRO     : PUSH
;  PARAMETERS: MEM1[,MEM2[,MEM3[,........MEM10]]]]]]]]]
;
;  This Macro can be used to push 1 to 10 different memory locations to
;  the stack. The memory locations are stacked in the order of appearance
;  in the parameter list.
;---------------------------------------------------------------------------

PUSH            .MA     MEM1,MEM2,MEM3,MEM4,.....MEM10
                PUSH    ]1
                .XM     ]#=1
                PUSH    ]2
                .XM     ]#=2
                PUSH    ]3
                .XM     ]#=3
                PUSH    ]4
                .XM     ]#=4
                PUSH    ]5
                .XM     ]#=5
                PUSH    ]6
                .XM     ]#=6
                PUSH    ]7
                .XM     ]#=7
                PUSH    ]8
                .XM     ]#=8
                PUSH    ]9
                .XM     ]#=9
                PUSH    ]0
                .EM

;---------------------------------------------------------------------------
;  MACRO     : POP
;  PARAMETERS: MEM1[,MEM2[,MEM3[,........MEM10]]]]]]]]]
;
;  This Macro can be used to pull 1 to 10 different memory locations from
;  the stack. The memory locations are pulled in the order of appearance
;  in the parameter list.
;---------------------------------------------------------------------------

POP             .MA     MEM1,MEM2,MEM3,MEM4,.....MEM10
                POP     ]1
                .XM     ]#=1
                POP     ]2
                .XM     ]#=2
                POP     ]3
                .XM     ]#=3
                POP     ]4
                .XM     ]#=4
                POP     ]5
                .XM     ]#=5
                POP     ]6
                .XM     ]#=6
                POP     ]7
                .XM     ]#=7
                POP     ]8
                .XM     ]#=8
                POP     ]9
                .XM     ]#=9
                POP     ]0
                .EM

;---------------------------------------------------------------------------
;  MACRO     : CMP
;  PARAMETERS: SOURCE,#DATA
;
;  C=0 if SOURCE>=#DATA
;  C=1 if SOURCE<#DATA
;---------------------------------------------------------------------------

CMP             .MA     SOURCE,#DATA
                CJNE    ]1,]2,*+3
                .EM

;---------------------------------------------------------------------------
;  MACRO     : GETC
;  PARAMETERS: [DPSEL]
;
;  Get a byte from constant memory, pointed to by DPTR.
;  DPSEL is only loaded prior to the get if DPSEL is given as parameter
;---------------------------------------------------------------------------

GETC            .MA     DPSEL
                .DO     ]#=1
                MOV     DPSEL,#]1
                .FI
                CLR     A
                MOVC    A,@A+DPTR
                .EM
