;---------------------------------------------------------------------------
;
;   MENU.ASM
;
;   Include file for SB6668 Frequency counter
;
;   Contains all Menu related routines
;
;---------------------------------------------------------------------------

MENU_SPEED      .EQ     1200            Time per menu step in ms
MENU_WAIT_TIME  .EQ     5000            Maximum delay to wait for new key
MENU_DONE_TIME  .EQ      500            Delay to show DONE in display

;---------------------------------------------------------------------------
;   ROUTINE    :  POLL_MENU
;   FUNCTION   :  Main entry for the menu routine
;---------------------------------------------------------------------------

POLL_MENU       JB      MENU_MODE,.ACTIVE  Menu is already active!
                JB      REMOTE_MODE,.RET   Don't bother if remote mode!
                JNB     KEY_STATE,.RET     Key is not pressed, forget it!
                SETB    MENU_MODE          Set menu mode
                SETB    P0.7               Stop the counter
                MOV     TIME_DELAY,#MENU_SPEED Setup timeout
                MOV     TIME_DELAY+1,/MENU_SPEED
                MOV     MENU_INDEX,#0      Clear index in main menu
                MOV     MENU_PNTR,#MAIN_MENU   Main menu with key in
                MOV     MENU_PNTR+1,/MAIN_MENU
                CLR     A                  Show 1st menu item on display
                MOV     DPTR,#MAIN_MENU_TEXT
                CALL    SHOW_TEXT
.RET            RET

.ACTIVE         JNB     REMOTE_MODE,.DO_IT OK to listen to the keys
                CLR     MENU_MODE          Clear menu mode
                JMP     RETRY_MEASURE      Restart the measuring

.DO_IT          CLR     ONLY6              Always display 7 digits in menu
                PUSH    MENU_PNTR          Push pointer on the stack
                PUSH    MENU_PNTR+1
                RET                         and execute the routine

;---------------------------------------------------------------------------
;   ROUTINE    :  MENU_DONE
;   FUNCTION   :  Menu done, show that on the display
;---------------------------------------------------------------------------

MENU_DONE       CLR     A               Show done on the display
                MOV     DPTR,#.DONE_TEXT
                CALL    SHOW_TEXT
                MOV     TIME_DELAY,#MENU_DONE_TIME Setup delay timer
                MOV     TIME_DELAY+1,/MENU_DONE_TIME
                CLR     MENU_MODE       Clear menu mode flag
                MOV     COUNTER_STAT,#0 Resume counting after delay
                RET

.DONE_TEXT      .DA     #CHR_D,#CHR_O,#CHR_N,#CHR_E,#BLANK,#BLANK,#BLANK

;---------------------------------------------------------------------------
;   ROUTINE    :  MAIN_MENU
;   FUNCTION   :  Main menu loop, while key is down
;---------------------------------------------------------------------------

MAIN_MENU       JB      KEY_STATE,.DELAY Key is still down!
                MOV     A,MENU_INDEX    Show action to be taken
                MOV     DPTR,#MAIN_MENU_END
                CALL    SHOW_TEXT
                MOV     TIME_DELAY,#MENU_WAIT_TIME  Setup delay time to
                MOV     TIME_DELAY+1,/MENU_WAIT_TIME  wait for keypress
                MOV     DPTR,#.MAIN_MENU_JMPS Setup jump table
                MOV     A,MENU_INDEX    Get index in table
                RL      A
                MOV     B,A             Save for later
                MOVC    A,@A+DPTR       Get high byte first
                MOV     MENU_PNTR+1,A
                MOV     A,B             Get low byte too
                INC     A
                MOVC    A,@A+DPTR
                MOV     MENU_PNTR,A
                RET

.DELAY          MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Keep waiting
                INC     MENU_INDEX      Increment menu index
                JNB     OPTION_EXT_REF,.EXTREF  Do external reference too
                MOV     A,MENU_INDEX    See if we're at reference source
                XRL     A,#.MAIN_REF_ITEM
                JNZ     .EXTREF         No!
                INC     MENU_INDEX      Increment index once more
.EXTREF         MOV     A,MENU_INDEX    See if index out of range
                >CMP    A,#.MAIN_ITEMS
                JC      .INDEX_OK       Index is OK!
                CLR     A               Wrap index around
                MOV     MENU_INDEX,A
.INDEX_OK       MOV     TIME_DELAY,#MENU_SPEED Setup timeout
                MOV     TIME_DELAY+1,/MENU_SPEED
                MOV     A,MENU_INDEX    Show next menu item on display
                MOV     DPTR,#MAIN_MENU_TEXT
                CALL    SHOW_TEXT
.RET            RET

.MAIN_ITEMS     .EQ     6

.MAIN_MENU_JMPS .DA     RATE_MENU_WAIT   Wait for RATE setup keypress
                .DA     RESET_MENU       Reset the modifiers
                .DA     OFFST_MENU_WAIT  Wait for OFFSET menu keypress
                .DA     SCALE_MENU_WAIT  Wait for SCALE menu keypress
                .DA     SPEED_MENU_WAIT  Wait for SPEED menu keypress
                .DA     REF_MENU_WAIT    Wait for REF menu keypress
.MAIN_REF_ITEM  .EQ     5

MAIN_MENU_TEXT  .DA     #CHR_R,#CHR_A,#CHR_T,#CHR_E,#BLANK,#BLANK,#BLANK
                .DA     #CHR_R,#CHR_E,#CHR_S,#CHR_E,#CHR_T,#BLANK,#BLANK
                .DA     #CHR_O,#CHR_F,#CHR_F,#CHR_S,#CHR_E,#CHR_T,#BLANK
                .DA     #CHR_S,#CHR_C,#CHR_A,#CHR_L,#CHR_E,#BLANK,#BLANK
                .DA     #CHR_S,#CHR_P,#CHR_E,#CHR_E,#CHR_D,#BLANK,#BLANK
                .DA     #CHR_R,#CHR_E,#CHR_F+SDP,#CHR_S,#CHR_R,#CHR_C,#BLANK

MAIN_MENU_END   .DA     #CHR_R,#CHR_A,#CHR_T,#CHR_E,#CHR_IS,#CHR_QST,#BLANK
                .DA     #CHR_D,#CHR_O,#CHR_N,#CHR_E,#BLANK,#BLANK,#BLANK
                .DA     #CHR_O,#CHR_F,#CHR_F,#CHR_S,#CHR_E,#CHR_T,#CHR_QST
                .DA     #CHR_S,#CHR_C,#CHR_A,#CHR_L,#CHR_E,#CHR_IS,#CHR_QST
                .DA     #CHR_N,#CHR_O,#CHR_R,#CHR_M,#CHR_IS,#CHR_QST,#BLANK
                .DA     #CHR_R,#CHR_E,#CHR_F+SDP,#CHR_S,#CHR_R,#CHR_C,#CHR_QST

;---------------------------------------------------------------------------
;   ROUTINE    :  RATE_MENU_WAIT
;   FUNCTION   :  Wait for RATE setup keypress
;---------------------------------------------------------------------------

RATE_MENU_WAIT  MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                CLR     A               Get old measure rate
                MOV     C,MEASURE_RATE  Will be inverted immediatly
                MOV     ACC.0,C
                MOV     MENU_INDEX,A
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#RATE_MENU_SET Setup rate menu pointer
                MOV     MENU_PNTR+1,/RATE_MENU_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  RATE_MENU_SET
;   FUNCTION   :  Key for RATE menu is pressed
;---------------------------------------------------------------------------

RATE_MENU_SET   JNB     KEY_STATE,.DO_IT   Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
                XRL     MENU_INDEX,#%0000.0001  Invert index value
                MOV     DPTR,#.RATE_TEXT Setup RATE text table pointer
                MOV     A,MENU_INDEX    Get offset in table
                CALL    SHOW_TEXT       Show the text
.RET            RET
.DO_IT          MOV     A,MENU_INDEX    Get index and copy it to the
                MOV     C,ACC.0           rate flag
                MOV     MEASURE_RATE,C
                CLR     ONLY6           Clear only 6 digit flag
                JMP     MENU_DONE       Ready with menu

.RATE_TEXT      .DA     #CHR_S,#CHR_L,#CHR_O,#CHR_W,#BLANK,#BLANK,#BLANK
                .DA     #CHR_F,#CHR_A,#CHR_S,#CHR_T,#BLANK,#BLANK,#BLANK

;---------------------------------------------------------------------------
;   ROUTINE    :  RESET_MENU
;   FUNCTION   :  Show Reset DONE message for 500ms
;---------------------------------------------------------------------------

RESET_MENU      CLR     MENU_MODE       We're not in menu mode anymore
                MOV     TIME_DELAY,#MENU_DONE_TIME  Show DONE a short while
                MOV     TIME_DELAY+1,/MENU_DONE_TIME
                MOV     COUNTER_STAT,#0 Start counting again after delay
                MOV     FUNCTION_FLAGS,#0 Reset all functions
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  OFFST_MENU_WAIT
;   FUNCTION   :  Wait for OFFSET setup keypress
;---------------------------------------------------------------------------

OFFST_MENU_WAIT MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                MOV     MENU_INDEX,#-1  Start showing the offset values
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#OFFSET_MENU_SET Point to propper function
                MOV     MENU_PNTR+1,/OFFSET_MENU_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  OFFSET_MENU_SET
;   FUNCTION   :  Show all possible offset values
;---------------------------------------------------------------------------

OFFSET_MENU_SET JNB     KEY_STATE,.DO_IT  Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
.SCALE_LOOP     CALL    INC_VALUE_INDEX Increment value index
                JNZ     .INDEX_VALUE    Index points to a value!
                MOV     DPTR,#.NONE_TEXT Show 'None' in the display
                CALL    SHOW_TEXT        Show the text
.RET            RET
.INDEX_VALUE    DEC     A               Multiply index with 5
                MOV     B,#5
                MUL     AB
                ADD     A,#VALUE_TABLE  Add value table address to offset
                MOV     DPL,A
                MOV     A,/VALUE_TABLE
                ADDC    A,#0
                MOV     DPH,A
                MOV     R0,#FP_ACCU_C   Point FP Accu C
                CALL    MEM_2_FP_ACCU
                JB      ACC.6,.SCALE_LOOP It's an empty memory place
                JNC     .SCALE_LOOP     It is a Scaling value!
                CALL    FP_C_2_DISPLAY  Show Offset value in FP_ACCU_C
                RET

.DO_IT          MOV     A,MENU_INDEX    Get index, see if it's 0
                JZ      .NONE           Switch offset value off

                MOV     DPTR,#.POL_TEXT Show 'polarit' in display
                CLR     A
                CALL    SHOW_TEXT
                MOV     TIME_DELAY,#MENU_WAIT_TIME  Setup delay time to
                MOV     TIME_DELAY+1,/MENU_WAIT_TIME  wait for keypress
                MOV     MENU_PNTR,#OFFSET_POL_WAIT    Setup polarity wait
                MOV     MENU_PNTR+1,/OFFSET_POL_WAIT   routine pointer
                RET

.NONE           CLR     OFFSET_ACTIVE   Switch offset mode off
                CLR     OFFSET_POL
                JNB     SCALE_ACTIVE,.DONE Don't care about priority!
                SETB    FUNC_PRIORITY   Scaling function has priority now!
.DONE           JMP     MENU_DONE       Ready with menu

.NONE_TEXT      .DA     #CHR_N,#CHR_O,#CHR_N,#CHR_E,#BLANK,#BLANK,#BLANK
.POL_TEXT       .DA     #CHR_P,#CHR_O,#CHR_L,#CHR_A,#CHR_R,#CHR_I,#CHR_T+SDP

;---------------------------------------------------------------------------
;   ROUTINE    :  OFFSET_POL_WAIT
;   FUNCTION   :  Wait for OFFSET polarity switch to be pressed
;---------------------------------------------------------------------------

OFFSET_POL_WAIT MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                MOV     MENU_INDEX,#1   Start menu with ADD
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#OFFSET_POL_SET  Point to propper function
                MOV     MENU_PNTR+1,/OFFSET_POL_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  OFFSET_POL_SET
;   FUNCTION   :  Set selected Offset polarity
;---------------------------------------------------------------------------

OFFSET_POL_SET  JNB     KEY_STATE,.DO_IT   Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
                XRL     MENU_INDEX,#%0000.0001  Invert index value
                MOV     DPTR,#.POL_TEXT Setup Polarity text table pointer
                MOV     A,MENU_INDEX    Get offset in table
                CALL    SHOW_TEXT       Show the text
.RET            RET
.DO_IT          MOV     A,MENU_INDEX    Get index and copy it to the
                MOV     C,ACC.0           polarity flag
                MOV     OFFSET_POL,C
                SETB    OFFSET_ACTIVE   Set offset active flag
                JB      SCALE_ACTIVE,.SKIP Don't change priority!
                CLR     FUNC_PRIORITY   Offset first if scaling not set yet
.SKIP           MOV     DPTR,#OFFSET_VALUE Copy Accu C to the offset value
                MOV     R0,#FP_ACCU_C
                CLR     C               No need to indicate scaling value!
                CALL    FP_ACCU_2_MEM
                JMP     MENU_DONE       Ready with menu

.POL_TEXT       .DA     #CHR_A,#CHR_D,#CHR_D,#BLANK,#BLANK,#BLANK,#BLANK
                .DA     #CHR_S,#CHR_U,#CHR_B,#BLANK,#BLANK,#BLANK,#BLANK

;---------------------------------------------------------------------------
;   ROUTINE    :  SCALE_MENU_WAIT
;   FUNCTION   :  Wait for SCALE setup keypress
;---------------------------------------------------------------------------

SCALE_MENU_WAIT MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                MOV     MENU_INDEX,#-1  Start showing the scaling values
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#SCALE_MENU_SET Point to propper function
                MOV     MENU_PNTR+1,/SCALE_MENU_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  SCALE_MENU_SET
;   FUNCTION   :  Show all possible scaling factors
;---------------------------------------------------------------------------

SCALE_MENU_SET  JNB     KEY_STATE,.DO_IT  Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
.SCALE_LOOP     CALL    INC_VALUE_INDEX Increment value index
                JNZ     .INDEX_VALUE    Index points to a value!
                MOV     DPTR,#.NONE_TEXT Show 'None' in the display
                CALL    SHOW_TEXT        Show the text
.RET            RET
.INDEX_VALUE    DEC     A               Multiply index with 5
                MOV     B,#5
                MUL     AB
                ADD     A,#VALUE_TABLE  Add value table address to offset
                MOV     DPL,A
                MOV     A,/VALUE_TABLE
                ADDC    A,#0
                MOV     DPH,A
                MOV     R0,#FP_ACCU_C   Point FP Accu C
                CALL    MEM_2_FP_ACCU
                JC      .SCALE_LOOP     It is not a Scaling value!
.VALUE_OK       CALL    SHOW_SCALE      Show scaling value in FP_ACCU_C
                RET

.DO_IT          MOV     A,MENU_INDEX    Get index, see if it's 0
                JZ      .NONE           Switch scaling factor off

                MOV     DPTR,#.POL_TEXT Show 'mul/div' in display
                CLR     A
                CALL    SHOW_TEXT
                MOV     TIME_DELAY,#MENU_WAIT_TIME  Setup delay time to
                MOV     TIME_DELAY+1,/MENU_WAIT_TIME  wait for keypress
                MOV     MENU_PNTR,#SCALE_POL_WAIT    Setup polarity wait
                MOV     MENU_PNTR+1,/SCALE_POL_WAIT   routine pointer
                RET

.NONE           CLR     SCALE_ACTIVE    Switch scaling mode off
                CLR     SCALE_POL
                JNB     OFFSET_ACTIVE,.DONE Don't care about priority!
                CLR     FUNC_PRIORITY   Offset function has priority now!
.DONE           JMP     MENU_DONE       Ready with menu

.NONE_TEXT      .DA     #CHR_N,#CHR_O,#CHR_N,#CHR_E,#BLANK,#BLANK,#BLANK
.POL_TEXT       .DA     #CHR_P,#CHR_O,#CHR_L,#CHR_A,#CHR_R,#CHR_I,#CHR_T+SDP

;---------------------------------------------------------------------------
;   ROUTINE    :  SCALE_POL_WAIT
;   FUNCTION   :  Wait for SCALE polarity switch to be pressed
;---------------------------------------------------------------------------

SCALE_POL_WAIT  MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                MOV     MENU_INDEX,#1   Start menu with MUL
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#SCALE_POL_SET  Point to propper function
                MOV     MENU_PNTR+1,/SCALE_POL_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  SCALE_POL_SET
;   FUNCTION   :  Set selected Scaling polarity
;---------------------------------------------------------------------------

SCALE_POL_SET   JNB     KEY_STATE,.DO_IT   Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
                XRL     MENU_INDEX,#%0000.0001  Invert index value
                MOV     DPTR,#.POL_TEXT Setup Polarity text table pointer
                MOV     A,MENU_INDEX    Get offset in table
                CALL    SHOW_TEXT       Show the text
.RET            RET
.DO_IT          MOV     A,MENU_INDEX    Get index and copy it to the
                MOV     C,ACC.0           polarity flag
                MOV     SCALE_POL,C
                SETB    SCALE_ACTIVE    Set scaling active flag
                JB      OFFSET_ACTIVE,.SKIP Don't change priority!
                SETB    FUNC_PRIORITY   Scaling first if offset not set yet
.SKIP           MOV     DPTR,#SCALE_VALUE Copy Accu C to the scale value
                MOV     R0,#FP_ACCU_C
                CLR     C               No need to indicate scaling value!
                CALL    FP_ACCU_2_MEM
                JMP     MENU_DONE       Ready with menu

.POL_TEXT       .DA     #CHR_M,#CHR_U,#CHR_L,#CHR_T,#CHR_I,#CHR_P+SDP,#BLANK
                .DA     #CHR_D,#CHR_I,#CHR_V,#CHR_I,#CHR_D,#CHR_E,#BLANK

;---------------------------------------------------------------------------
;   ROUTINE    :  INC_VALUE_INDEX
;   FUNCTION   :  Increment index in value table in RAM
;---------------------------------------------------------------------------

INC_VALUE_INDEX MOV     A,MENU_INDEX    Get menu index
                INC     A                and increment it
                >CMP    A,#50+1         See if index OK
                JC      .INDEX_OK       It is!
                CLR     A               Restart the loop
.INDEX_OK       MOV     MENU_INDEX,A    Save new index
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  SHOW_SCALE
;   FUNCTION   :  Show Scaling factor in Accu C
;---------------------------------------------------------------------------

SHOW_SCALE      MOV     R1,#DSP_BUFFER  Clear display first
                MOV     R0,#8
                CLR     A
.CLR_DISPLAY    MOV     @R1,A
                INC     R1
                DJNZ    R0,.CLR_DISPLAY Do all 7 digits + annunciators
                MOV     A,FP_EXP_C      Get the exponent
                INC     A
                MOV     R7,A
                MOV     R0,#FP_ACCU_C+2 Point to LSD of scaling factor
                CLR     F0              Point to 2nd BCD number in byte
                MOV     R2,#7           Set meaningful digits counter
.MEAN_DIG_LOOP  MOV     A,@R0           Get digit
                JNB     F0,.LOW_DIGIT1  Get low digit of byte
                SWAP    A               Get upper digit of byte
.LOW_DIGIT1     ANL     A,#%0000.1111   Throw away high part
                JNZ     .MEAN_DIGIT     This is a meaningful digit!
                CPL     F0              Complement digit flag
                JB      F0,.SKIP1       Skip incrementing pointer
                INC     R0              Increment digit pointer
.SKIP1          DJNZ    R2,.MEAN_DIG_LOOP Find 1st meaningful digit!
                INC     R2              Must be 1 digit at least!
.MEAN_DIGIT     MOV     R0,#FP_ACCU_C+5 Point to MSD
                CLR     F0              Point to 2nd BCD number in byte
                MOV     R1,#DSP_BUFFER  Point to display buffer

.SHOW_LOOP      MOV     A,@R0           Get digit
                JNB     F0,.LOW_DIGIT2  Use low digit
                SWAP    A
.LOW_DIGIT2     ANL     A,#%0000.1111   Use only low nibble
                CPL     F0              Complement digit flag
                JNB     F0,.SKIP2       Don't increment pointer in ACCU!
                DEC     R0              Decrement pointer in ACCU
.SKIP2          CALL    BCD_2_7SEGM     Convert digit to 7 segments
                MOV     @R1,A           Put it in display buffer
                DEC     R2              Decrement meaningful digit counter
                DEC     R7              Show decimal point here?
                MOV     A,R7
                JNZ     .NO_POINT       No!
                MOV     A,R2            No meaningful digits after DP?
                JZ      .DONE           No!
                JB      ACC.7,.DONE     No!
                MOV     A,@R1           Show decimal point
                SETB    ACC.7
                MOV     @R1,A
.NO_POINT       INC     R1              Increment display pointer
                MOV     A,R7            Is decimal point shown yet?
                JNB     ACC.7,.SHOW_LOOP Not yet! Go on!
                MOV     A,R2            More meaningful digits?
                JNZ     .SHOW_LOOP      Yes! Do them all
.DONE           CALL    UPDATE_DSP      Update display
                RET

;---------------------------------------------------------------------------
;   ROUTINE    :  SPEED_MENU_WAIT
;   FUNCTION   :  Wait for SPEED function setup keypress
;---------------------------------------------------------------------------

SPEED_MENU_WAIT MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                SETB    SPEED_FUNCTION  Set speed function for a start
                MOV     MENU_INDEX,#1   Start with CCIR norm
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#SPEED_MENU_SET Setup SPEED norm pointer
                MOV     MENU_PNTR+1,/SPEED_MENU_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  SPEED_MENU_SET
;   FUNCTION   :  Set selected Speed norm
;---------------------------------------------------------------------------

SPEED_MENU_SET  JNB     KEY_STATE,.DO_IT   Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
                XRL     MENU_INDEX,#%0000.0001  Invert index value
                MOV     DPTR,#.NORM_TEXT Setup Norm text table pointer
                MOV     A,MENU_INDEX    Get offset in table
                CALL    SHOW_TEXT       Show the text
.RET            RET
.DO_IT          MOV     A,MENU_INDEX    Get index and copy it to the
                MOV     C,ACC.0           reference source flag
                MOV     SPEED_REFERENCE,C
                JMP     MENU_DONE       Ready with menu

.NORM_TEXT      .DA     #CHR_C,#CHR_C,#CHR_I,#CHR_R,#BLANK,#BLANK,#BLANK
                .DA     #CHR_D,#CHR_I,#CHR_N,#BLANK,#BLANK,#BLANK,#BLANK

;---------------------------------------------------------------------------
;   ROUTINE    :  REF_MENU_WAIT
;   FUNCTION   :  Wait for REFERENCE source setup keypress
;---------------------------------------------------------------------------

REF_MENU_WAIT   MOV     A,TIME_DELAY    Timer still running?
                ORL     A,TIME_DELAY+1
                JNZ     .CHECK_KEY      Yes! Key still valid
                CLR     MENU_MODE       Quit menu mode
                JMP     RETRY_MEASURE   Resume counting
.CHECK_KEY      JNB     KEY_STATE,.RET  Key not pressed!
                CLR     A               Get old reference source
                MOV     C,REF_SOURCE    Will be inverted immediatly
                MOV     ACC.0,C
                MOV     MENU_INDEX,A
                MOV     TIME_DELAY,#0   Set delay to minimum
                MOV     TIME_DELAY+1,#0
                MOV     MENU_PNTR,#REF_MENU_SET Setup REF source pointer
                MOV     MENU_PNTR+1,/REF_MENU_SET
.RET            RET

;---------------------------------------------------------------------------
;   ROUTINE    :  REF_MENU_SET
;   FUNCTION   :  Key for REF source menu is pressed
;---------------------------------------------------------------------------

REF_MENU_SET    JNB     KEY_STATE,.DO_IT   Key released! Use this index
                MOV     A,TIME_DELAY    See if timer still running
                ORL     A,TIME_DELAY+1
                JNZ     .RET            Yes! Don't do anything
                MOV     TIME_DELAY,#MENU_SPEED  Setup menu time speed
                MOV     TIME_DELAY+1,/MENU_SPEED
                XRL     MENU_INDEX,#%0000.0001  Invert index value
                MOV     DPTR,#.REF_TEXT  Setup REF text table pointer
                MOV     A,MENU_INDEX    Get offset in table
                CALL    SHOW_TEXT       Show the text
.RET            RET
.DO_IT          MOV     A,MENU_INDEX    Get index and copy it to the
                MOV     C,ACC.0           reference source flag
                MOV     REF_SOURCE,C
                CPL     C               Set output port
                MOV     P3.2,C
                JMP     MENU_DONE       Ready with menu

.REF_TEXT       .DA     #CHR_I,#CHR_N,#CHR_T+SDP,#BLANK,#BLANK,#BLANK,#BLANK
                .DA     #CHR_E,#CHR_X,#CHR_T+SDP,#BLANK,#BLANK,#BLANK,#BLANK
